@@ -1,47 +1,11 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Mozilla Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Marco Bonardo <mak77@bonardo.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /**
  * Tests the bookmarks Properties dialog.
  */
 
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-
 // DOM ids of Places sidebar trees.
 const SIDEBAR_HISTORY_TREE_ID = "historyTree";
 const SIDEBAR_BOOKMARKS_TREE_ID = "bookmarks-view";
@@ -65,7 +29,7 @@ const ACTION_ADD = 1;
 const TYPE_FOLDER = 0;
 const TYPE_BOOKMARK = 1;
 
-const TEST_URL = "http://www.mozilla.org/";
+const TEST_URL = "http://www.example.com/";
 
 const DIALOG_URL = "chrome://browser/content/places/bookmarkProperties.xul";
 const DIALOG_URL_MINIMAL_UI = "chrome://browser/content/places/bookmarkProperties2.xul";
@@ -76,17 +40,6 @@ var win = wm.getMostRecentWindow("navigator:browser");
 var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].
          getService(Ci.nsIWindowWatcher);
 
-function add_visit(aURI, aDate) {
-  var visitId = PlacesUtils.history
-                           .addVisit(aURI,
-                                     aDate,
-                                     null, // no referrer
-                                     PlacesUtils.history.TRANSITION_TYPED,
-                                     false, // not redirect
-                                     0);
-  return visitId;
-}
-
 function add_bookmark(aURI) {
   var bId = PlacesUtils.bookmarks
                        .insertBookmark(PlacesUtils.unfiledBookmarksFolderId,
@@ -111,8 +64,9 @@ gTests.push({
   historyView: SIDEBAR_HISTORY_BYLASTVISITED_VIEW, // See constants above, only for History sidebar.
   window: null, // Will contain handle of dialog window
 
-  setup: function() {
+  setup: function(aCallback) {
     // Setup everything needed for this test, runs before everything else.
+    aCallback();
   },
 
   selectNode: function(tree) {
@@ -146,8 +100,9 @@ gTests.push({
   itemType: null,
   window: null,
 
-  setup: function() {
+  setup: function(aCallback) {
     // Nothing to do.
+    aCallback();
   },
 
   selectNode: function(tree) {
@@ -203,19 +158,23 @@ gTests.push({
   itemType: null,
   window: null,
   _itemId: null,
+  _cleanShutdown: false,
 
-  setup: function() {
+  setup: function(aCallback) {
     // Add a bookmark in unsorted bookmarks folder.
     this._itemId = add_bookmark(PlacesUtils._uri(TEST_URL));
     ok(this._itemId > 0, "Correctly added a bookmark");
     // Add a tag to this bookmark.
     PlacesUtils.tagging.tagURI(PlacesUtils._uri(TEST_URL),
                                ["testTag"]);
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Correctly added a tag");
+    aCallback();
   },
 
   selectNode: function(tree) {
+    tree.selectItems([PlacesUtils.unfiledBookmarksFolderId]);
+    PlacesUtils.asContainer(tree.selectedNode).containerOpen = true;
     tree.selectItems([this._itemId]);
     is(tree.selectedNode.itemId, this._itemId, "Bookmark has been selected");
   },
@@ -225,33 +184,27 @@ gTests.push({
     var tagsField = this.window.document.getElementById("editBMPanel_tagsField");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed" &&
-            aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL) {
-          ww.unregisterNotification(this);
-          tagsField.popup.removeEventListener("popuphidden", popupListener, true);
-          ok(false, "Dialog window should not be closed by pressing Enter on the autocomplete popup");
-          self.finish();
-        }
-      }
-    };
+    this.window.addEventListener("unload", function(event) {
+      self.window.removeEventListener("unload", arguments.callee, true);
+      tagsField.popup.removeEventListener("popuphidden", popupListener, true);
+      ok(self._cleanShutdown, "Dialog window should not be closed by pressing Enter on the autocomplete popup");
+      executeSoon(function () {
+        self.finish();
+      });
+    }, true);
 
     var popupListener = {
       handleEvent: function(aEvent) {
         switch (aEvent.type) {
           case "popuphidden":
             // Everything worked fine, we can stop observing the window.
-            ww.unregisterNotification(windowObserver);
-            tagsField.popup.removeEventListener("popuphidden", this, true);
+            self._cleanShutdown = true;
             self.window.document.documentElement.cancelDialog();
-            self.finish();
             break;
           case "popupshown":
             tagsField.popup.removeEventListener("popupshown", this, true);
-            // In case this test fails the window will close, we should mark the
-            // failure and continue, to avoid timing out.
-            ww.registerNotification(windowObserver);
+            // In case this test fails the window will close, the test will fail
+            // since we didn't set _cleanShutdown.
             var tree = tagsField.popup.tree;
             // Focus and select first result.
             isnot(tree, null, "Autocomplete results tree exists");
@@ -286,7 +239,7 @@ gTests.push({
 
   cleanup: function() {
     // Check tags have not changed.
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Tag on node has not changed");
 
     // Cleanup.
@@ -306,8 +259,9 @@ gTests.push({
   window: null,
   _itemId: null,
 
-  setup: function() {
+  setup: function(aCallback) {
     // Nothing to do.
+    aCallback();
   },
 
   selectNode: function(tree) {
@@ -321,16 +275,16 @@ gTests.push({
     this._itemId = this.window.gEditItemOverlay._itemId;
     // Change folder name
     var namePicker = this.window.document.getElementById("editBMPanel_namePicker");
-    var userEnteredName = this.window.document.getElementById("editBMPanel_userEnteredName");
     var self = this;
+
     this.window.addEventListener("unload", function(event) {
-        this.window.removeEventListener("unload", arguments.callee, false);
-        executeSoon(function() {
-          self.finish();
-        });
-      }, false);
+      self.window.removeEventListener("unload", arguments.callee, false);
+      executeSoon(function () {
+        self.finish();
+      });
+    }, false);
+
     namePicker.value = "n";
-    userEnteredName.label = "n";
     info("About to focus the namePicker field");
     namePicker.focus();
     EventUtils.synthesizeKey("VK_RETURN", {}, this.window);
@@ -362,19 +316,23 @@ gTests.push({
   itemType: null,
   window: null,
   _itemId: null,
+  _cleanShutdown: false,
 
-  setup: function() {
+  setup: function(aCallback) {
     // Add a bookmark in unsorted bookmarks folder.
     this._itemId = add_bookmark(PlacesUtils._uri(TEST_URL));
     ok(this._itemId > 0, "Correctly added a bookmark");
     // Add a tag to this bookmark.
     PlacesUtils.tagging.tagURI(PlacesUtils._uri(TEST_URL),
                                ["testTag"]);
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Correctly added a tag");
+    aCallback();
   },
 
   selectNode: function(tree) {
+    tree.selectItems([PlacesUtils.unfiledBookmarksFolderId]);
+    PlacesUtils.asContainer(tree.selectedNode).containerOpen = true;
     tree.selectItems([this._itemId]);
     is(tree.selectedNode.itemId, this._itemId, "Bookmark has been selected");
   },
@@ -384,33 +342,27 @@ gTests.push({
     var tagsField = this.window.document.getElementById("editBMPanel_tagsField");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed" &&
-            aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL) {
-          ww.unregisterNotification(this);
-          tagsField.popup.removeEventListener("popuphidden", popupListener, true);
-          ok(false, "Dialog window should not be closed by pressing Escape on the autocomplete popup");
-          self.finish();
-        }
-      }
-    };
+    this.window.addEventListener("unload", function(event) {
+      self.window.removeEventListener("unload", arguments.callee, true);
+      tagsField.popup.removeEventListener("popuphidden", popupListener, true);
+      ok(self._cleanShutdown, "Dialog window should not be closed by pressing Escape on the autocomplete popup");
+      executeSoon(function () {
+        self.finish();
+      });
+    }, true);
 
     var popupListener = {
       handleEvent: function(aEvent) {
         switch (aEvent.type) {
           case "popuphidden":
-            // Everything worked fine, we can stop observing the window.
-            ww.unregisterNotification(windowObserver);
-            tagsField.popup.removeEventListener("popuphidden", this, true);
+            // Everything worked fine.
+            self._cleanShutdown = true;
             self.window.document.documentElement.cancelDialog();
-            self.finish();
             break;
           case "popupshown":
             tagsField.popup.removeEventListener("popupshown", this, true);
-            // In case this test fails the window will close, we should mark the
-            // failure and continue, to avoid timing out.
-            ww.registerNotification(windowObserver);
+            // In case this test fails the window will close, the test will fail
+            // since we didn't set _cleanShutdown.
             var tree = tagsField.popup.tree;
             // Focus and select first result.
             isnot(tree, null, "Autocomplete results tree exists");
@@ -445,7 +397,7 @@ gTests.push({
 
   cleanup: function() {
     // Check tags have not changed.
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Tag on node has not changed");
 
     // Cleanup.
@@ -465,12 +417,13 @@ gTests.push({
   historyView: SIDEBAR_HISTORY_BYLASTVISITED_VIEW,
   window: null,
 
-  setup: function() {
+  setup: function(aCallback) {
     // Add a visit.
-    add_visit(PlacesUtils._uri(TEST_URL), Date.now() * 1000);
-    // Sanity check.
-    var gh = PlacesUtils.history.QueryInterface(Ci.nsIGlobalHistory2);
-    ok(gh.isVisited(PlacesUtils._uri(TEST_URL)), TEST_URL + " is a visited url.");
+    addVisits(
+      {uri: PlacesUtils._uri(TEST_URL),
+        transition: PlacesUtils.history.TRANSITION_TYPED},
+      window,
+      aCallback);
   },
 
   selectNode: function(tree) {
@@ -486,18 +439,13 @@ gTests.push({
     var folderTree = this.window.document.getElementById("editBMPanel_folderTree");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed" &&
-            aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL_MINIMAL_UI) {
-          ww.unregisterNotification(this);
-          ok(self._cleanShutdown,
-             "Dialog window should not be closed by pressing ESC in folder name textbox");
-          self.finish();
-        }
-      }
-    };
-    ww.registerNotification(windowObserver);
+    this.window.addEventListener("unload", function(event) {
+      self.window.removeEventListener("unload", arguments.callee, true);
+      ok(self._cleanShutdown, "Dialog window should not be closed by pressing ESC in folder name textbox");
+      executeSoon(function () {
+        self.finish();
+      });
+    }, true);
 
     folderTree.addEventListener("DOMAttrModified", function onDOMAttrModified(event) {
       if (event.attrName != "place")
@@ -536,6 +484,10 @@ gTests.push({
 
 function test() {
   waitForExplicitFinish();
+  // This test can take some time, if we timeout too early it could run
+  // in the middle of other tests, or hang them.
+  requestLongerTimeout(2);
+
   // Sanity checks.
   ok(PlacesUtils, "PlacesUtils in context");
   ok(PlacesUIUtils, "PlacesUIUtils in context");
@@ -550,7 +502,7 @@ function runNextTest() {
     gCurrentTest.cleanup();
     info("End of test: " + gCurrentTest.desc);
     gCurrentTest = null;
-    executeSoon(runNextTest);
+    waitForAsyncUpdates(runNextTest);
     return;
   }
 
@@ -558,8 +510,9 @@ function runNextTest() {
     // Goto next tests.
     gCurrentTest = gTests.shift();
     info("Start of test: " + gCurrentTest.desc);
-    gCurrentTest.setup();
-    execute_test_in_sidebar();
+    gCurrentTest.setup(function() {
+      execute_test_in_sidebar();
+    });
   }
   else {
     // Finished all tests.
@@ -601,32 +554,30 @@ function open_properties_dialog() {
        "We have a places node selected: " + tree.selectedNode.title);
 
     // Wait for the Properties dialog.
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowopened") {
-          ww.unregisterNotification(this);
-          var win = aSubject.QueryInterface(Ci.nsIDOMWindow);
-          win.addEventListener("focus", function(event) {
-            win.removeEventListener("focus", arguments.callee, false);
-            // Windows has been loaded, execute our test now.
-            executeSoon(function () {
-              // Ensure overlay is loaded
-              ok(win.gEditItemOverlay._initialized, "EditItemOverlay is initialized");
-              gCurrentTest.window = win;
-              try {
-                gCurrentTest.run();
-              } catch (ex) {
-                ok(false, "An error occured during test run: " + ex.message);
-              }
-            });
-          }, false);
-        }
-      }
-    };
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic != "domwindowopened")
+        return;
+      ww.unregisterNotification(windowObserver);
+      var win = aSubject.QueryInterface(Ci.nsIDOMWindow);
+      win.addEventListener("focus", function (event) {
+        win.removeEventListener("focus", arguments.callee, false);
+        // Windows has been loaded, execute our test now.
+        executeSoon(function () {
+          // Ensure overlay is loaded
+          ok(win.gEditItemOverlay._initialized, "EditItemOverlay is initialized");
+          gCurrentTest.window = win;
+          try {
+            gCurrentTest.run();
+          } catch (ex) {
+            ok(false, "An error occured during test run: " + ex.message);
+          }
+        });
+      }, false);
+    }
     ww.registerNotification(windowObserver);
 
     var command = null;
-    switch(gCurrentTest.action) {
+    switch (gCurrentTest.action) {
       case ACTION_EDIT:
         command = "placesCmd_show:info";
         break;
