@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *         Robert O'Callahan <robert@ocallahan.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsLineBreaker.h"
 #include "nsContentUtils.h"
@@ -42,12 +9,14 @@
 #include "gfxFont.h" // for the gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_* values
 #include "nsHyphenationManager.h"
 #include "nsHyphenator.h"
+#include "mozilla/gfx/2D.h"
 
 nsLineBreaker::nsLineBreaker()
-  : mCurrentWordLangGroup(nsnull),
-    mCurrentWordContainsMixedLang(PR_FALSE),
-    mCurrentWordContainsComplexChar(PR_FALSE),
-    mAfterBreakableSpace(PR_FALSE), mBreakHere(PR_FALSE)
+  : mCurrentWordLanguage(nullptr),
+    mCurrentWordContainsMixedLang(false),
+    mCurrentWordContainsComplexChar(false),
+    mAfterBreakableSpace(false), mBreakHere(false),
+    mWordBreak(nsILineBreaker::kWordBreak_Normal)
 {
 }
 
@@ -57,20 +26,30 @@ nsLineBreaker::~nsLineBreaker()
 }
 
 static void
-SetupCapitalization(const PRUnichar* aWord, PRUint32 aLength,
-                    PRPackedBool* aCapitalization)
+SetupCapitalization(const char16_t* aWord, uint32_t aLength,
+                    bool* aCapitalization)
 {
-  // Capitalize the first non-punctuation character after a space or start
+  // Capitalize the first alphanumeric character after a space or start
   // of the word.
   // The only space character a word can contain is NBSP.
-  PRBool capitalizeNextChar = PR_TRUE;
-  for (PRUint32 i = 0; i < aLength; ++i) {
-    if (capitalizeNextChar && !nsContentUtils::IsPunctuationMark(aWord[i])) {
-      aCapitalization[i] = PR_TRUE;
-      capitalizeNextChar = PR_FALSE;
+  bool capitalizeNextChar = true;
+  for (uint32_t i = 0; i < aLength; ++i) {
+    uint32_t ch = aWord[i];
+    if (capitalizeNextChar) {
+      if (NS_IS_HIGH_SURROGATE(ch) && i + 1 < aLength &&
+          NS_IS_LOW_SURROGATE(aWord[i + 1])) {
+        ch = SURROGATE_TO_UCS4(ch, aWord[i + 1]);
+      }
+      if (nsContentUtils::IsAlphanumeric(ch)) {
+        aCapitalization[i] = true;
+        capitalizeNextChar = false;
+      }
+      if (!IS_IN_BMP(ch)) {
+        ++i;
+      }
     }
-    if (aWord[i] == 0xA0 /*NBSP*/) {
-      capitalizeNextChar = PR_TRUE;
+    if (ch == 0xA0 /*NBSP*/) {
+      capitalizeNextChar = true;
     }
   }
 }
@@ -78,35 +57,39 @@ SetupCapitalization(const PRUnichar* aWord, PRUint32 aLength,
 nsresult
 nsLineBreaker::FlushCurrentWord()
 {
-  PRUint32 length = mCurrentWord.Length();
-  nsAutoTArray<PRUint8,4000> breakState;
+  uint32_t length = mCurrentWord.Length();
+  nsAutoTArray<uint8_t,4000> breakState;
   if (!breakState.AppendElements(length))
     return NS_ERROR_OUT_OF_MEMORY;
   
-  nsTArray<PRPackedBool> capitalizationState;
+  nsTArray<bool> capitalizationState;
 
   if (!mCurrentWordContainsComplexChar) {
-    // Just set everything internal to "no break"!
+    // For break-strict set everything internal to "break", otherwise
+    // to "no break"!
     memset(breakState.Elements(),
-           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE,
-           length*sizeof(PRUint8));
+           mWordBreak == nsILineBreaker::kWordBreak_BreakAll ?
+             gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NORMAL :
+             gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE,
+           length*sizeof(uint8_t));
   } else {
     nsContentUtils::LineBreaker()->
-      GetJISx4051Breaks(mCurrentWord.Elements(), length, breakState.Elements());
+      GetJISx4051Breaks(mCurrentWord.Elements(), length, mWordBreak,
+                        breakState.Elements());
   }
 
-  PRBool autoHyphenate = mCurrentWordLangGroup &&
+  bool autoHyphenate = mCurrentWordLanguage &&
     !mCurrentWordContainsMixedLang;
-  PRUint32 i;
+  uint32_t i;
   for (i = 0; autoHyphenate && i < mTextItems.Length(); ++i) {
     TextItem* ti = &mTextItems[i];
     if (!(ti->mFlags & BREAK_USE_AUTO_HYPHENATION)) {
-      autoHyphenate = PR_FALSE;
+      autoHyphenate = false;
     }
   }
   if (autoHyphenate) {
     nsRefPtr<nsHyphenator> hyphenator =
-      nsHyphenationManager::Instance()->GetHyphenator(mCurrentWordLangGroup);
+      nsHyphenationManager::Instance()->GetHyphenator(mCurrentWordLanguage);
     if (hyphenator) {
       FindHyphenationPoints(hyphenator,
                             mCurrentWord.Elements(),
@@ -115,7 +98,7 @@ nsLineBreaker::FlushCurrentWord()
     }
   }
 
-  PRUint32 offset = 0;
+  uint32_t offset = 0;
   for (i = 0; i < mTextItems.Length(); ++i) {
     TextItem* ti = &mTextItems[i];
     NS_ASSERTION(ti->mLength > 0, "Zero length word contribution?");
@@ -124,16 +107,16 @@ nsLineBreaker::FlushCurrentWord()
       breakState[offset] = gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE;
     }
     if (ti->mFlags & BREAK_SUPPRESS_INSIDE) {
-      PRUint32 exclude = ti->mSinkOffset == 0 ? 1 : 0;
+      uint32_t exclude = ti->mSinkOffset == 0 ? 1 : 0;
       memset(breakState.Elements() + offset + exclude,
              gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE,
-             (ti->mLength - exclude)*sizeof(PRUint8));
+             (ti->mLength - exclude)*sizeof(uint8_t));
     }
 
     // Don't set the break state for the first character of the word, because
     // it was already set correctly earlier and we don't know what the true
     // value should be.
-    PRUint32 skipSet = i == 0 ? 1 : 0;
+    uint32_t skipSet = i == 0 ? 1 : 0;
     if (ti->mSink) {
       ti->mSink->SetBreaks(ti->mSinkOffset + skipSet, ti->mLength - skipSet,
                            breakState.Elements() + offset + skipSet);
@@ -142,7 +125,7 @@ nsLineBreaker::FlushCurrentWord()
         if (capitalizationState.Length() == 0) {
           if (!capitalizationState.AppendElements(length))
             return NS_ERROR_OUT_OF_MEMORY;
-          memset(capitalizationState.Elements(), PR_FALSE, length*sizeof(PRPackedBool));
+          memset(capitalizationState.Elements(), false, length*sizeof(bool));
           SetupCapitalization(mCurrentWord.Elements(), length,
                               capitalizationState.Elements());
         }
@@ -156,19 +139,19 @@ nsLineBreaker::FlushCurrentWord()
 
   mCurrentWord.Clear();
   mTextItems.Clear();
-  mCurrentWordContainsComplexChar = PR_FALSE;
-  mCurrentWordContainsMixedLang = PR_FALSE;
-  mCurrentWordLangGroup = nsnull;
+  mCurrentWordContainsComplexChar = false;
+  mCurrentWordContainsMixedLang = false;
+  mCurrentWordLanguage = nullptr;
   return NS_OK;
 }
 
 nsresult
-nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32 aLength,
-                          PRUint32 aFlags, nsILineBreakSink* aSink)
+nsLineBreaker::AppendText(nsIAtom* aHyphenationLanguage, const char16_t* aText, uint32_t aLength,
+                          uint32_t aFlags, nsILineBreakSink* aSink)
 {
   NS_ASSERTION(aLength > 0, "Appending empty text...");
 
-  PRUint32 offset = 0;
+  uint32_t offset = 0;
 
   // Continue the current word
   if (mCurrentWord.Length() > 0) {
@@ -177,9 +160,9 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
     while (offset < aLength && !IsSpace(aText[offset])) {
       mCurrentWord.AppendElement(aText[offset]);
       if (!mCurrentWordContainsComplexChar && IsComplexChar(aText[offset])) {
-        mCurrentWordContainsComplexChar = PR_TRUE;
+        mCurrentWordContainsComplexChar = true;
       }
-      UpdateCurrentWordLangGroup(aLangGroup);
+      UpdateCurrentWordLanguage(aHyphenationLanguage);
       ++offset;
     }
 
@@ -196,21 +179,21 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
       return rv;
   }
 
-  nsAutoTArray<PRUint8,4000> breakState;
+  nsAutoTArray<uint8_t,4000> breakState;
   if (aSink) {
     if (!breakState.AppendElements(aLength))
       return NS_ERROR_OUT_OF_MEMORY;
   }
   
-  nsTArray<PRPackedBool> capitalizationState;
+  nsTArray<bool> capitalizationState;
   if (aSink && (aFlags & BREAK_NEED_CAPITALIZATION)) {
     if (!capitalizationState.AppendElements(aLength))
       return NS_ERROR_OUT_OF_MEMORY;
-    memset(capitalizationState.Elements(), PR_FALSE, aLength);
+    memset(capitalizationState.Elements(), false, aLength*sizeof(bool));
   }
 
-  PRUint32 start = offset;
-  PRBool noBreaksNeeded = !aSink ||
+  uint32_t start = offset;
+  bool noBreaksNeeded = !aSink ||
     (aFlags == (BREAK_SUPPRESS_INITIAL | BREAK_SUPPRESS_INSIDE | BREAK_SKIP_SETTING_NO_BREAKS) &&
      !mBreakHere && !mAfterBreakableSpace);
   if (noBreaksNeeded) {
@@ -225,26 +208,29 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
         break;
     }
   }
-  PRUint32 wordStart = offset;
-  PRBool wordHasComplexChar = PR_FALSE;
+  uint32_t wordStart = offset;
+  bool wordHasComplexChar = false;
 
   nsRefPtr<nsHyphenator> hyphenator;
-  if ((aFlags & BREAK_USE_AUTO_HYPHENATION) && !(aFlags & BREAK_SUPPRESS_INSIDE)) {
-    hyphenator = nsHyphenationManager::Instance()->GetHyphenator(aLangGroup);
+  if ((aFlags & BREAK_USE_AUTO_HYPHENATION) &&
+      !(aFlags & BREAK_SUPPRESS_INSIDE) &&
+      aHyphenationLanguage) {
+    hyphenator = nsHyphenationManager::Instance()->GetHyphenator(aHyphenationLanguage);
   }
 
   for (;;) {
-    PRUnichar ch = aText[offset];
-    PRBool isSpace = IsSpace(ch);
-    PRBool isBreakableSpace = isSpace && !(aFlags & BREAK_SUPPRESS_INSIDE);
+    char16_t ch = aText[offset];
+    bool isSpace = IsSpace(ch);
+    bool isBreakableSpace = isSpace && !(aFlags & BREAK_SUPPRESS_INSIDE);
 
     if (aSink) {
       breakState[offset] =
-        mBreakHere || (mAfterBreakableSpace && !isBreakableSpace) ?
+        mBreakHere || (mAfterBreakableSpace && !isBreakableSpace) ||
+        (mWordBreak == nsILineBreaker::kWordBreak_BreakAll)  ?
           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NORMAL :
           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE;
     }
-    mBreakHere = PR_FALSE;
+    mBreakHere = false;
     mAfterBreakableSpace = isBreakableSpace;
 
     if (isSpace) {
@@ -253,9 +239,10 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
           if (wordHasComplexChar) {
             // Save current start-of-word state because GetJISx4051Breaks will
             // set it to false
-            PRUint8 currentStart = breakState[wordStart];
+            uint8_t currentStart = breakState[wordStart];
             nsContentUtils::LineBreaker()->
               GetJISx4051Breaks(aText + wordStart, offset - wordStart,
+                                mWordBreak,
                                 breakState.Elements() + wordStart);
             breakState[wordStart] = currentStart;
           }
@@ -270,28 +257,28 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
                               capitalizationState.Elements() + wordStart);
         }
       }
-      wordHasComplexChar = PR_FALSE;
+      wordHasComplexChar = false;
       ++offset;
       if (offset >= aLength)
         break;
       wordStart = offset;
     } else {
       if (!wordHasComplexChar && IsComplexChar(ch)) {
-        wordHasComplexChar = PR_TRUE;
+        wordHasComplexChar = true;
       }
       ++offset;
       if (offset >= aLength) {
         // Save this word
         mCurrentWordContainsComplexChar = wordHasComplexChar;
-        PRUint32 len = offset - wordStart;
-        PRUnichar* elems = mCurrentWord.AppendElements(len);
+        uint32_t len = offset - wordStart;
+        char16_t* elems = mCurrentWord.AppendElements(len);
         if (!elems)
           return NS_ERROR_OUT_OF_MEMORY;
-        memcpy(elems, aText + wordStart, sizeof(PRUnichar)*len);
+        memcpy(elems, aText + wordStart, sizeof(char16_t)*len);
         mTextItems.AppendElement(TextItem(aSink, wordStart, len, aFlags));
         // Ensure that the break-before for this word is written out
         offset = wordStart + 1;
-        UpdateCurrentWordLangGroup(aLangGroup);
+        UpdateCurrentWordLanguage(aHyphenationLanguage);
         break;
       }
     }
@@ -310,14 +297,14 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUnichar* aText, PRUint32
 
 void
 nsLineBreaker::FindHyphenationPoints(nsHyphenator *aHyphenator,
-                                     const PRUnichar *aTextStart,
-                                     const PRUnichar *aTextLimit,
-                                     PRPackedBool *aBreakState)
+                                     const char16_t *aTextStart,
+                                     const char16_t *aTextLimit,
+                                     uint8_t *aBreakState)
 {
   nsDependentSubstring string(aTextStart, aTextLimit);
-  nsAutoTArray<PRPackedBool,200> hyphens;
+  AutoFallibleTArray<bool,200> hyphens;
   if (NS_SUCCEEDED(aHyphenator->Hyphenate(string, hyphens))) {
-    for (PRUint32 i = 0; i + 1 < string.Length(); ++i) {
+    for (uint32_t i = 0; i + 1 < string.Length(); ++i) {
       if (hyphens[i]) {
         aBreakState[i + 1] =
           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_HYPHEN;
@@ -327,8 +314,8 @@ nsLineBreaker::FindHyphenationPoints(nsHyphenator *aHyphenator,
 }
 
 nsresult
-nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aLength,
-                          PRUint32 aFlags, nsILineBreakSink* aSink)
+nsLineBreaker::AppendText(nsIAtom* aHyphenationLanguage, const uint8_t* aText, uint32_t aLength,
+                          uint32_t aFlags, nsILineBreakSink* aSink)
 {
   NS_ASSERTION(aLength > 0, "Appending empty text...");
 
@@ -337,10 +324,10 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
     nsAutoString str;
     const char* cp = reinterpret_cast<const char*>(aText);
     CopyASCIItoUTF16(nsDependentCSubstring(cp, cp + aLength), str);
-    return AppendText(aLangGroup, str.get(), aLength, aFlags, aSink);
+    return AppendText(aHyphenationLanguage, str.get(), aLength, aFlags, aSink);
   }
 
-  PRUint32 offset = 0;
+  uint32_t offset = 0;
 
   // Continue the current word
   if (mCurrentWord.Length() > 0) {
@@ -350,7 +337,7 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
       mCurrentWord.AppendElement(aText[offset]);
       if (!mCurrentWordContainsComplexChar &&
           IsComplexASCIIChar(aText[offset])) {
-        mCurrentWordContainsComplexChar = PR_TRUE;
+        mCurrentWordContainsComplexChar = true;
       }
       ++offset;
     }
@@ -370,14 +357,14 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
       return rv;
   }
 
-  nsAutoTArray<PRUint8,4000> breakState;
+  nsAutoTArray<uint8_t,4000> breakState;
   if (aSink) {
     if (!breakState.AppendElements(aLength))
       return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  PRUint32 start = offset;
-  PRBool noBreaksNeeded = !aSink ||
+  uint32_t start = offset;
+  bool noBreaksNeeded = !aSink ||
     (aFlags == (BREAK_SUPPRESS_INITIAL | BREAK_SUPPRESS_INSIDE | BREAK_SKIP_SETTING_NO_BREAKS) &&
      !mBreakHere && !mAfterBreakableSpace);
   if (noBreaksNeeded) {
@@ -392,21 +379,24 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
         break;
     }
   }
-  PRUint32 wordStart = offset;
-  PRBool wordHasComplexChar = PR_FALSE;
+  uint32_t wordStart = offset;
+  bool wordHasComplexChar = false;
 
   for (;;) {
-    PRUint8 ch = aText[offset];
-    PRBool isSpace = IsSpace(ch);
-    PRBool isBreakableSpace = isSpace && !(aFlags & BREAK_SUPPRESS_INSIDE);
+    uint8_t ch = aText[offset];
+    bool isSpace = IsSpace(ch);
+    bool isBreakableSpace = isSpace && !(aFlags & BREAK_SUPPRESS_INSIDE);
 
     if (aSink) {
+      // Consider word-break style.  Since the break position of CJK scripts
+      // will be set by nsILineBreaker, we don't consider CJK at this point.
       breakState[offset] =
-        mBreakHere || (mAfterBreakableSpace && !isBreakableSpace) ?
+        mBreakHere || (mAfterBreakableSpace && !isBreakableSpace) ||
+        (mWordBreak == nsILineBreaker::kWordBreak_BreakAll) ?
           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NORMAL :
           gfxTextRun::CompressedGlyph::FLAG_BREAK_TYPE_NONE;
     }
-    mBreakHere = PR_FALSE;
+    mBreakHere = false;
     mAfterBreakableSpace = isBreakableSpace;
 
     if (isSpace) {
@@ -414,13 +404,14 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
         if (aSink && !(aFlags & BREAK_SUPPRESS_INSIDE)) {
           // Save current start-of-word state because GetJISx4051Breaks will
           // set it to false
-          PRPackedBool currentStart = breakState[wordStart];
+          uint8_t currentStart = breakState[wordStart];
           nsContentUtils::LineBreaker()->
             GetJISx4051Breaks(aText + wordStart, offset - wordStart,
+                              mWordBreak,
                               breakState.Elements() + wordStart);
           breakState[wordStart] = currentStart;
         }
-        wordHasComplexChar = PR_FALSE;
+        wordHasComplexChar = false;
       }
 
       ++offset;
@@ -429,17 +420,17 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
       wordStart = offset;
     } else {
       if (!wordHasComplexChar && IsComplexASCIIChar(ch)) {
-        wordHasComplexChar = PR_TRUE;
+        wordHasComplexChar = true;
       }
       ++offset;
       if (offset >= aLength) {
         // Save this word
         mCurrentWordContainsComplexChar = wordHasComplexChar;
-        PRUint32 len = offset - wordStart;
-        PRUnichar* elems = mCurrentWord.AppendElements(len);
+        uint32_t len = offset - wordStart;
+        char16_t* elems = mCurrentWord.AppendElements(len);
         if (!elems)
           return NS_ERROR_OUT_OF_MEMORY;
-        PRUint32 i;
+        uint32_t i;
         for (i = wordStart; i < offset; ++i) {
           elems[i - wordStart] = aText[i];
         }
@@ -458,39 +449,39 @@ nsLineBreaker::AppendText(nsIAtom* aLangGroup, const PRUint8* aText, PRUint32 aL
 }
 
 void
-nsLineBreaker::UpdateCurrentWordLangGroup(nsIAtom *aLangGroup)
+nsLineBreaker::UpdateCurrentWordLanguage(nsIAtom *aHyphenationLanguage)
 {
-  if (mCurrentWordLangGroup && mCurrentWordLangGroup != aLangGroup) {
-    mCurrentWordContainsMixedLang = PR_TRUE;
+  if (mCurrentWordLanguage && mCurrentWordLanguage != aHyphenationLanguage) {
+    mCurrentWordContainsMixedLang = true;
   } else {
-    mCurrentWordLangGroup = aLangGroup;
+    mCurrentWordLanguage = aHyphenationLanguage;
   }
 }
 
 nsresult
-nsLineBreaker::AppendInvisibleWhitespace(PRUint32 aFlags)
+nsLineBreaker::AppendInvisibleWhitespace(uint32_t aFlags)
 {
   nsresult rv = FlushCurrentWord();
   if (NS_FAILED(rv))
     return rv;
 
-  PRBool isBreakableSpace = !(aFlags & BREAK_SUPPRESS_INSIDE);
+  bool isBreakableSpace = !(aFlags & BREAK_SUPPRESS_INSIDE);
   if (mAfterBreakableSpace && !isBreakableSpace) {
-    mBreakHere = PR_TRUE;
+    mBreakHere = true;
   }
   mAfterBreakableSpace = isBreakableSpace;
   return NS_OK;
 }
 
 nsresult
-nsLineBreaker::Reset(PRBool* aTrailingBreak)
+nsLineBreaker::Reset(bool* aTrailingBreak)
 {
   nsresult rv = FlushCurrentWord();
   if (NS_FAILED(rv))
     return rv;
 
   *aTrailingBreak = mBreakHere || mAfterBreakableSpace;
-  mBreakHere = PR_FALSE;
-  mAfterBreakableSpace = PR_FALSE;
+  mBreakHere = false;
+  mAfterBreakableSpace = false;
   return NS_OK;
 }
