@@ -1,39 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifdef MOZ_WIDGET_QT
 #include <QString>
@@ -45,20 +13,15 @@
 #include "nsILocale.h"
 #include "nsILocaleService.h"
 #include "nsLocale.h"
-#include "nsLocaleCID.h"
-#include "nsServiceManagerUtils.h"
-#include "nsReadableUtils.h"
 #include "nsCRT.h"
 #include "prprf.h"
 #include "nsTArray.h"
+#include "nsString.h"
 
 #include <ctype.h>
 
 #if defined(XP_WIN)
 #  include "nsWin32Locale.h"
-#elif defined(XP_OS2)
-#  include "unidef.h"
-#  include "nsIOS2Locale.h"
 #elif defined(XP_MACOSX)
 #  include <Carbon/Carbon.h>
 #elif defined(XP_UNIX)
@@ -83,7 +46,7 @@ const char* LocaleList[LocaleListLength] =
 #define NSILOCALE_MAX_ACCEPT_LANGUAGE	16
 #define NSILOCALE_MAX_ACCEPT_LENGTH		18
 
-#if (defined(XP_UNIX) && !defined(XP_MACOSX)) || defined(XP_OS2)
+#if (defined(XP_UNIX) && !defined(XP_MACOSX))
 static int posix_locale_category[LocaleListLength] =
 {
   LC_COLLATE,
@@ -109,7 +72,7 @@ public:
 	//
 	// nsISupports
 	//
-	NS_DECL_ISUPPORTS
+	NS_DECL_THREADSAFE_ISUPPORTS
 
 	//
 	// nsILocaleService
@@ -134,7 +97,6 @@ protected:
 // nsLocaleService methods
 //
 nsLocaleService::nsLocaleService(void) 
-    : mSystemLocale(0), mApplicationLocale(0)
 {
 #ifdef XP_WIN
     nsAutoString        xpLocale;
@@ -142,26 +104,26 @@ nsLocaleService::nsLocaleService(void)
     // get the system LCID
     //
     LCID win_lcid = GetSystemDefaultLCID();
-    NS_ENSURE_TRUE(win_lcid, );
+    NS_ENSURE_TRUE_VOID(win_lcid);
     nsWin32Locale::GetXPLocale(win_lcid, xpLocale);
     nsresult rv = NewLocale(xpLocale, getter_AddRefs(mSystemLocale));
-    NS_ENSURE_SUCCESS(rv, );
+    NS_ENSURE_SUCCESS_VOID(rv);
 
     //
     // get the application LCID
     //
     win_lcid = GetUserDefaultLCID();
-    NS_ENSURE_TRUE(win_lcid, );
+    NS_ENSURE_TRUE_VOID(win_lcid);
     nsWin32Locale::GetXPLocale(win_lcid, xpLocale);
     rv = NewLocale(xpLocale, getter_AddRefs(mApplicationLocale));
-    NS_ENSURE_SUCCESS(rv, );
+    NS_ENSURE_SUCCESS_VOID(rv);
 #endif
 #if defined(XP_UNIX) && !defined(XP_MACOSX)
     nsRefPtr<nsLocale> resultLocale(new nsLocale());
-    NS_ENSURE_TRUE(resultLocale, );
+    NS_ENSURE_TRUE_VOID(resultLocale);
 
 #ifdef MOZ_WIDGET_QT
-    const char* lang = QLocale::system().name().toAscii();
+    const char* lang = QLocale::system().name().toUtf8();
 #else
     // Get system configuration
     const char* lang = getenv("LANG");
@@ -178,11 +140,11 @@ nsLocaleService::nsLocaleService(void)
         CopyASCIItoUTF16(LocaleList[i], category);
         category_platform = category;
         category_platform.AppendLiteral("##PLATFORM");
-        if (lc_temp != nsnull) {
+        if (lc_temp != nullptr) {
             result = nsPosixLocale::GetXPLocale(lc_temp, xpLocale);
             CopyASCIItoUTF16(lc_temp, platformLocale);
         } else {
-            if ( lang == nsnull ) {
+            if ( lang == nullptr ) {
                 platformLocale.AssignLiteral("en_US");
                 result = nsPosixLocale::GetXPLocale("en-US", xpLocale);
             } else {
@@ -200,57 +162,6 @@ nsLocaleService::nsLocaleService(void)
     mApplicationLocale = do_QueryInterface(resultLocale);
        
 #endif // XP_UNIX
-#ifdef XP_OS2
-    nsCOMPtr<nsIOS2Locale> os2Converter = do_GetService(NS_OS2LOCALE_CONTRACTID);
-    nsAutoString xpLocale;
-    if (os2Converter) {
-        nsAutoString category;
-        int i;
-
-        nsRefPtr<nsLocale> resultLocale(new nsLocale());
-        if ( resultLocale == NULL ) { 
-            return; 
-        }
-
-        LocaleObject locale_object = NULL;
-        int result = UniCreateLocaleObject(UNI_UCS_STRING_POINTER,
-                                           (UniChar *)L"", &locale_object);
-        if (result != ULS_SUCCESS) {
-            int result = UniCreateLocaleObject(UNI_UCS_STRING_POINTER,
-                                               (UniChar *)L"en_US", &locale_object);
-        }
-        char* lc_temp;
-        for( i = 0; i < LocaleListLength; i++ ) {
-            lc_temp = nsnull;
-            UniQueryLocaleObject(locale_object,
-                                 posix_locale_category[i],
-                                 UNI_MBS_STRING_POINTER,
-                                 (void **)&lc_temp);
-            category.AssignWithConversion(LocaleList[i]);
-            nsresult result;
-            if (lc_temp != nsnull)
-                result = os2Converter->GetXPLocale(lc_temp, xpLocale);
-            else {
-                char* lang = getenv("LANG");
-                if ( lang == nsnull ) {
-                    result = os2Converter->GetXPLocale("en-US", xpLocale);
-                }
-                else
-                    result = os2Converter->GetXPLocale(lang, xpLocale); 
-            }
-            if (NS_FAILED(result)) {
-                UniFreeMem(lc_temp);
-                UniFreeLocaleObject(locale_object);
-                return;
-            }
-            resultLocale->AddCategory(category, xpLocale);
-            UniFreeMem(lc_temp);
-        }
-        UniFreeLocaleObject(locale_object);
-        mSystemLocale = do_QueryInterface(resultLocale);
-        mApplicationLocale = do_QueryInterface(resultLocale);
-    }  // if ( NS_SUCCEEDED )...
-#endif  // XP_OS2
 
 #ifdef XP_MACOSX
     // Get string representation of user's current locale
@@ -260,20 +171,18 @@ nsLocaleService::nsLocaleService(void)
 
     nsAutoTArray<UniChar, 32> buffer;
     int size = ::CFStringGetLength(userLocaleStr);
-    if (buffer.SetLength(size + 1))
-    {
-        CFRange range = ::CFRangeMake(0, size);
-        ::CFStringGetCharacters(userLocaleStr, range, buffer.Elements());
-        buffer[size] = 0;
+    buffer.SetLength(size + 1);
+    CFRange range = ::CFRangeMake(0, size);
+    ::CFStringGetCharacters(userLocaleStr, range, buffer.Elements());
+    buffer[size] = 0;
 
-        // Convert the locale string to the format that Mozilla expects
-        nsAutoString xpLocale(buffer.Elements());
-        xpLocale.ReplaceChar('_', '-');
+    // Convert the locale string to the format that GNU expects
+    nsAutoString xpLocale(reinterpret_cast<char16_t*>(buffer.Elements()));
+    xpLocale.ReplaceChar('_', '-');
 
-        nsresult rv = NewLocale(xpLocale, getter_AddRefs(mSystemLocale));
-        if (NS_SUCCEEDED(rv)) {
-            mApplicationLocale = mSystemLocale;
-        }
+    nsresult rv = NewLocale(xpLocale, getter_AddRefs(mSystemLocale));
+    if (NS_SUCCEEDED(rv)) {
+        mApplicationLocale = mSystemLocale;
     }
 
     ::CFRelease(userLocaleStr);
@@ -287,20 +196,20 @@ nsLocaleService::~nsLocaleService(void)
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsLocaleService, nsILocaleService)
+NS_IMPL_ISUPPORTS(nsLocaleService, nsILocaleService)
 
 NS_IMETHODIMP
 nsLocaleService::NewLocale(const nsAString &aLocale, nsILocale **_retval)
 {
     nsresult result;
 
-    *_retval = nsnull;
+    *_retval = nullptr;
 
     nsRefPtr<nsLocale> resultLocale(new nsLocale());
     if (!resultLocale) return NS_ERROR_OUT_OF_MEMORY;
 
-    for (PRInt32 i = 0; i < LocaleListLength; i++) {
-      nsString category; category.AssignWithConversion(LocaleList[i]);
+    for (int32_t i = 0; i < LocaleListLength; i++) {
+      NS_ConvertASCIItoUTF16 category(LocaleList[i]);
       result = resultLocale->AddCategory(category, aLocale);
       if (NS_FAILED(result)) return result;
 #if defined(XP_UNIX) && !defined(XP_MACOSX)
@@ -323,7 +232,7 @@ nsLocaleService::GetSystemLocale(nsILocale **_retval)
 		return NS_OK;
 	}
 
-	*_retval = (nsILocale*)nsnull;
+	*_retval = (nsILocale*)nullptr;
 	return NS_ERROR_FAILURE;
 }
 
@@ -335,14 +244,13 @@ nsLocaleService::GetApplicationLocale(nsILocale **_retval)
 		return NS_OK;
 	}
 
-	*_retval=(nsILocale*)nsnull;
+	*_retval=(nsILocale*)nullptr;
 	return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsLocaleService::GetLocaleFromAcceptLanguage(const char *acceptLanguage, nsILocale **_retval)
 {
-  char* input;
   char* cPtr;
   char* cPtr1;
   char* cPtr2;
@@ -352,9 +260,7 @@ nsLocaleService::GetLocaleFromAcceptLanguage(const char *acceptLanguage, nsILoca
   char	acceptLanguageList[NSILOCALE_MAX_ACCEPT_LANGUAGE][NSILOCALE_MAX_ACCEPT_LENGTH];
   nsresult	result;
 
-  input = new char[strlen(acceptLanguage)+1];
-  NS_ASSERTION(input!=nsnull,"nsLocaleFactory::GetLocaleFromAcceptLanguage: memory allocation failed.");
-  if (input == (char*)NULL){ return NS_ERROR_OUT_OF_MEMORY; }
+  nsAutoArrayPtr<char> input(new char[strlen(acceptLanguage)+1]);
 
   strcpy(input, acceptLanguage);
   cPtr1 = input-1;
@@ -385,7 +291,7 @@ nsLocaleService::GetLocaleFromAcceptLanguage(const char *acceptLanguage, nsILoca
     while (cPtr) {
       qvalue[countLang] = 1.0f;
       /* add extra parens to get rid of warning */
-      if ((cPtr1 = strchr(cPtr,';')) != nsnull) {
+      if ((cPtr1 = strchr(cPtr,';')) != nullptr) {
         PR_sscanf(cPtr1,";q=%f",&qvalue[countLang]);
         *cPtr1 = '\0';
       }
@@ -439,7 +345,6 @@ nsLocaleService::GetLocaleFromAcceptLanguage(const char *acceptLanguage, nsILoca
   //
   // clean up
   //
-  delete[] input;
   return result;
 }
 
