@@ -1,58 +1,21 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 
 #include "prmem.h"
 #include "prprf.h"
-#include "nsIServiceManager.h"
-#include "nsIComponentManager.h"
 #include "nsICharsetConverterManager.h"
 #include "nsSaveAsCharset.h"
-#include "nsCRT.h"
-#include "nsUnicharUtils.h"
-#include "nsCompressedCharMap.h"
-#include "nsReadableUtils.h"
 #include "nsWhitespaceTokenizer.h"
+#include "nsServiceManagerUtils.h"
 
 //
 // nsISupports methods
 //
-NS_IMPL_ISUPPORTS1(nsSaveAsCharset, nsISaveAsCharset)
+NS_IMPL_ISUPPORTS(nsSaveAsCharset, nsISaveAsCharset)
 
 //
 // nsSaveAsCharset
@@ -69,7 +32,7 @@ nsSaveAsCharset::~nsSaveAsCharset()
 }
 
 NS_IMETHODIMP
-nsSaveAsCharset::Init(const char *charset, PRUint32 attr, PRUint32 entityVersion)
+nsSaveAsCharset::Init(const char *charset, uint32_t attr, uint32_t entityVersion)
 {
   nsresult rv = NS_OK;
 
@@ -91,12 +54,10 @@ nsSaveAsCharset::Init(const char *charset, PRUint32 attr, PRUint32 entityVersion
 }
 
 NS_IMETHODIMP
-nsSaveAsCharset::Convert(const PRUnichar *inString, char **_retval)
+nsSaveAsCharset::Convert(const char16_t *inString, char **_retval)
 {
-  if (nsnull == _retval)
-    return NS_ERROR_NULL_POINTER;
-  if (nsnull == inString)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(_retval);
+  NS_ENSURE_ARG_POINTER(inString);
   if (0 == *inString)
     return NS_ERROR_ILLEGAL_VALUE;
   nsresult rv = NS_OK;
@@ -104,7 +65,7 @@ nsSaveAsCharset::Convert(const PRUnichar *inString, char **_retval)
   NS_ASSERTION(mEncoder, "need to call Init() before Convert()");
   NS_ENSURE_TRUE(mEncoder, NS_ERROR_FAILURE);
 
-  *_retval = nsnull;
+  *_retval = nullptr;
 
   // make sure to start from the first charset in the list
   if (mCharsetListIndex > 0) {
@@ -127,7 +88,7 @@ nsSaveAsCharset::Convert(const PRUnichar *inString, char **_retval)
     if (attr_EntityBeforeCharsetConv == MASK_ENTITY(mAttribute)) {
       NS_ASSERTION(mEntityConverter, "need to call Init() before Convert()");
       NS_ENSURE_TRUE(mEntityConverter, NS_ERROR_FAILURE);
-      PRUnichar *entity = nsnull;
+      char16_t *entity = nullptr;
       // do the entity conversion first
       rv = mEntityConverter->ConvertToEntities(inString, mEntityVersion, &entity);
       if(NS_SUCCEEDED(rv)) {
@@ -152,40 +113,46 @@ nsSaveAsCharset::GetCharset(char * *aCharset)
 
   const char* charset = mCharsetList[mCharsetListIndex].get();
   if (!charset) {
-    *aCharset = nsnull;
+    *aCharset = nullptr;
     NS_ASSERTION(charset, "make sure to call Init() with non empty charset list");
     return NS_ERROR_FAILURE;
   }
 
-  *aCharset = nsCRT::strdup(charset);
+  *aCharset = strdup(charset);
   return (*aCharset) ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////
 
+#define RESERVE_FALLBACK_BYTES 512
+
 // do the fallback, reallocate the buffer if necessary
 // need to pass destination buffer info (size, current position and estimation of rest of the conversion)
 NS_IMETHODIMP
-nsSaveAsCharset::HandleFallBack(PRUint32 character, char **outString, PRInt32 *bufferLength, 
-                                PRInt32 *currentPos, PRInt32 estimatedLength)
+nsSaveAsCharset::HandleFallBack(uint32_t character, char **outString, int32_t *bufferLength, 
+                                int32_t *currentPos, int32_t estimatedLength)
 {
-  if((nsnull == outString ) || (nsnull == bufferLength) ||(nsnull ==currentPos))
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(outString);
+  NS_ENSURE_ARG_POINTER(bufferLength);
+  NS_ENSURE_ARG_POINTER(currentPos);
+
   char fallbackStr[256];
   nsresult rv = DoConversionFallBack(character, fallbackStr, 256);
   if (NS_SUCCEEDED(rv)) {
-    PRInt32 tempLen = (PRInt32) PL_strlen(fallbackStr);
+    int32_t tempLen = (int32_t) strlen(fallbackStr);
 
     // reallocate if the buffer is not large enough
     if ((tempLen + estimatedLength) >= (*bufferLength - *currentPos)) {
-      char *temp = (char *) PR_Realloc(*outString, *bufferLength + tempLen);
-      if (NULL != temp) {
+      int32_t addLength = tempLen + RESERVE_FALLBACK_BYTES;
+      // + 1 is for the terminating NUL, don't add that to bufferLength
+      char *temp = (char *) PR_Realloc(*outString, *bufferLength + addLength + 1);
+      if (temp) {
         // adjust length/pointer after realloc
-        *bufferLength += tempLen;
+        *bufferLength += addLength;
         *outString = temp;
       } else {
-        *outString = NULL;
-        *bufferLength =0;
+        *outString = nullptr;
+        *bufferLength = 0;
         return NS_ERROR_OUT_OF_MEMORY;
       }
     }
@@ -196,35 +163,37 @@ nsSaveAsCharset::HandleFallBack(PRUint32 character, char **outString, PRInt32 *b
 }
 
 NS_IMETHODIMP
-nsSaveAsCharset::DoCharsetConversion(const PRUnichar *inString, char **outString)
+nsSaveAsCharset::DoCharsetConversion(const char16_t *inString, char **outString)
 {
-  if(nsnull == outString )
-    return NS_ERROR_NULL_POINTER;
-  NS_ASSERTION(outString, "invalid input");
+  NS_ENSURE_ARG_POINTER(outString);
 
-  *outString = NULL;
+  *outString = nullptr;
 
   nsresult rv;
-  PRInt32 inStringLength = nsCRT::strlen(inString);   // original input string length
-  PRInt32 bufferLength;                               // allocated buffer length
-  PRInt32 srcLength = inStringLength;
-  PRInt32 dstLength;
-  char *dstPtr = NULL;
-  PRInt32 pos1, pos2;
+  int32_t inStringLength = NS_strlen(inString);       // original input string length
+  int32_t bufferLength;                               // allocated buffer length
+  int32_t srcLength = inStringLength;
+  int32_t dstLength;
+  int32_t pos1, pos2;
   nsresult saveResult = NS_OK;                         // to remember NS_ERROR_UENC_NOMAPPING
 
   // estimate and allocate the target buffer (reserve extra memory for fallback)
   rv = mEncoder->GetMaxLength(inString, inStringLength, &dstLength);
   if (NS_FAILED(rv)) return rv;
 
-  bufferLength = dstLength + 512; // reserve 512 byte for fallback.
-  dstPtr = (char *) PR_Malloc(bufferLength);
-  if (NULL == dstPtr) return NS_ERROR_OUT_OF_MEMORY;
-
+  bufferLength = dstLength + RESERVE_FALLBACK_BYTES; // extra bytes for fallback
+  // + 1 is for the terminating NUL -- we don't add that to bufferLength so that
+  // we can always write dstPtr[pos2] = '\0' even when the encoder filled the
+  // buffer.
+  char *dstPtr = (char *) PR_Malloc(bufferLength + 1);
+  if (!dstPtr) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
   
   for (pos1 = 0, pos2 = 0; pos1 < inStringLength;) {
     // convert from unicode
     dstLength = bufferLength - pos2;
+    NS_ASSERTION(dstLength >= 0, "out of bounds write");
     rv = mEncoder->Convert(&inString[pos1], &srcLength, &dstPtr[pos2], &dstLength);
 
     pos1 += srcLength ? srcLength : 1;
@@ -250,7 +219,7 @@ nsSaveAsCharset::DoCharsetConversion(const PRUnichar *inString, char **outString
 
     // do the fallback
     if (!ATTR_NO_FALLBACK(mAttribute)) {
-      PRUint32 unMappedChar;
+      uint32_t unMappedChar;
       if (NS_IS_HIGH_SURROGATE(inString[pos1-1]) && 
           inStringLength > pos1 && NS_IS_LOW_SURROGATE(inString[pos1])) {
         unMappedChar = SURROGATE_TO_UCS4(inString[pos1-1], inString[pos1]);
@@ -296,11 +265,9 @@ nsSaveAsCharset::DoCharsetConversion(const PRUnichar *inString, char **outString
 }
 
 NS_IMETHODIMP
-nsSaveAsCharset::DoConversionFallBack(PRUint32 inUCS4, char *outString, PRInt32 bufferLength)
+nsSaveAsCharset::DoConversionFallBack(uint32_t inUCS4, char *outString, int32_t bufferLength)
 {
-  NS_ASSERTION(outString, "invalid input");
-  if(nsnull == outString )
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(outString);
 
   *outString = '\0';
 
@@ -310,10 +277,10 @@ nsSaveAsCharset::DoConversionFallBack(PRUint32 inUCS4, char *outString, PRInt32
     return NS_OK;
   }
   if (attr_EntityAfterCharsetConv == MASK_ENTITY(mAttribute)) {
-    char *entity = NULL;
+    char *entity = nullptr;
     rv = mEntityConverter->ConvertUTF32ToEntity(inUCS4, mEntityVersion, &entity);
     if (NS_SUCCEEDED(rv)) {
-      if (NULL == entity || (PRInt32)strlen(entity) > bufferLength) {
+      if (!entity || (int32_t)strlen(entity) > bufferLength) {
         return NS_ERROR_OUT_OF_MEMORY;
       }
       PL_strcpy(outString, entity);
@@ -390,8 +357,8 @@ nsresult nsSaveAsCharset::SetupCharsetList(const char *charsetList)
 
 const char * nsSaveAsCharset::GetNextCharset()
 {
-  if ((mCharsetListIndex + 1) >= PRInt32(mCharsetList.Length()))
-    return nsnull;
+  if ((mCharsetListIndex + 1) >= int32_t(mCharsetList.Length()))
+    return nullptr;
 
   // bump the index and return the next charset
   return mCharsetList[++mCharsetListIndex].get();
