@@ -1,57 +1,24 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: sw=2 ts=2 et :
  */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Plugin App.
- *
- * The Initial Developer of the Original Code is
- *   Ben Turner <bent.mozilla@gmail.com>
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Jim Mathies <jmathies@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+#include "mozilla/DebugOnly.h"
 
 #include "WindowsMessageLoop.h"
-#include "RPCChannel.h"
+#include "MessageChannel.h"
 
 #include "nsAutoPtr.h"
 #include "nsServiceManagerUtils.h"
-#include "nsStringGlue.h"
+#include "nsString.h"
 #include "nsIXULAppInfo.h"
 
-#include "mozilla/Mutex.h"
-
-using mozilla::ipc::SyncChannel;
-using mozilla::ipc::RPCChannel;
-using mozilla::MutexAutoUnlock;
+#include "mozilla/PaintTracker.h"
 
+using namespace mozilla;
+using namespace mozilla::ipc;
 using namespace mozilla::ipc::windows;
 
 /**
@@ -94,13 +61,28 @@ using namespace mozilla::ipc::windows;
  * in a special window procedure where we can either ignore the message or
  * process it in some fashion.
  *
- * Queued and "non-queued" messages will be processed during RPC calls if
- * modal UI related api calls block an RPC in-call in the child. To prevent
+ * Queued and "non-queued" messages will be processed during Interrupt calls if
+ * modal UI related api calls block an Interrupt in-call in the child. To prevent
  * windows from freezing, and to allow concurrent processing of critical
  * events (such as painting), we spin a native event dispatch loop while
  * these in-calls are blocked.
  */
 
+#if defined(ACCESSIBILITY)
+// pulled from accessibility's win utils
+extern const wchar_t* kPropNameTabContent;
+#endif
+
+// widget related message id constants we need to defer
+namespace mozilla {
+namespace widget {
+extern UINT sAppShellGeckoMsgId;
+#ifdef MOZ_METRO
+extern UINT sDefaultBrowserMsgId;
+#endif
+}
+}
+
 namespace {
 
 const wchar_t kOldWndProcProp[] = L"MozillaIPCOldWndProc";
@@ -108,19 +90,18 @@ const wchar_t kOldWndProcProp[] = L"MozillaIPCOldWndProc";
 // This isn't defined before Windows XP.
 enum { WM_XP_THEMECHANGED = 0x031A };
 
-PRUnichar gAppMessageWindowName[256] = { 0 };
-PRInt32 gAppMessageWindowNameLength = 0;
+char16_t gAppMessageWindowName[256] = { 0 };
+int32_t gAppMessageWindowNameLength = 0;
 
-nsTArray<HWND>* gNeuteredWindows = nsnull;
+nsTArray<HWND>* gNeuteredWindows = nullptr;
 
 typedef nsTArray<nsAutoPtr<DeferredMessage> > DeferredMessageArray;
-DeferredMessageArray* gDeferredMessages = nsnull;
+DeferredMessageArray* gDeferredMessages = nullptr;
 
-HHOOK gDeferredGetMsgHook = NULL;
-HHOOK gDeferredCallWndProcHook = NULL;
+HHOOK gDeferredGetMsgHook = nullptr;
+HHOOK gDeferredCallWndProcHook = nullptr;
 
 DWORD gUIThreadId = 0;
-int gEventLoopDepth = 0;
 
 LRESULT CALLBACK
 DeferredMessageHook(int nCode,
@@ -134,10 +115,10 @@ DeferredMessageHook(int nCode,
   // Only run deferred messages if all of these conditions are met:
   //   1. The |nCode| indicates that this hook should do something.
   //   2. We have deferred messages to run.
-  //   3. We're not being called from the PeekMessage within the WaitForNotify
-  //      function (indicated with SyncChannel::IsPumpingMessages). We really
+  //   3. We're not being called from the PeekMessage within the WaitFor*Notify
+  //      function (indicated with MessageChannel::IsPumpingMessages). We really
   //      only want to run after returning to the main event loop.
-  if (nCode >= 0 && gDeferredMessages && !SyncChannel::IsPumpingMessages()) {
+  if (nCode >= 0 && gDeferredMessages && !MessageChannel::IsPumpingMessages()) {
     NS_ASSERTION(gDeferredGetMsgHook && gDeferredCallWndProcHook,
                  "These hooks must be set if we're being called!");
     NS_ASSERTION(gDeferredMessages->Length(), "No deferred messages?!");
@@ -150,17 +131,17 @@ DeferredMessageHook(int nCode,
 
     // Unset the global and make sure we delete it when we're done here.
     nsAutoPtr<DeferredMessageArray> messages(gDeferredMessages);
-    gDeferredMessages = nsnull;
+    gDeferredMessages = nullptr;
 
     // Run all the deferred messages in order.
-    PRUint32 count = messages->Length();
-    for (PRUint32 index = 0; index < count; index++) {
+    uint32_t count = messages->Length();
+    for (uint32_t index = 0; index < count; index++) {
       messages->ElementAt(index)->Run();
     }
   }
 
   // Always call the next hook.
-  return CallNextHookEx(NULL, nCode, wParam, lParam);
+  return CallNextHookEx(nullptr, nCode, wParam, lParam);
 }
 
 void
@@ -171,9 +152,9 @@ ScheduleDeferredMessageRun()
     NS_ASSERTION(gDeferredMessages->Length(), "No deferred messages?!");
 
     gDeferredGetMsgHook = ::SetWindowsHookEx(WH_GETMESSAGE, DeferredMessageHook,
-                                             NULL, gUIThreadId);
+                                             nullptr, gUIThreadId);
     gDeferredCallWndProcHook = ::SetWindowsHookEx(WH_CALLWNDPROC,
-                                                  DeferredMessageHook, NULL,
+                                                  DeferredMessageHook, nullptr,
                                                   gUIThreadId);
     NS_ASSERTION(gDeferredGetMsgHook && gDeferredCallWndProcHook,
                  "Failed to set hooks!");
@@ -186,7 +167,7 @@ ProcessOrDeferMessage(HWND hwnd,
                       WPARAM wParam,
                       LPARAM lParam)
 {
-  DeferredMessage* deferred = nsnull;
+  DeferredMessage* deferred = nullptr;
 
   // Most messages ask for 0 to be returned if the message is processed.
   LRESULT res = 0;
@@ -209,6 +190,7 @@ ProcessOrDeferMessage(HWND hwnd,
     case WM_PARENTNOTIFY:
     case WM_SETFOCUS:
     case WM_SYSCOMMAND:
+    case WM_DISPLAYCHANGE:
     case WM_SHOWWINDOW: // Intentional fall-through.
     case WM_XP_THEMECHANGED: {
       deferred = new DeferredSendMessage(hwnd, uMsg, wParam, lParam);
@@ -216,6 +198,7 @@ ProcessOrDeferMessage(HWND hwnd,
     }
 
     case WM_DEVICECHANGE:
+    case WM_POWERBROADCAST:
     case WM_NCACTIVATE: // Intentional fall-through.
     case WM_SETCURSOR: {
       // Friggin unconventional return value...
@@ -239,12 +222,6 @@ ProcessOrDeferMessage(HWND hwnd,
       deferred = new DeferredRedrawMessage(hwnd, flags);
       break;
     }
-    case WM_NCPAINT: {
-      UINT flags = RDW_INVALIDATE | RDW_FRAME | RDW_NOINTERNALPAINT |
-                   RDW_NOERASE | RDW_NOCHILDREN | RDW_ERASENOW;
-      deferred = new DeferredRedrawMessage(hwnd, flags);
-      break;
-    }
 
     // This message will generate a WM_PAINT message if there are invalid
     // areas.
@@ -288,35 +265,56 @@ ProcessOrDeferMessage(HWND hwnd,
     // Messages that are safe to pass to DefWindowProc go here.
     case WM_ENTERIDLE:
     case WM_GETICON:
+    case WM_NCPAINT: // (never trap nc paint events)
     case WM_GETMINMAXINFO:
     case WM_GETTEXT:
     case WM_NCHITTEST:
-    case WM_STYLECHANGING:
-    case WM_SYNCPAINT: // Intentional fall-through.
-    case WM_WINDOWPOSCHANGING: {
+    case WM_STYLECHANGING:  // Intentional fall-through.
+    case WM_WINDOWPOSCHANGING: { 
       return DefWindowProc(hwnd, uMsg, wParam, lParam);
     }
 
-    // Unknown messages only.
+    // Just return, prevents DefWindowProc from messaging the window
+    // syncronously with other events, which may be deferred. Prevents 
+    // random shutdown of aero composition on the window. 
+    case WM_SYNCPAINT:
+      return 0;
+
+    // This message causes QuickTime to make re-entrant calls.
+    // Simply discarding it doesn't seem to hurt anything.
+    case WM_APP-1:
+      return 0;
+
     default: {
-#ifdef DEBUG
-      nsCAutoString log("Received \"nonqueued\" message ");
-      log.AppendInt(uMsg);
-      log.AppendLiteral(" during a synchronous IPC message for window ");
-      log.AppendInt((PRInt64)hwnd);
-
-      wchar_t className[256] = { 0 };
-      if (GetClassNameW(hwnd, className, sizeof(className) - 1) > 0) {
-        log.AppendLiteral(" (\"");
-        log.Append(NS_ConvertUTF16toUTF8((PRUnichar*)className));
-        log.AppendLiteral("\")");
-      }
-
-      log.AppendLiteral(", sending it to DefWindowProc instead of the normal "
-                        "window procedure.");
-      NS_ERROR(log.get());
+      if (uMsg && uMsg == mozilla::widget::sAppShellGeckoMsgId) {
+        // Widget's registered native event callback
+        deferred = new DeferredSendMessage(hwnd, uMsg, wParam, lParam);
+#ifdef MOZ_METRO
+      } else if (uMsg && uMsg == mozilla::widget::sDefaultBrowserMsgId) {
+        // Metro widget's system shutdown message
+        deferred = new DeferredSendMessage(hwnd, uMsg, wParam, lParam);
 #endif
-      return DefWindowProc(hwnd, uMsg, wParam, lParam);
+      } else {
+        // Unknown messages only
+#ifdef DEBUG
+        nsAutoCString log("Received \"nonqueued\" message ");
+        log.AppendInt(uMsg);
+        log.AppendLiteral(" during a synchronous IPC message for window ");
+        log.AppendInt((int64_t)hwnd);
+
+        wchar_t className[256] = { 0 };
+        if (GetClassNameW(hwnd, className, sizeof(className) - 1) > 0) {
+          log.AppendLiteral(" (\"");
+          log.Append(NS_ConvertUTF16toUTF8((char16_t*)className));
+          log.AppendLiteral("\")");
+        }
+
+        log.AppendLiteral(", sending it to DefWindowProc instead of the normal "
+                          "window procedure.");
+        NS_ERROR(log.get());
+#endif
+        return DefWindowProc(hwnd, uMsg, wParam, lParam);
+      }
     }
   }
 
@@ -333,6 +331,9 @@ ProcessOrDeferMessage(HWND hwnd,
   return res;
 }
 
+} // anonymous namespace
+
+// We need the pointer value of this in PluginInstanceChild.
 LRESULT CALLBACK
 NeuteredWindowProc(HWND hwnd,
                    UINT uMsg,
@@ -351,6 +352,8 @@ NeuteredWindowProc(HWND hwnd,
   return ProcessOrDeferMessage(hwnd, uMsg, wParam, lParam);
 }
 
+namespace {
+
 static bool
 WindowIsDeferredWindow(HWND hWnd)
 {
@@ -359,13 +362,21 @@ WindowIsDeferredWindow(HWND hWnd)
     return false;
   }
 
-  PRUnichar buffer[256] = { 0 };
+  char16_t buffer[256] = { 0 };
   int length = GetClassNameW(hWnd, (wchar_t*)buffer, sizeof(buffer) - 1);
   if (length <= 0) {
     NS_WARNING("Failed to get class name!");
     return false;
   }
 
+#if defined(ACCESSIBILITY)
+  // Tab content creates a window that responds to accessible WM_GETOBJECT
+  // calls. This window can safely be ignored.
+  if (::GetPropW(hWnd, kPropNameTabContent)) {
+    return false;
+  }
+#endif
+
   // Common mozilla windows we must defer messages to.
   nsDependentString className(buffer, length);
   if (StringBeginsWith(className, NS_LITERAL_STRING("Mozilla")) ||
@@ -375,22 +386,38 @@ WindowIsDeferredWindow(HWND hWnd)
     return true;
   }
 
+#ifdef MOZ_METRO
+  // immersive UI ICoreWindow
+  if (className.EqualsLiteral("Windows.UI.Core.CoreWindow")) {
+    return true;
+  }
+#endif
+
   // Plugin windows that can trigger ipc calls in child:
   // 'ShockwaveFlashFullScreen' - flash fullscreen window
+  // 'QTNSHIDDEN' - QuickTime
   // 'AGFullScreenWinClass' - silverlight fullscreen window
   if (className.EqualsLiteral("ShockwaveFlashFullScreen") ||
+      className.EqualsLiteral("QTNSHIDDEN") ||
       className.EqualsLiteral("AGFullScreenWinClass")) {
     return true;
   }
 
-  // nsNativeAppSupport makes a window like "FirefoxMessageWindow" based on the
+  // Google Earth bridging msg window between the plugin instance and a separate
+  // earth process. The earth process can trigger a plugin incall on the browser
+  // at any time, which is badness if the instance is already making an incall.
+  if (className.EqualsLiteral("__geplugin_bridge_window__")) {
+    return true;
+  }
+
+  // nsNativeAppSupport makes a window like "IceCatMessageWindow" based on the
   // toolkit app's name. It's pretty expensive to calculate this so we only try
   // once.
   if (gAppMessageWindowNameLength == 0) {
     nsCOMPtr<nsIXULAppInfo> appInfo =
       do_GetService("@mozilla.org/xre/app-info;1");
     if (appInfo) {
-      nsCAutoString appName;
+      nsAutoCString appName;
       if (NS_SUCCEEDED(appInfo->GetName(appName))) {
         appName.Append("MessageWindow");
         nsDependentString windowName(gAppMessageWindowName);
@@ -424,9 +451,9 @@ NeuterWindowProcedure(HWND hWnd)
 
   NS_ASSERTION(!GetProp(hWnd, kOldWndProcProp), "This should always be null!");
 
-  // It's possible to get NULL out of SetWindowLongPtr, and the only way to know
-  // if that's a valid old value is to use GetLastError. Clear the error here so
-  // we can tell.
+  // It's possible to get nullptr out of SetWindowLongPtr, and the only way to
+  // know if that's a valid old value is to use GetLastError. Clear the error
+  // here so we can tell.
   SetLastError(ERROR_SUCCESS);
 
   LONG_PTR currentWndProc =
@@ -458,17 +485,17 @@ RestoreWindowProcedure(HWND hWnd)
 {
   NS_ASSERTION(WindowIsDeferredWindow(hWnd),
                "Not a deferred window, this shouldn't be in our list!");
-
-  LONG_PTR oldWndProc = (LONG_PTR)RemoveProp(hWnd, kOldWndProcProp);
+  LONG_PTR oldWndProc = (LONG_PTR)GetProp(hWnd, kOldWndProcProp);
   if (oldWndProc) {
     NS_ASSERTION(oldWndProc != (LONG_PTR)NeuteredWindowProc,
                  "This shouldn't be possible!");
 
-    LONG_PTR currentWndProc =
+    DebugOnly<LONG_PTR> currentWndProc =
       SetWindowLongPtr(hWnd, GWLP_WNDPROC, oldWndProc);
     NS_ASSERTION(currentWndProc == (LONG_PTR)NeuteredWindowProc,
                  "This should never be switched out from under us!");
   }
+  RemoveProp(hWnd, kOldWndProcProp);
 }
 
 LRESULT CALLBACK
@@ -488,7 +515,7 @@ CallWindowProcedureHook(int nCode,
       }
     }
   }
-  return CallNextHookEx(NULL, nCode, wParam, lParam);
+  return CallNextHookEx(nullptr, nCode, wParam, lParam);
 }
 
 inline void
@@ -506,26 +533,13 @@ UnhookNeuteredWindows()
 {
   if (!gNeuteredWindows)
     return;
-  PRUint32 count = gNeuteredWindows->Length();
-  for (PRUint32 index = 0; index < count; index++) {
+  uint32_t count = gNeuteredWindows->Length();
+  for (uint32_t index = 0; index < count; index++) {
     RestoreWindowProcedure(gNeuteredWindows->ElementAt(index));
   }
   gNeuteredWindows->Clear();
 }
 
-void
-Init()
-{
-  // If we aren't setup before a call to NotifyWorkerThread, we'll hang
-  // on startup.
-  if (!gUIThreadId) {
-    gUIThreadId = GetCurrentThreadId();
-  }
-  NS_ASSERTION(gUIThreadId, "ThreadId should not be 0!");
-  NS_ASSERTION(gUIThreadId == GetCurrentThreadId(),
-               "Running on different threads!");
-}
-
 // This timeout stuff assumes a sane value of mTimeoutMs (less than the overflow
 // value for GetTickCount(), which is something like 50 days). It uses the
 // cheapest (and least accurate) method supported by Windows 2000.
@@ -538,7 +552,7 @@ struct TimeoutData
 
 void
 InitTimeoutData(TimeoutData* aData,
-                int32 aTimeoutMs)
+                int32_t aTimeoutMs)
 {
   aData->startTicks = GetTickCount();
   if (!aData->startTicks) {
@@ -567,13 +581,42 @@ TimeoutHasExpired(const TimeoutData& aData)
 
 } // anonymous namespace
 
-RPCChannel::SyncStackFrame::SyncStackFrame(SyncChannel* channel, bool rpc)
-  : mRPC(rpc)
+namespace mozilla {
+namespace ipc {
+namespace windows {
+
+void
+InitUIThread()
+{
+  // If we aren't setup before a call to NotifyWorkerThread, we'll hang
+  // on startup.
+  if (!gUIThreadId) {
+    gUIThreadId = GetCurrentThreadId();
+  }
+  MOZ_ASSERT(gUIThreadId);
+  MOZ_ASSERT(gUIThreadId == GetCurrentThreadId(),
+               "Called InitUIThread multiple times on different threads!");
+}
+
+} // namespace windows
+} // namespace ipc
+} // namespace mozilla
+
+// See SpinInternalEventLoop below
+MessageChannel::SyncStackFrame::SyncStackFrame(MessageChannel* channel, bool interrupt)
+  : mInterrupt(interrupt)
   , mSpinNestedEvents(false)
+  , mListenerNotified(false)
   , mChannel(channel)
   , mPrev(mChannel->mTopFrame)
   , mStaticPrev(sStaticTopFrame)
 {
+  // Only track stack frames when Windows message deferral behavior
+  // is request for the channel.
+  if (!(mChannel->GetChannelFlags() & REQUIRE_DEFERRED_MESSAGE_PROTECTION)) {
+    return;
+  }
+
   mChannel->mTopFrame = this;
   sStaticTopFrame = this;
 
@@ -584,12 +627,16 @@ RPCChannel::SyncStackFrame::SyncStackFrame(SyncChannel* channel, bool rpc)
   }
 }
 
-RPCChannel::SyncStackFrame::~SyncStackFrame()
+MessageChannel::SyncStackFrame::~SyncStackFrame()
 {
+  if (!(mChannel->GetChannelFlags() & REQUIRE_DEFERRED_MESSAGE_PROTECTION)) {
+    return;
+  }
+
   NS_ASSERTION(this == mChannel->mTopFrame,
-               "Mismatched RPC stack frames");
+               "Mismatched interrupt stack frames");
   NS_ASSERTION(this == sStaticTopFrame,
-               "Mismatched static RPC stack frames");
+               "Mismatched static Interrupt stack frames");
 
   mChannel->mTopFrame = mPrev;
   sStaticTopFrame = mStaticPrev;
@@ -597,32 +644,56 @@ RPCChannel::SyncStackFrame::~SyncStackFrame()
   if (!mStaticPrev) {
     NS_ASSERTION(gNeuteredWindows, "Bad pointer!");
     delete gNeuteredWindows;
-    gNeuteredWindows = NULL;
+    gNeuteredWindows = nullptr;
   }
 }
 
-SyncChannel::SyncStackFrame* SyncChannel::sStaticTopFrame;
+MessageChannel::SyncStackFrame* MessageChannel::sStaticTopFrame;
 
-void
-RPCChannel::ProcessNativeEventsInRPCCall()
+// nsAppShell's notification that gecko events are being processed.
+// If we are here and there is an Interrupt Incall active, we are spinning
+// a nested gecko event loop. In which case the remote process needs
+// to know about it.
+void /* static */
+MessageChannel::NotifyGeckoEventDispatch()
 {
+  // sStaticTopFrame is only valid for Interrupt channels
+  if (!sStaticTopFrame || sStaticTopFrame->mListenerNotified)
+    return;
+
+  sStaticTopFrame->mListenerNotified = true;
+  MessageChannel* channel = static_cast<MessageChannel*>(sStaticTopFrame->mChannel);
+  channel->Listener()->ProcessRemoteNativeEventsInInterruptCall();
+}
+
+// invoked by the module that receives the spin event loop
+// message.
+void
+MessageChannel::ProcessNativeEventsInInterruptCall()
+{
+  NS_ASSERTION(GetCurrentThreadId() == gUIThreadId,
+               "Shouldn't be on a non-main thread in here!");
   if (!mTopFrame) {
-    NS_ERROR("Child logic error: no RPC frame");
+    NS_ERROR("Spin logic error: no Interrupt frame");
     return;
   }
 
   mTopFrame->mSpinNestedEvents = true;
 }
 
-// Spin loop is called in place of WaitForNotify when modal ui is being shown
+// Spin loop is called in place of WaitFor*Notify when modal ui is being shown
 // in a child. There are some intricacies in using it however. Spin loop is
-// enabled for a particular RPC frame by the client calling
-// RPCChannel::ProcessNativeEventsInRPCCall().
-// This call can be nested for multiple RPC frames in a single plugin or 
+// enabled for a particular Interrupt frame by the client calling
+// MessageChannel::ProcessNativeEventsInInterrupt().
+// This call can be nested for multiple Interrupt frames in a single plugin or 
 // multiple unrelated plugins.
 void
-RPCChannel::SpinInternalEventLoop()
+MessageChannel::SpinInternalEventLoop()
 {
+  if (mozilla::PaintTracker::IsPainting()) {
+    NS_RUNTIMEABORT("Don't spin an event loop while painting.");
+  }
+
   NS_ASSERTION(mTopFrame && mTopFrame->mSpinNestedEvents,
                "Spinning incorrectly");
 
@@ -637,21 +708,21 @@ RPCChannel::SpinInternalEventLoop()
 
     // Don't get wrapped up in here if the child connection dies.
     {
-      MutexAutoLock lock(mMutex);
+      MonitorAutoLock lock(*mMonitor);
       if (!Connected()) {
         return;
       }
     }
 
     // Retrieve window or thread messages
-    if (PeekMessageW(&msg, NULL, 0, 0, PM_REMOVE)) {
+    if (PeekMessageW(&msg, nullptr, 0, 0, PM_REMOVE)) {
       // The child UI should have been destroyed before the app is closed, in
       // which case, we should never get this here.
       if (msg.message == WM_QUIT) {
           NS_ERROR("WM_QUIT received in SpinInternalEventLoop!");
       } else {
           TranslateMessage(&msg);
-          DispatchMessageW(&msg);
+          ::DispatchMessageW(&msg);
           return;
       }
     }
@@ -671,22 +742,55 @@ RPCChannel::SpinInternalEventLoop()
   } while (true);
 }
 
-bool
-SyncChannel::WaitForNotify()
+static inline bool
+IsTimeoutExpired(PRIntervalTime aStart, PRIntervalTime aTimeout)
 {
-  mMutex.AssertCurrentThreadOwns();
+  return (aTimeout != PR_INTERVAL_NO_TIMEOUT) &&
+    (aTimeout <= (PR_IntervalNow() - aStart));
+}
 
-  // Initialize global objects used in deferred messaging.
-  Init();
+bool
+MessageChannel::WaitForSyncNotify()
+{
+  mMonitor->AssertCurrentThreadOwns();
 
-  NS_ASSERTION(mTopFrame && !mTopFrame->mRPC,
+  MOZ_ASSERT(gUIThreadId, "InitUIThread was not called!");
+
+  // Use a blocking wait if this channel does not require
+  // Windows message deferral behavior.
+  if (!(mFlags & REQUIRE_DEFERRED_MESSAGE_PROTECTION)) {
+    PRIntervalTime timeout = (kNoTimeout == mTimeoutMs) ?
+                             PR_INTERVAL_NO_TIMEOUT :
+                             PR_MillisecondsToInterval(mTimeoutMs);
+    // XXX could optimize away this syscall for "no timeout" case if desired
+    PRIntervalTime waitStart = 0;
+    
+    if (timeout != PR_INTERVAL_NO_TIMEOUT) {
+      waitStart = PR_IntervalNow();
+    }
+
+    mIsSyncWaitingOnNonMainThread = true;
+
+    mMonitor->Wait(timeout);
+
+    mIsSyncWaitingOnNonMainThread = false;
+
+    // If the timeout didn't expire, we know we received an event. The
+    // converse is not true.
+    return WaitResponse(timeout == PR_INTERVAL_NO_TIMEOUT ?
+                        false : IsTimeoutExpired(waitStart, timeout));
+  }
+
+  NS_ASSERTION(mFlags & REQUIRE_DEFERRED_MESSAGE_PROTECTION,
+               "Shouldn't be here for channels that don't use message deferral!");
+  NS_ASSERTION(mTopFrame && !mTopFrame->mInterrupt,
                "Top frame is not a sync frame!");
 
-  MutexAutoUnlock unlock(mMutex);
+  MonitorAutoUnlock unlock(*mMonitor);
 
-  bool retval = true;
+  bool timedout = false;
 
-  UINT_PTR timerId = NULL;
+  UINT_PTR timerId = 0;
   TimeoutData timeoutData = { 0 };
 
   if (mTimeoutMs != kNoTimeout) {
@@ -694,17 +798,17 @@ SyncChannel::WaitForNotify()
 
     // We only do this to ensure that we won't get stuck in
     // MsgWaitForMultipleObjects below.
-    timerId = SetTimer(NULL, 0, mTimeoutMs, NULL);
+    timerId = SetTimer(nullptr, 0, mTimeoutMs, nullptr);
     NS_ASSERTION(timerId, "SetTimer failed!");
   }
 
   // Setup deferred processing of native events while we wait for a response.
-  NS_ASSERTION(!SyncChannel::IsPumpingMessages(),
+  NS_ASSERTION(!MessageChannel::IsPumpingMessages(),
                "Shouldn't be pumping already!");
 
-  SyncChannel::SetIsPumpingMessages(true);
+  MessageChannel::SetIsPumpingMessages(true);
   HHOOK windowHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWindowProcedureHook,
-                                      NULL, gUIThreadId);
+                                      nullptr, gUIThreadId);
   NS_ASSERTION(windowHook, "Failed to set hook!");
 
   {
@@ -712,7 +816,7 @@ SyncChannel::WaitForNotify()
       MSG msg = { 0 };
       // Don't get wrapped up in here if the child connection dies.
       {
-        MutexAutoLock lock(mMutex);
+        MonitorAutoLock lock(*mMonitor);
         if (!Connected()) {
           break;
         }
@@ -738,7 +842,7 @@ SyncChannel::WaitForNotify()
 
       if (TimeoutHasExpired(timeoutData)) {
         // A timeout was specified and we've passed it. Break out.
-        retval = false;
+        timedout = true;
         break;
       }
 
@@ -763,7 +867,7 @@ SyncChannel::WaitForNotify()
       // have woken up for a message designated for a window in another thread.
       // If we loop immediately then we could enter a tight loop, so we'll give
       // up our time slice here to let the child process its message.
-      if (!PeekMessageW(&msg, NULL, 0, 0, PM_NOREMOVE) &&
+      if (!PeekMessageW(&msg, nullptr, 0, 0, PM_NOREMOVE) &&
           !haveSentMessagesPending) {
         // Message was for child, we should wait a bit.
         SwitchToThread();
@@ -785,59 +889,66 @@ SyncChannel::WaitForNotify()
   ScheduleDeferredMessageRun();
 
   if (timerId) {
-    KillTimer(NULL, timerId);
+    KillTimer(nullptr, timerId);
   }
 
-  SyncChannel::SetIsPumpingMessages(false);
+  MessageChannel::SetIsPumpingMessages(false);
 
-  return retval;
+  return WaitResponse(timedout);
 }
 
 bool
-RPCChannel::WaitForNotify()
+MessageChannel::WaitForInterruptNotify()
 {
-  mMutex.AssertCurrentThreadOwns();
+  mMonitor->AssertCurrentThreadOwns();
 
-  if (!StackDepth() && !mBlockedOnParent) {
-    // There is currently no way to recover from this condition.
-    NS_RUNTIMEABORT("StackDepth() is 0 in call to RPCChannel::WaitForNotify!");
+  MOZ_ASSERT(gUIThreadId, "InitUIThread was not called!");
+
+  // Re-use sync notification wait code if this channel does not require
+  // Windows message deferral behavior. 
+  if (!(mFlags & REQUIRE_DEFERRED_MESSAGE_PROTECTION)) {
+    return WaitForSyncNotify();
   }
 
-  // Initialize global objects used in deferred messaging.
-  Init();
+  if (!InterruptStackDepth()) {
+    // There is currently no way to recover from this condition.
+    NS_RUNTIMEABORT("StackDepth() is 0 in call to MessageChannel::WaitForNotify!");
+  }
 
-  NS_ASSERTION(mTopFrame && mTopFrame->mRPC,
+  NS_ASSERTION(mFlags & REQUIRE_DEFERRED_MESSAGE_PROTECTION,
+               "Shouldn't be here for channels that don't use message deferral!");
+  NS_ASSERTION(mTopFrame && mTopFrame->mInterrupt,
                "Top frame is not a sync frame!");
 
-  MutexAutoUnlock unlock(mMutex);
+  MonitorAutoUnlock unlock(*mMonitor);
 
-  bool retval = true;
+  bool timedout = false;
 
-  UINT_PTR timerId = NULL;
+  UINT_PTR timerId = 0;
   TimeoutData timeoutData = { 0 };
 
   // windowHook is used as a flag variable for the loop below: if it is set
   // and we start to spin a nested event loop, we need to clear the hook and
   // process deferred/pending messages.
-  // If windowHook is NULL, SyncChannel::IsPumpingMessages should be false.
-  HHOOK windowHook = NULL;
+  // If windowHook is nullptr, MessageChannel::IsPumpingMessages should be false.
+  HHOOK windowHook = nullptr;
 
   while (1) {
-    NS_ASSERTION((!!windowHook) == SyncChannel::IsPumpingMessages(),
+    NS_ASSERTION((!!windowHook) == MessageChannel::IsPumpingMessages(),
                  "windowHook out of sync with reality");
 
     if (mTopFrame->mSpinNestedEvents) {
       if (windowHook) {
         UnhookWindowsHookEx(windowHook);
-        windowHook = NULL;
+        windowHook = nullptr;
 
         if (timerId) {
-          KillTimer(NULL, timerId);
-          timerId = NULL;
+          KillTimer(nullptr, timerId);
+          timerId = 0;
         }
 
         // Used by widget to assert on incoming native events
-        SyncChannel::SetIsPumpingMessages(false);
+        MessageChannel::SetIsPumpingMessages(false);
 
         // Unhook any neutered windows procedures so messages can be delievered
         // normally.
@@ -854,16 +965,16 @@ RPCChannel::WaitForNotify()
     }
 
     if (!windowHook) {
-      SyncChannel::SetIsPumpingMessages(true);
+      MessageChannel::SetIsPumpingMessages(true);
       windowHook = SetWindowsHookEx(WH_CALLWNDPROC, CallWindowProcedureHook,
-                                    NULL, gUIThreadId);
+                                    nullptr, gUIThreadId);
       NS_ASSERTION(windowHook, "Failed to set hook!");
 
       NS_ASSERTION(!timerId, "Timer already initialized?");
 
       if (mTimeoutMs != kNoTimeout) {
         InitTimeoutData(&timeoutData, mTimeoutMs);
-        timerId = SetTimer(NULL, 0, mTimeoutMs, NULL);
+        timerId = SetTimer(nullptr, 0, mTimeoutMs, nullptr);
         NS_ASSERTION(timerId, "SetTimer failed!");
       }
     }
@@ -872,7 +983,7 @@ RPCChannel::WaitForNotify()
 
     // Don't get wrapped up in here if the child connection dies.
     {
-      MutexAutoLock lock(mMutex);
+      MonitorAutoLock lock(*mMonitor);
       if (!Connected()) {
         break;
       }
@@ -892,17 +1003,17 @@ RPCChannel::WaitForNotify()
 
     if (TimeoutHasExpired(timeoutData)) {
       // A timeout was specified and we've passed it. Break out.
-      retval = false;
+      timedout = true;
       break;
     }
 
-    // See SyncChannel's WaitForNotify for details.
+    // See MessageChannel's WaitFor*Notify for details.
     bool haveSentMessagesPending =
       (HIWORD(GetQueueStatus(QS_SENDMESSAGE)) & QS_SENDMESSAGE) != 0;
 
     // PeekMessage markes the messages as "old" so that they don't wake up
     // MsgWaitForMultipleObjects every time.
-    if (!PeekMessageW(&msg, NULL, 0, 0, PM_NOREMOVE) &&
+    if (!PeekMessageW(&msg, nullptr, 0, 0, PM_NOREMOVE) &&
         !haveSentMessagesPending) {
       // Message was for child, we should wait a bit.
       SwitchToThread();
@@ -924,19 +1035,25 @@ RPCChannel::WaitForNotify()
     ScheduleDeferredMessageRun();
 
     if (timerId) {
-      KillTimer(NULL, timerId);
+      KillTimer(nullptr, timerId);
     }
   }
 
-  SyncChannel::SetIsPumpingMessages(false);
+  MessageChannel::SetIsPumpingMessages(false);
 
-  return retval;
+  return WaitResponse(timedout);
 }
 
 void
-SyncChannel::NotifyWorkerThread()
+MessageChannel::NotifyWorkerThread()
 {
-  mMutex.AssertCurrentThreadOwns();
+  mMonitor->AssertCurrentThreadOwns();
+
+  if (mIsSyncWaitingOnNonMainThread) {
+    mMonitor->Notify();
+    return;
+  }
+
   NS_ASSERTION(mEvent, "No signal event to set, this is really bad!");
   if (!SetEvent(mEvent)) {
     NS_WARNING("Failed to set NotifyWorkerThread event!");
@@ -974,23 +1091,34 @@ DeferredRedrawMessage::Run()
 #ifdef DEBUG
   BOOL ret =
 #endif
-  RedrawWindow(hWnd, NULL, NULL, flags);
+  RedrawWindow(hWnd, nullptr, nullptr, flags);
   NS_ASSERTION(ret, "RedrawWindow failed!");
 }
 
+DeferredUpdateMessage::DeferredUpdateMessage(HWND aHWnd)
+{
+  mWnd = aHWnd;
+  if (!GetUpdateRect(mWnd, &mUpdateRect, FALSE)) {
+    memset(&mUpdateRect, 0, sizeof(RECT));
+    return;
+  }
+  ValidateRect(mWnd, &mUpdateRect);
+}
+
 void
 DeferredUpdateMessage::Run()
 {
-  AssertWindowIsNotNeutered(hWnd);
-  if (!IsWindow(hWnd)) {
+  AssertWindowIsNotNeutered(mWnd);
+  if (!IsWindow(mWnd)) {
     NS_ERROR("Invalid window!");
     return;
   }
 
+  InvalidateRect(mWnd, &mUpdateRect, FALSE);
 #ifdef DEBUG
   BOOL ret =
 #endif
-  UpdateWindow(hWnd);
+  UpdateWindow(mWnd);
   NS_ASSERTION(ret, "UpdateWindow failed!");
 }
 
@@ -1006,8 +1134,8 @@ DeferredSettingChangeMessage::DeferredSettingChangeMessage(HWND aHWnd,
     lParam = reinterpret_cast<LPARAM>(lParamString);
   }
   else {
-    lParamString = NULL;
-    lParam = NULL;
+    lParamString = nullptr;
+    lParam = 0;
   }
 }
 
@@ -1031,7 +1159,7 @@ DeferredWindowPosMessage::DeferredWindowPosMessage(HWND aHWnd,
     else {
       RECT* arg = reinterpret_cast<RECT*>(aLParam);
       windowPos.hwnd = aHWnd;
-      windowPos.hwndInsertAfter = NULL;
+      windowPos.hwndInsertAfter = nullptr;
       windowPos.x = arg->left;
       windowPos.y = arg->top;
       windowPos.cx = arg->right - arg->left;
@@ -1109,7 +1237,7 @@ DeferredCopyDataMessage::DeferredCopyDataMessage(HWND aHWnd,
     }
   }
   else {
-    copyData.lpData = NULL;
+    copyData.lpData = nullptr;
   }
 
   lParam = reinterpret_cast<LPARAM>(&copyData);
