@@ -1,17 +1,21 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=99:
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "tests.h"
-#include "jsdbgapi.h"
+#include "js/OldDebugAPI.h"
+#include "jsapi-tests/tests.h"
 
 static int emptyTrapCallCount = 0;
 
 static JSTrapStatus
 EmptyTrapHandler(JSContext *cx, JSScript *script, jsbytecode *pc, jsval *rval,
-                 jsval closure)
+                 jsval closureArg)
 {
-    JS_GC(cx);
+    JS::RootedValue closure(cx, closureArg);
+    JS_GC(JS_GetRuntime(cx));
     if (JSVAL_IS_STRING(closure))
         ++emptyTrapCallCount;
     return JSTRAP_CONTINUE;
@@ -30,49 +34,49 @@ BEGIN_TEST(testTrap_gc)
         ;
 
     // compile
-    JSObject *scriptObj = JS_CompileScript(cx, global, source, strlen(source), __FILE__, 1);
-    CHECK(scriptObj);
+    JS::CompileOptions options(cx);
+    options.setFileAndLine(__FILE__, 1);
+    JS::RootedScript script(cx, JS_CompileScript(cx, global, source,
+                                                 strlen(source), options));
+    CHECK(script);
 
     // execute
-    jsvalRoot v2(cx);
-    CHECK(JS_ExecuteScript(cx, global, scriptObj, v2.addr()));
-    CHECK(JSVAL_IS_OBJECT(v2));
+    JS::RootedValue v2(cx);
+    CHECK(JS_ExecuteScript(cx, global, script, &v2));
+    CHECK(v2.isObject());
     CHECK_EQUAL(emptyTrapCallCount, 0);
 
-    // Disable JIT for debugging
-    JS_SetOptions(cx, JS_GetOptions(cx) & ~JSOPTION_JIT);
-
     // Enable debug mode
-    CHECK(JS_SetDebugMode(cx, JS_TRUE));
+    CHECK(JS_SetDebugMode(cx, true));
 
     static const char trapClosureText[] = "some trap closure";
 
     // scope JSScript  usage to make sure that it is not used after
     // JS_ExecuteScript. This way we avoid using Anchor.
-    JSString *trapClosure;
+    JS::RootedString trapClosure(cx);
     {
-        JSScript *script = JS_GetScriptFromObject(scriptObj);
         jsbytecode *line2 = JS_LineNumberToPC(cx, script, 1);
         CHECK(line2);
-        
+
         jsbytecode *line6 = JS_LineNumberToPC(cx, script, 5);
         CHECK(line2);
-        
+
         trapClosure = JS_NewStringCopyZ(cx, trapClosureText);
         CHECK(trapClosure);
-        JS_SetTrap(cx, script, line2, EmptyTrapHandler, STRING_TO_JSVAL(trapClosure));
-        JS_SetTrap(cx, script, line6, EmptyTrapHandler, STRING_TO_JSVAL(trapClosure));
-        
-        JS_GC(cx);
-        
+        JS::RootedValue closureValue(cx, JS::StringValue(trapClosure));
+        JS_SetTrap(cx, script, line2, EmptyTrapHandler, closureValue);
+        JS_SetTrap(cx, script, line6, EmptyTrapHandler, closureValue);
+
+        JS_GC(rt);
+
         CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(trapClosure), trapClosureText));
     }
 
     // execute
-    CHECK(JS_ExecuteScript(cx, global, scriptObj, v2.addr()));
+    CHECK(JS_ExecuteScript(cx, global, script, &v2));
     CHECK_EQUAL(emptyTrapCallCount, 11);
 
-    JS_GC(cx);
+    JS_GC(rt);
 
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(trapClosure), trapClosureText));
 
