@@ -1,42 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Steve Clark <buster@netscape.com>
- *   Robert O'Callahan <roc+moz@cs.cmu.edu>
- *   L. David Baron <dbaron@dbaron.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* state used in reflow of block frames */
 
@@ -45,8 +10,11 @@
 
 #include "nsFloatManager.h"
 #include "nsLineBox.h"
-#include "nsFrameList.h"
-#include "nsBlockFrame.h"
+#include "nsHTMLReflowState.h"
+
+class nsBlockFrame;
+class nsFrameList;
+class nsOverflowContinuationTracker;
 
   // block reflow state flags
 #define BRS_UNCONSTRAINEDHEIGHT   0x00000001
@@ -71,9 +39,9 @@ public:
   nsBlockReflowState(const nsHTMLReflowState& aReflowState,
                      nsPresContext* aPresContext,
                      nsBlockFrame* aFrame,
-                     const nsHTMLReflowMetrics& aMetrics,
-                     PRBool aTopMarginRoot, PRBool aBottomMarginRoot,
-                     PRBool aBlockNeedsFloatManager);
+                     bool aTopMarginRoot, bool aBottomMarginRoot,
+                     bool aBlockNeedsFloatManager,
+                     nscoord aConsumedHeight = NS_INTRINSICSIZE);
 
   /**
    * Get the available reflow space (the area not occupied by floats)
@@ -87,7 +55,7 @@ public:
   nsFlowAreaRect GetFloatAvailableSpace() const
     { return GetFloatAvailableSpace(mY); }
   nsFlowAreaRect GetFloatAvailableSpace(nscoord aY) const
-    { return GetFloatAvailableSpaceWithState(aY, nsnull); }
+    { return GetFloatAvailableSpaceWithState(aY, nullptr); }
   nsFlowAreaRect
     GetFloatAvailableSpaceWithState(nscoord aY,
                                     nsFloatManager::SavedState *aState) const;
@@ -96,20 +64,20 @@ public:
                                     nsFloatManager::SavedState *aState) const;
 
   /*
-   * The following functions all return PR_TRUE if they were able to
-   * place the float, PR_FALSE if the float did not fit in available
+   * The following functions all return true if they were able to
+   * place the float, false if the float did not fit in available
    * space.
    * aLineLayout is null when we are reflowing pushed floats (because
    * they are not associated with a line box).
    */
-  PRBool AddFloat(nsLineLayout*       aLineLayout,
+  bool AddFloat(nsLineLayout*       aLineLayout,
                   nsIFrame*           aFloat,
                   nscoord             aAvailableWidth);
 private:
-  PRBool CanPlaceFloat(nscoord aFloatWidth,
+  bool CanPlaceFloat(nscoord aFloatWidth,
                        const nsFlowAreaRect& aFloatAvailableSpace);
 public:
-  PRBool FlowAndPlaceFloat(nsIFrame* aFloat);
+  bool FlowAndPlaceFloat(nsIFrame* aFloat);
 private:
   void PushFloatPastBreak(nsIFrame* aFloat);
 public:
@@ -119,13 +87,13 @@ public:
   // Returns the first coordinate >= aY that clears the
   // floats indicated by aBreakType and has enough width between floats
   // (or no floats remaining) to accomodate aReplacedBlock.
-  nscoord ClearFloats(nscoord aY, PRUint8 aBreakType,
-                      nsIFrame *aReplacedBlock = nsnull,
-                      PRUint32 aFlags = 0);
+  nscoord ClearFloats(nscoord aY, uint8_t aBreakType,
+                      nsIFrame *aReplacedBlock = nullptr,
+                      uint32_t aFlags = 0);
 
-  PRBool IsAdjacentWithTop() const {
+  bool IsAdjacentWithTop() const {
     return mY ==
-      ((mFlags & BRS_ISFIRSTINFLOW) ? mReflowState.mComputedBorderPadding.top : 0);
+      ((mFlags & BRS_ISFIRSTINFLOW) ? mReflowState.ComputedPhysicalBorderPadding().top : 0);
   }
 
   /**
@@ -133,7 +101,7 @@ public:
    * we are not the first in flow.
    */
   nsMargin BorderPadding() const {
-    nsMargin result = mReflowState.mComputedBorderPadding;
+    nsMargin result = mReflowState.ComputedPhysicalBorderPadding();
     if (!(mFlags & BRS_ISFIRSTINFLOW)) {
       result.top = 0;
       if (mFlags & BRS_ISOVERFLOWCONTAINER) {
@@ -143,29 +111,26 @@ public:
     return result;
   }
 
-  // XXX maybe we should do the same adjustment for continuations here
-  const nsMargin& Margin() const {
-    return mReflowState.mComputedMargin;
-  }
+  /**
+   * Retrieve the height "consumed" by any previous-in-flows.
+   */
+  nscoord GetConsumedHeight();
 
   // Reconstruct the previous bottom margin that goes above |aLine|.
   void ReconstructMarginAbove(nsLineList::iterator aLine);
 
   // Caller must have called GetAvailableSpace for the correct position
-  // (which need not be the current mY).  Callers need only pass
-  // aReplacedWidth for outer table frames.
+  // (which need not be the current mY).
   void ComputeReplacedBlockOffsetsForFloats(nsIFrame* aFrame,
                                             const nsRect& aFloatAvailableSpace,
                                             nscoord& aLeftResult,
-                                            nscoord& aRightResult,
-                                       nsBlockFrame::ReplacedElementWidthToClear
-                                                      *aReplacedWidth = nsnull);
+                                            nscoord& aRightResult);
 
   // Caller must have called GetAvailableSpace for the current mY
   void ComputeBlockAvailSpace(nsIFrame* aFrame,
                               const nsStyleDisplay* aDisplay,
                               const nsFlowAreaRect& aFloatAvailableSpace,
-                              PRBool aBlockAvoidsFloats,
+                              bool aBlockAvoidsFloats,
                               nsRect& aResult);
 
 protected:
@@ -176,16 +141,12 @@ public:
 
   void AdvanceToNextLine() {
     if (GetFlag(BRS_LINE_LAYOUT_EMPTY)) {
-      SetFlag(BRS_LINE_LAYOUT_EMPTY, PR_FALSE);
+      SetFlag(BRS_LINE_LAYOUT_EMPTY, false);
     } else {
       mLineNumber++;
     }
   }
 
-  nsLineBox* NewLineBox(nsIFrame* aFrame, PRInt32 aCount, PRBool aIsBlock);
-
-  void FreeLineBox(nsLineBox* aLine);
-
   //----------------------------------------
 
   // This state is the "global" state computed once for the reflow of
@@ -227,6 +188,7 @@ public:
   // mContentArea.height is not NS_UNCONSTRAINEDSIZE; otherwise
   // coordinate overflow may occur.
   nsRect mContentArea;
+  nscoord mContainerWidth;
 
   // Continuation out-of-flow float frames that need to move to our
   // next in flow are placed here during reflow.  It's a pointer to
@@ -236,11 +198,7 @@ public:
   // StealFrame. Call it before adding any frames to mPushedFloats.
   void SetupPushedFloatList();
   // Use this method to append to mPushedFloats.
-  void AppendPushedFloat(nsIFrame* aFloatCont) {
-    SetupPushedFloatList();
-    aFloatCont->AddStateBits(NS_FRAME_IS_PUSHED_FLOAT);
-    mPushedFloats->AppendFrame(mBlock, aFloatCont);
-  }
+  void AppendPushedFloat(nsIFrame* aFloatCont);
 
   // Track child overflow continuations.
   nsOverflowContinuationTracker* mOverflowTracker;
@@ -300,16 +258,18 @@ public:
 
   nscoord mMinLineHeight;
 
-  PRInt32 mLineNumber;
+  int32_t mLineNumber;
 
-  PRInt16 mFlags;
+  int16_t mFlags;
  
-  PRUint8 mFloatBreakType;
+  uint8_t mFloatBreakType;
 
-  void SetFlag(PRUint32 aFlag, PRBool aValue)
+  // The amount of computed height "consumed" by previous-in-flows.
+  nscoord mConsumedHeight;
+
+  void SetFlag(uint32_t aFlag, bool aValue)
   {
     NS_ASSERTION(aFlag<=BRS_LASTFLAG, "bad flag");
-    NS_ASSERTION(aValue==PR_FALSE || aValue==PR_TRUE, "bad value");
     if (aValue) { // set flag
       mFlags |= aFlag;
     }
@@ -318,7 +278,7 @@ public:
     }
   }
 
-  PRBool GetFlag(PRUint32 aFlag) const
+  bool GetFlag(uint32_t aFlag) const
   {
     NS_ASSERTION(aFlag<=BRS_LASTFLAG, "bad flag");
     return !!(mFlags & aFlag);
