@@ -1,38 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla.
- *
- * The Initial Developer of the Original Code is IBM Corporation.
- * Portions created by IBM Corporation are Copyright (C) 2003
- * IBM Corporation. All Rights Reserved.
- *
- * Contributor(s):
- *   IBM Corp.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/Mutex.h"
 #include "nsTransportUtils.h"
@@ -51,16 +19,16 @@ class nsTransportStatusEvent;
 class nsTransportEventSinkProxy : public nsITransportEventSink
 {
 public:
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSITRANSPORTEVENTSINK
 
     nsTransportEventSinkProxy(nsITransportEventSink *sink,
                               nsIEventTarget *target,
-                              PRBool coalesceAll)
+                              bool coalesceAll)
         : mSink(sink)
         , mTarget(target)
         , mLock("nsTransportEventSinkProxy.mLock")
-        , mLastEvent(nsnull)
+        , mLastEvent(nullptr)
         , mCoalesceAll(coalesceAll)
     {
         NS_ADDREF(mSink);
@@ -77,7 +45,7 @@ public:
     nsCOMPtr<nsIEventTarget>         mTarget;
     Mutex                            mLock;
     nsTransportStatusEvent          *mLastEvent;
-    PRBool                           mCoalesceAll;
+    bool                             mCoalesceAll;
 };
 
 class nsTransportStatusEvent : public nsRunnable
@@ -86,8 +54,8 @@ public:
     nsTransportStatusEvent(nsTransportEventSinkProxy *proxy,
                            nsITransport *transport,
                            nsresult status,
-                           PRUint64 progress,
-                           PRUint64 progressMax)
+                           uint64_t progress,
+                           uint64_t progressMax)
         : mProxy(proxy)
         , mTransport(transport)
         , mStatus(status)
@@ -104,12 +72,12 @@ public:
         {
             MutexAutoLock lock(mProxy->mLock);
             if (mProxy->mLastEvent == this)
-                mProxy->mLastEvent = nsnull;
+                mProxy->mLastEvent = nullptr;
         }
 
         mProxy->mSink->OnTransportStatus(mTransport, mStatus, mProgress,
                                          mProgressMax);
-        return nsnull;
+        return NS_OK;
     }
 
     nsRefPtr<nsTransportEventSinkProxy> mProxy;
@@ -117,17 +85,17 @@ public:
     // parameters to OnTransportStatus
     nsCOMPtr<nsITransport> mTransport;
     nsresult               mStatus;
-    PRUint64               mProgress;
-    PRUint64               mProgressMax;
+    uint64_t               mProgress;
+    uint64_t               mProgressMax;
 };
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsTransportEventSinkProxy, nsITransportEventSink)
+NS_IMPL_ISUPPORTS(nsTransportEventSinkProxy, nsITransportEventSink)
 
 NS_IMETHODIMP
 nsTransportEventSinkProxy::OnTransportStatus(nsITransport *transport,
                                              nsresult status,
-                                             PRUint64 progress,
-                                             PRUint64 progressMax)
+                                             uint64_t progress,
+                                             uint64_t progressMax)
 {
     nsresult rv = NS_OK;
     nsRefPtr<nsTransportStatusEvent> event;
@@ -154,7 +122,7 @@ nsTransportEventSinkProxy::OnTransportStatus(nsITransport *transport,
             NS_WARNING("unable to post transport status event");
 
             MutexAutoLock lock(mLock); // cleanup.. don't reference anymore!
-            mLastEvent = nsnull;
+            mLastEvent = nullptr;
         }
     }
     return rv;
@@ -166,7 +134,7 @@ nsresult
 net_NewTransportEventSinkProxy(nsITransportEventSink **result,
                                nsITransportEventSink *sink,
                                nsIEventTarget *target,
-                               PRBool coalesceAll)
+                               bool coalesceAll)
 {
     *result = new nsTransportEventSinkProxy(sink, target, coalesceAll);
     if (!*result)
