@@ -1,51 +1,18 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Radha Kulkarni(radha@netscape.com)
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsWyciwyg.h"
 #include "nsWyciwygChannel.h"
 #include "nsWyciwygProtocolHandler.h"
-#include "nsIURL.h"
-#include "nsIComponentManager.h"
 #include "nsNetCID.h"
 #include "nsServiceManagerUtils.h"
 #include "plstr.h"
 #include "nsNetUtil.h"
+#include "nsIObserverService.h"
+#include "mozIApplicationClearPrivateDataParams.h"
 
 #include "mozilla/net/NeckoChild.h"
 
@@ -61,15 +28,16 @@ nsWyciwygProtocolHandler::nsWyciwygProtocolHandler()
     gWyciwygLog = PR_NewLogModule("nsWyciwygChannel");
 #endif
 
-  LOG(("Creating nsWyciwygProtocolHandler [this=%x].\n", this));
+  LOG(("Creating nsWyciwygProtocolHandler [this=%p].\n", this));
 }
 
 nsWyciwygProtocolHandler::~nsWyciwygProtocolHandler() 
 {
-  LOG(("Deleting nsWyciwygProtocolHandler [this=%x]\n", this));
+  LOG(("Deleting nsWyciwygProtocolHandler [this=%p]\n", this));
 }
 
-NS_IMPL_ISUPPORTS1(nsWyciwygProtocolHandler, nsIProtocolHandler)
+NS_IMPL_ISUPPORTS(nsWyciwygProtocolHandler,
+                  nsIProtocolHandler)
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsIProtocolHandler methods:
@@ -83,16 +51,16 @@ nsWyciwygProtocolHandler::GetScheme(nsACString &result)
 }
 
 NS_IMETHODIMP
-nsWyciwygProtocolHandler::GetDefaultPort(PRInt32 *result) 
+nsWyciwygProtocolHandler::GetDefaultPort(int32_t *result) 
 {
   return NS_ERROR_NOT_AVAILABLE;
 }
 
 NS_IMETHODIMP 
-nsWyciwygProtocolHandler::AllowPort(PRInt32 port, const char *scheme, PRBool *_retval)
+nsWyciwygProtocolHandler::AllowPort(int32_t port, const char *scheme, bool *_retval)
 {
   // don't override anything.  
-  *_retval = PR_FALSE;
+  *_retval = false;
   return NS_OK;
 }
 
@@ -127,7 +95,7 @@ nsWyciwygProtocolHandler::NewChannel(nsIURI* url, nsIChannel* *result)
 
   nsCOMPtr<nsIWyciwygChannel> channel;
   if (IsNeckoChild()) {
-    NS_ENSURE_TRUE(gNeckoChild != nsnull, NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(gNeckoChild != nullptr, NS_ERROR_FAILURE);
 
     WyciwygChannelChild *wcc = static_cast<WyciwygChannelChild *>(
                                  gNeckoChild->SendPWyciwygChannelConstructor());
@@ -142,13 +110,13 @@ nsWyciwygProtocolHandler::NewChannel(nsIURI* url, nsIChannel* *result)
   {
     // If original channel used https, make sure PSM is initialized
     // (this may be first channel to load during a session restore)
-    nsCAutoString path;
+    nsAutoCString path;
     rv = url->GetPath(path);
     NS_ENSURE_SUCCESS(rv, rv);
-    PRInt32 slashIndex = path.FindChar('/', 2);
+    int32_t slashIndex = path.FindChar('/', 2);
     if (slashIndex == kNotFound)
       return NS_ERROR_FAILURE;
-    if (path.Length() < (PRUint32)slashIndex + 1 + 5)
+    if (path.Length() < (uint32_t)slashIndex + 1 + 5)
       return NS_ERROR_FAILURE;
     if (!PL_strncasecmp(path.get() + slashIndex + 1, "https", 5))
       net_EnsurePSMInit();
@@ -161,12 +129,12 @@ nsWyciwygProtocolHandler::NewChannel(nsIURI* url, nsIChannel* *result)
   if (NS_FAILED(rv))
     return rv;
 
-  *result = channel.forget().get();
+  channel.forget(result);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsWyciwygProtocolHandler::GetProtocolFlags(PRUint32 *result) 
+nsWyciwygProtocolHandler::GetProtocolFlags(uint32_t *result) 
 {
   // Should this be an an nsINestedURI?  We don't really want random webpages
   // loading these URIs...
