@@ -21,6 +21,7 @@
  */
 
 HTML5_ATOM(emptystring, "")
+HTML5_ATOM(template_, "template")
 HTML5_ATOM(title, "title")
 HTML5_ATOM(textarea, "textarea")
 HTML5_ATOM(style, "style")
@@ -31,6 +32,8 @@ HTML5_ATOM(noframes, "noframes")
 HTML5_ATOM(noscript, "noscript")
 HTML5_ATOM(plaintext, "plaintext")
 HTML5_ATOM(script, "script")
+HTML5_ATOM(svg, "svg")
+HTML5_ATOM(table, "table")
 HTML5_ATOM(caption, "caption")
 HTML5_ATOM(p, "p")
 HTML5_ATOM(address, "address")
@@ -42,7 +45,9 @@ HTML5_ATOM(option, "option")
 HTML5_ATOM(ruby, "ruby")
 HTML5_ATOM(select, "select")
 HTML5_ATOM(optgroup, "optgroup")
-HTML5_ATOM(table, "table")
+HTML5_ATOM(tbody, "tbody")
+HTML5_ATOM(tfoot, "tfoot")
+HTML5_ATOM(thead, "thead")
 HTML5_ATOM(frameset, "frameset")
 HTML5_ATOM(button, "button")
 HTML5_ATOM(ul, "ul")
@@ -51,9 +56,6 @@ HTML5_ATOM(html, "html")
 HTML5_ATOM(td, "td")
 HTML5_ATOM(th, "th")
 HTML5_ATOM(tr, "tr")
-HTML5_ATOM(tbody, "tbody")
-HTML5_ATOM(thead, "thead")
-HTML5_ATOM(tfoot, "tfoot")
 HTML5_ATOM(colgroup, "colgroup")
 HTML5_ATOM(head, "head")
 HTML5_ATOM(body, "body")
@@ -128,6 +130,7 @@ HTML5_ATOM(mode, "mode")
 HTML5_ATOM(mask, "mask")
 HTML5_ATOM(link, "link")
 HTML5_ATOM(lang, "lang")
+HTML5_ATOM(loop, "loop")
 HTML5_ATOM(list, "list")
 HTML5_ATOM(type, "type")
 HTML5_ATOM(when, "when")
@@ -223,6 +226,7 @@ HTML5_ATOM(vspace, "vspace")
 HTML5_ATOM(poster, "poster")
 HTML5_ATOM(points, "points")
 HTML5_ATOM(prompt, "prompt")
+HTML5_ATOM(srcdoc, "srcdoc")
 HTML5_ATOM(scoped, "scoped")
 HTML5_ATOM(string, "string")
 HTML5_ATOM(scheme, "scheme")
@@ -271,7 +275,6 @@ HTML5_ATOM(opacity, "opacity")
 HTML5_ATOM(noshade, "noshade")
 HTML5_ATOM(minsize, "minsize")
 HTML5_ATOM(maxsize, "maxsize")
-HTML5_ATOM(loopend, "loopend")
 HTML5_ATOM(largeop, "largeop")
 HTML5_ATOM(unicode_, "unicode")
 HTML5_ATOM(targetx, "targetx")
@@ -330,7 +333,6 @@ HTML5_ATOM(mathsize, "mathsize")
 HTML5_ATOM(multiple, "multiple")
 HTML5_ATOM(longdesc, "longdesc")
 HTML5_ATOM(language, "language")
-HTML5_ATOM(template_, "template")
 HTML5_ATOM(tabindex, "tabindex")
 HTML5_ATOM(readonly, "readonly")
 HTML5_ATOM(selected, "selected")
@@ -379,7 +381,6 @@ HTML5_ATOM(maskunits, "maskunits")
 HTML5_ATOM(maskUnits, "maskUnits")
 HTML5_ATOM(maxlength, "maxlength")
 HTML5_ATOM(linebreak, "linebreak")
-HTML5_ATOM(loopstart, "loopstart")
 HTML5_ATOM(transform, "transform")
 HTML5_ATOM(v_hanging, "v-hanging")
 HTML5_ATOM(valuetype, "valuetype")
@@ -389,7 +390,6 @@ HTML5_ATOM(pointsatx, "pointsatx")
 HTML5_ATOM(pointsAtX, "pointsAtX")
 HTML5_ATOM(pointsaty, "pointsaty")
 HTML5_ATOM(pointsAtY, "pointsAtY")
-HTML5_ATOM(playcount, "playcount")
 HTML5_ATOM(symmetric, "symmetric")
 HTML5_ATOM(scrolling, "scrolling")
 HTML5_ATOM(repeatdur, "repeatdur")
@@ -458,6 +458,7 @@ HTML5_ATOM(arabic_form, "arabic-form")
 HTML5_ATOM(cellpadding, "cellpadding")
 HTML5_ATOM(cellspacing, "cellspacing")
 HTML5_ATOM(columnwidth, "columnwidth")
+HTML5_ATOM(crossorigin, "crossorigin")
 HTML5_ATOM(columnalign, "columnalign")
 HTML5_ATOM(columnlines, "columnlines")
 HTML5_ATOM(contextmenu, "contextmenu")
@@ -770,7 +771,6 @@ HTML5_ATOM(pre, "pre")
 HTML5_ATOM(rem, "rem")
 HTML5_ATOM(sub, "sub")
 HTML5_ATOM(sec, "sec")
-HTML5_ATOM(svg, "svg")
 HTML5_ATOM(sum, "sum")
 HTML5_ATOM(sin, "sin")
 HTML5_ATOM(sep, "sep")
@@ -799,6 +799,7 @@ HTML5_ATOM(msub, "msub")
 HTML5_ATOM(math, "math")
 HTML5_ATOM(mark, "mark")
 HTML5_ATOM(mean, "mean")
+HTML5_ATOM(main, "main")
 HTML5_ATOM(msup, "msup")
 HTML5_ATOM(menu, "menu")
 HTML5_ATOM(mrow, "mrow")
@@ -906,7 +907,6 @@ HTML5_ATOM(arccosh, "arccosh")
 HTML5_ATOM(arccoth, "arccoth")
 HTML5_ATOM(acronym, "acronym")
 HTML5_ATOM(bgsound, "bgsound")
-HTML5_ATOM(command, "command")
 HTML5_ATOM(compose, "compose")
 HTML5_ATOM(ceiling, "ceiling")
 HTML5_ATOM(csymbol, "csymbol")
@@ -964,6 +964,7 @@ HTML5_ATOM(listener, "listener")
 HTML5_ATOM(lowlimit, "lowlimit")
 HTML5_ATOM(metadata, "metadata")
 HTML5_ATOM(menclose, "menclose")
+HTML5_ATOM(menuitem, "menuitem")
 HTML5_ATOM(mphantom, "mphantom")
 HTML5_ATOM(polyline, "polyline")
 HTML5_ATOM(prefetch, "prefetch")
@@ -1026,6 +1027,8 @@ HTML5_ATOM(feturbulence, "feturbulence")
 HTML5_ATOM(feTurbulence, "feTurbulence")
 HTML5_ATOM(fepointlight, "fepointlight")
 HTML5_ATOM(fePointLight, "fePointLight")
+HTML5_ATOM(fedropshadow, "fedropshadow")
+HTML5_ATOM(feDropShadow, "feDropShadow")
 HTML5_ATOM(femorphology, "femorphology")
 HTML5_ATOM(feMorphology, "feMorphology")
 HTML5_ATOM(outerproduct, "outerproduct")
