@@ -1,46 +1,16 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Red Hat, Inc.
- * Portions created by the Initial Developer are Copyright (C) 2006
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Kai Engert <kengert@redhat.com>
- *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsCertOverrideService.h"
+
+#include "pkix/pkixtypes.h"
 #include "nsIX509Cert.h"
+#include "NSSCertDBTrustDomain.h"
 #include "nsNSSCertificate.h"
+#include "nsNSSCertHelper.h"
 #include "nsCRT.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsStreamUtils.h"
@@ -50,18 +20,19 @@
 #include "nsIObserverService.h"
 #include "nsISupportsPrimitives.h"
 #include "nsPromiseFlatString.h"
-#include "nsProxiedService.h"
+#include "nsThreadUtils.h"
 #include "nsStringBuffer.h"
-#include "nsAutoLock.h"
-#include "nsAutoPtr.h"
+#include "ScopedNSSTypes.h"
+#include "SharedSSLState.h"
+
 #include "nspr.h"
 #include "pk11pub.h"
 #include "certdb.h"
 #include "sechash.h"
 #include "ssl.h" // For SSL_ClearSessionCache
 
-#include "nsNSSCleaner.h"
-NSSCleanupAutoPtrClass(CERTCertificate, CERT_DestroyCertificate)
+using namespace mozilla;
+using namespace mozilla::psm;
 
 static const char kCertOverrideFileName[] = "cert_override.txt";
 
@@ -113,27 +84,27 @@ nsCertOverride::convertStringToBits(const nsACString &str, OverrideBits &ob)
   }
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS3(nsCertOverrideService, 
-                              nsICertOverrideService,
-                              nsIObserver,
-                              nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(nsCertOverrideService,
+                  nsICertOverrideService,
+                  nsIObserver,
+                  nsISupportsWeakReference)
 
 nsCertOverrideService::nsCertOverrideService()
+  : monitor("nsCertOverrideService.monitor")
 {
-  monitor = PR_NewMonitor();
 }
 
 nsCertOverrideService::~nsCertOverrideService()
 {
-  if (monitor)
-    PR_DestroyMonitor(monitor);
 }
 
 nsresult
 nsCertOverrideService::Init()
 {
-  if (!mSettingsTable.Init())
-    return NS_ERROR_OUT_OF_MEMORY;
+  if (!NS_IsMainThread()) {
+    NS_NOTREACHED("nsCertOverrideService initialized off main thread");
+    return NS_ERROR_NOT_SAME_THREAD;
+  }
 
   mOidTagForStoringNewHashes = SEC_OID_SHA256;
 
@@ -148,45 +119,40 @@ nsCertOverrideService::Init()
   mDottedOidForStoringNewHashes = dotted_oid;
   PR_smprintf_free(dotted_oid);
 
-  // cache mSettingsFile
-  NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mSettingsFile));
-  if (mSettingsFile) {
-    mSettingsFile->AppendNative(NS_LITERAL_CSTRING(kCertOverrideFileName));
-  }
+  nsCOMPtr<nsIObserverService> observerService =
+      mozilla::services::GetObserverService();
 
-  Read();
-
-  nsresult rv;
-  NS_WITH_ALWAYS_PROXIED_SERVICE(nsIObserverService, mObserverService,
-                                 "@mozilla.org/observer-service;1",
-                                 NS_PROXY_TO_MAIN_THREAD, &rv);
-
-  if (mObserverService) {
-    mObserverService->AddObserver(this, "profile-before-change", PR_TRUE);
-    mObserverService->AddObserver(this, "profile-do-change", PR_TRUE);
-    mObserverService->AddObserver(this, "shutdown-cleanse", PR_TRUE);
+  // If we cannot add ourselves as a profile change observer, then we will not
+  // attempt to read/write any settings file. Otherwise, we would end up
+  // reading/writing the wrong settings file after a profile change.
+  if (observerService) {
+    observerService->AddObserver(this, "profile-before-change", true);
+    observerService->AddObserver(this, "profile-do-change", true);
+    // simulate a profile change so we read the current profile's settings file
+    Observe(nullptr, "profile-do-change", nullptr);
   }
 
+  SharedSSLState::NoteCertOverrideServiceInstantiated();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::Observe(nsISupports     *aSubject,
+nsCertOverrideService::Observe(nsISupports     *,
                                const char      *aTopic,
-                               const PRUnichar *aData)
+                               const char16_t *aData)
 {
   // check the topic
   if (!nsCRT::strcmp(aTopic, "profile-before-change")) {
     // The profile is about to change,
     // or is going away because the application is shutting down.
 
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
 
-    if (!nsCRT::strcmp(aData, NS_LITERAL_STRING("shutdown-cleanse").get())) {
+    if (!nsCRT::strcmp(aData, MOZ_UTF16("shutdown-cleanse"))) {
       RemoveAllFromMemory();
       // delete the storage file
       if (mSettingsFile) {
-        mSettingsFile->Remove(PR_FALSE);
+        mSettingsFile->Remove(false);
       }
     } else {
       RemoveAllFromMemory();
@@ -197,11 +163,13 @@ nsCertOverrideService::Observe(nsISupports     *aSubject,
     // Now read from the new profile location.
     // we also need to update the cached file location
 
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
 
     nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mSettingsFile));
     if (NS_SUCCEEDED(rv)) {
       mSettingsFile->AppendNative(NS_LITERAL_CSTRING(kCertOverrideFileName));
+    } else {
+      mSettingsFile = nullptr;
     }
     Read();
 
@@ -213,16 +181,16 @@ nsCertOverrideService::Observe(nsISupports     *aSubject,
 void
 nsCertOverrideService::RemoveAllFromMemory()
 {
-  nsAutoMonitor lock(monitor);
+  ReentrantMonitorAutoEnter lock(monitor);
   mSettingsTable.Clear();
 }
 
-PR_STATIC_CALLBACK(PLDHashOperator)
+static PLDHashOperator
 RemoveTemporariesCallback(nsCertOverrideEntry *aEntry,
                           void *aArg)
 {
   if (aEntry && aEntry->mSettings.mIsTemporary) {
-    aEntry->mSettings.mCert = nsnull;
+    aEntry->mSettings.mCert = nullptr;
     return PL_DHASH_REMOVE;
   }
 
@@ -233,8 +201,8 @@ void
 nsCertOverrideService::RemoveAllTemporaryOverrides()
 {
   {
-    nsAutoMonitor lock(monitor);
-    mSettingsTable.EnumerateEntries(RemoveTemporariesCallback, nsnull);
+    ReentrantMonitorAutoEnter lock(monitor);
+    mSettingsTable.EnumerateEntries(RemoveTemporariesCallback, nullptr);
     // no need to write, as temporaries are never written to disk
   }
 }
@@ -242,7 +210,11 @@ nsCertOverrideService::RemoveAllTemporaryOverrides()
 nsresult
 nsCertOverrideService::Read()
 {
-  nsAutoMonitor lock(monitor);
+  ReentrantMonitorAutoEnter lock(monitor);
+
+  // If we don't have a profile, then we won't try to read any settings file.
+  if (!mSettingsFile)
+    return NS_OK;
 
   nsresult rv;
   nsCOMPtr<nsIInputStream> fileInputStream;
@@ -256,9 +228,9 @@ nsCertOverrideService::Read()
     return rv;
   }
 
-  nsCAutoString buffer;
-  PRBool isMore = PR_TRUE;
-  PRInt32 hostIndex = 0, algoIndex, fingerprintIndex, overrideBitsIndex, dbKeyIndex;
+  nsAutoCString buffer;
+  bool isMore = true;
+  int32_t hostIndex = 0, algoIndex, fingerprintIndex, overrideBitsIndex, dbKeyIndex;
 
   /* file format is:
    *
@@ -294,33 +266,33 @@ nsCertOverrideService::Read()
     const nsASingleFragmentCString &bits_string = Substring(buffer, overrideBitsIndex, dbKeyIndex - overrideBitsIndex - 1);
     const nsASingleFragmentCString &db_key = Substring(buffer, dbKeyIndex, buffer.Length() - dbKeyIndex);
 
-    nsCAutoString host(tmp);
+    nsAutoCString host(tmp);
     nsCertOverride::OverrideBits bits;
     nsCertOverride::convertStringToBits(bits_string, bits);
 
-    PRInt32 port;
-    PRInt32 portIndex = host.RFindChar(':');
+    int32_t port;
+    int32_t portIndex = host.RFindChar(':');
     if (portIndex == kNotFound)
       continue; // Ignore broken entries
 
-    PRInt32 portParseError;
-    nsCAutoString portString(Substring(host, portIndex+1));
+    nsresult portParseError;
+    nsAutoCString portString(Substring(host, portIndex+1));
     port = portString.ToInteger(&portParseError);
-    if (portParseError)
+    if (NS_FAILED(portParseError))
       continue; // Ignore broken entries
 
     host.Truncate(portIndex);
     
     AddEntryToList(host, port, 
-                   nsnull, // don't have the cert
-                   PR_FALSE, // not temporary
+                   nullptr, // don't have the cert
+                   false, // not temporary
                    algo_string, fingerprint, bits, db_key);
   }
 
   return NS_OK;
 }
 
-PR_STATIC_CALLBACK(PLDHashOperator)
+static PLDHashOperator
 WriteEntryCallback(nsCertOverrideEntry *aEntry,
                    void *aArg)
 {
@@ -328,7 +300,7 @@ WriteEntryCallback(nsCertOverrideEntry *aEntry,
 
   nsIOutputStream *rawStreamPtr = (nsIOutputStream *)aArg;
 
-  nsresult rv;
+  uint32_t unused;
 
   if (rawStreamPtr && aEntry)
   {
@@ -336,23 +308,23 @@ WriteEntryCallback(nsCertOverrideEntry *aEntry,
     if (settings.mIsTemporary)
       return PL_DHASH_NEXT;
 
-    nsCAutoString bits_string;
+    nsAutoCString bits_string;
     nsCertOverride::convertBitsToString(settings.mOverrideBits, 
                                             bits_string);
 
-    rawStreamPtr->Write(aEntry->mHostWithPort.get(), aEntry->mHostWithPort.Length(), &rv);
-    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &rv);
+    rawStreamPtr->Write(aEntry->mHostWithPort.get(), aEntry->mHostWithPort.Length(), &unused);
+    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &unused);
     rawStreamPtr->Write(settings.mFingerprintAlgOID.get(), 
-                        settings.mFingerprintAlgOID.Length(), &rv);
-    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &rv);
+                        settings.mFingerprintAlgOID.Length(), &unused);
+    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &unused);
     rawStreamPtr->Write(settings.mFingerprint.get(), 
-                        settings.mFingerprint.Length(), &rv);
-    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &rv);
+                        settings.mFingerprint.Length(), &unused);
+    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &unused);
     rawStreamPtr->Write(bits_string.get(), 
-                        bits_string.Length(), &rv);
-    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &rv);
-    rawStreamPtr->Write(settings.mDBKey.get(), settings.mDBKey.Length(), &rv);
-    rawStreamPtr->Write(NS_LINEBREAK, NS_LINEBREAK_LEN, &rv);
+                        bits_string.Length(), &unused);
+    rawStreamPtr->Write(kTab, sizeof(kTab) - 1, &unused);
+    rawStreamPtr->Write(settings.mDBKey.get(), settings.mDBKey.Length(), &unused);
+    rawStreamPtr->Write(NS_LINEBREAK, NS_LINEBREAK_LEN, &unused);
   }
 
   return PL_DHASH_NEXT;
@@ -361,10 +333,11 @@ WriteEntryCallback(nsCertOverrideEntry *aEntry,
 nsresult
 nsCertOverrideService::Write()
 {
-  nsAutoMonitor lock(monitor);
+  ReentrantMonitorAutoEnter lock(monitor);
 
+  // If we don't have any profile, then we won't try to write any file
   if (!mSettingsFile) {
-    return NS_ERROR_NULL_POINTER;
+    return NS_OK;
   }
 
   nsresult rv;
@@ -391,7 +364,8 @@ nsCertOverrideService::Write()
 
   /* see ::Read for file format */
 
-  bufferedOutputStream->Write(kHeader, sizeof(kHeader) - 1, &rv);
+  uint32_t unused;
+  bufferedOutputStream->Write(kHeader, sizeof(kHeader) - 1, &unused);
 
   nsIOutputStream *rawStreamPtr = bufferedOutputStream;
   mSettingsTable.EnumerateEntries(WriteEntryCallback, rawStreamPtr);
@@ -411,30 +385,6 @@ nsCertOverrideService::Write()
   return NS_OK;
 }
 
-static nsresult
-GetCertFingerprintByOidTag(CERTCertificate* nsscert,
-                           SECOidTag aOidTag, 
-                           nsCString &fp)
-{
-  unsigned int hash_len = HASH_ResultLenByOidTag(aOidTag);
-  nsStringBuffer* fingerprint = nsStringBuffer::Alloc(hash_len);
-  if (!fingerprint)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  PK11_HashBuf(aOidTag, (unsigned char*)fingerprint->Data(), 
-               nsscert->derCert.data, nsscert->derCert.len);
-
-  SECItem fpItem;
-  fpItem.data = (unsigned char*)fingerprint->Data();
-  fpItem.len = hash_len;
-
-  char *tmpstr = CERT_Hexify(&fpItem, 1);
-  fp.Assign(tmpstr);
-  PORT_Free(tmpstr);
-  fingerprint->Release();
-  return NS_OK;
-}
-
 static nsresult
 GetCertFingerprintByOidTag(nsIX509Cert *aCert,
                            SECOidTag aOidTag, 
@@ -444,12 +394,11 @@ GetCertFingerprintByOidTag(nsIX509Cert *aCert,
   if (!cert2)
     return NS_ERROR_FAILURE;
 
-  CERTCertificate* nsscert = cert2->GetCert();
+  mozilla::pkix::ScopedCERTCertificate nsscert(cert2->GetCert());
   if (!nsscert)
     return NS_ERROR_FAILURE;
 
-  CERTCertificateCleaner nsscertCleaner(nsscert);
-  return GetCertFingerprintByOidTag(nsscert, aOidTag, fp);
+  return GetCertFingerprintByOidTag(nsscert.get(), aOidTag, fp);
 }
 
 static nsresult
@@ -458,15 +407,15 @@ GetCertFingerprintByDottedOidString(CERTCertificate* nsscert,
                                     nsCString &fp)
 {
   SECItem oid;
-  oid.data = nsnull;
+  oid.data = nullptr;
   oid.len = 0;
-  SECStatus srv = SEC_StringToOID(nsnull, &oid, 
+  SECStatus srv = SEC_StringToOID(nullptr, &oid, 
                     dottedOid.get(), dottedOid.Length());
   if (srv != SECSuccess)
     return NS_ERROR_FAILURE;
 
   SECOidTag oid_tag = SECOID_FindOIDTag(&oid);
-  SECITEM_FreeItem(&oid, PR_FALSE);
+  SECITEM_FreeItem(&oid, false);
 
   if (oid_tag == SEC_OID_UNKNOWN)
     return NS_ERROR_FAILURE;
@@ -483,19 +432,18 @@ GetCertFingerprintByDottedOidString(nsIX509Cert *aCert,
   if (!cert2)
     return NS_ERROR_FAILURE;
 
-  CERTCertificate* nsscert = cert2->GetCert();
+  mozilla::pkix::ScopedCERTCertificate nsscert(cert2->GetCert());
   if (!nsscert)
     return NS_ERROR_FAILURE;
 
-  CERTCertificateCleaner nsscertCleaner(nsscert);
-  return GetCertFingerprintByDottedOidString(nsscert, dottedOid, fp);
+  return GetCertFingerprintByDottedOidString(nsscert.get(), dottedOid, fp);
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::RememberValidityOverride(const nsACString & aHostName, PRInt32 aPort, 
+nsCertOverrideService::RememberValidityOverride(const nsACString & aHostName, int32_t aPort, 
                                                 nsIX509Cert *aCert,
-                                                PRUint32 aOverrideBits, 
-                                                PRBool aTemporary)
+                                                uint32_t aOverrideBits, 
+                                                bool aTemporary)
 {
   NS_ENSURE_ARG_POINTER(aCert);
   if (aHostName.IsEmpty())
@@ -507,35 +455,35 @@ nsCertOverrideService::RememberValidityOverride(const nsACString & aHostName, PR
   if (!cert2)
     return NS_ERROR_FAILURE;
 
-  CERTCertificate* nsscert = cert2->GetCert();
+  mozilla::pkix::ScopedCERTCertificate nsscert(cert2->GetCert());
   if (!nsscert)
     return NS_ERROR_FAILURE;
 
-  CERTCertificateCleaner nsscertCleaner(nsscert);
-
-  nsCAutoString nickname;
-  nickname = nsNSSCertificate::defaultServerNickname(nsscert);
-  if (!aTemporary && !nickname.IsEmpty())
+  char* nickname = DefaultServerNicknameForCert(nsscert.get());
+  if (!aTemporary && nickname && *nickname)
   {
-    PK11SlotInfo *slot = PK11_GetInternalKeySlot();
-    if (!slot)
+    ScopedPK11SlotInfo slot(PK11_GetInternalKeySlot());
+    if (!slot) {
+      PR_Free(nickname);
       return NS_ERROR_FAILURE;
+    }
   
-    SECStatus srv = PK11_ImportCert(slot, nsscert, CK_INVALID_HANDLE, 
-                                    const_cast<char*>(nickname.get()), PR_FALSE);
-    PK11_FreeSlot(slot);
-  
-    if (srv != SECSuccess)
+    SECStatus srv = PK11_ImportCert(slot, nsscert.get(), CK_INVALID_HANDLE,
+                                    nickname, false);
+    if (srv != SECSuccess) {
+      PR_Free(nickname);
       return NS_ERROR_FAILURE;
+    }
   }
+  PR_FREEIF(nickname);
 
-  nsCAutoString fpStr;
-  nsresult rv = GetCertFingerprintByOidTag(nsscert, 
+  nsAutoCString fpStr;
+  nsresult rv = GetCertFingerprintByOidTag(nsscert.get(),
                   mOidTagForStoringNewHashes, fpStr);
   if (NS_FAILED(rv))
     return rv;
 
-  char *dbkey = NULL;
+  char *dbkey = nullptr;
   rv = aCert->GetDbKey(&dbkey);
   if (NS_FAILED(rv) || !dbkey)
     return rv;
@@ -551,9 +499,9 @@ nsCertOverrideService::RememberValidityOverride(const nsACString & aHostName, PR
   }
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     AddEntryToList(aHostName, aPort,
-                   aTemporary ? aCert : nsnull,
+                   aTemporary ? aCert : nullptr,
                      // keep a reference to the cert for temporary overrides
                    aTemporary, 
                    mDottedOidForStoringNewHashes, fpStr, 
@@ -567,11 +515,11 @@ nsCertOverrideService::RememberValidityOverride(const nsACString & aHostName, PR
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::HasMatchingOverride(const nsACString & aHostName, PRInt32 aPort,
+nsCertOverrideService::HasMatchingOverride(const nsACString & aHostName, int32_t aPort,
                                            nsIX509Cert *aCert, 
-                                           PRUint32 *aOverrideBits,
-                                           PRBool *aIsTemporary,
-                                           PRBool *_retval)
+                                           uint32_t *aOverrideBits,
+                                           bool *aIsTemporary,
+                                           bool *_retval)
 {
   if (aHostName.IsEmpty())
     return NS_ERROR_INVALID_ARG;
@@ -582,15 +530,15 @@ nsCertOverrideService::HasMatchingOverride(const nsACString & aHostName, PRInt32
   NS_ENSURE_ARG_POINTER(aOverrideBits);
   NS_ENSURE_ARG_POINTER(aIsTemporary);
   NS_ENSURE_ARG_POINTER(_retval);
-  *_retval = PR_FALSE;
+  *_retval = false;
   *aOverrideBits = nsCertOverride::ob_None;
 
-  nsCAutoString hostPort;
+  nsAutoCString hostPort;
   GetHostWithPort(aHostName, aPort, hostPort);
   nsCertOverride settings;
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     nsCertOverrideEntry *entry = mSettingsTable.GetEntry(hostPort.get());
   
     if (!entry)
@@ -602,7 +550,7 @@ nsCertOverrideService::HasMatchingOverride(const nsACString & aHostName, PRInt32
   *aOverrideBits = settings.mOverrideBits;
   *aIsTemporary = settings.mIsTemporary;
 
-  nsCAutoString fpStr;
+  nsAutoCString fpStr;
   nsresult rv;
 
   if (settings.mFingerprintAlgOID.Equals(mDottedOidForStoringNewHashes)) {
@@ -619,29 +567,29 @@ nsCertOverrideService::HasMatchingOverride(const nsACString & aHostName, PRInt32
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::GetValidityOverride(const nsACString & aHostName, PRInt32 aPort,
+nsCertOverrideService::GetValidityOverride(const nsACString & aHostName, int32_t aPort,
                                            nsACString & aHashAlg, 
                                            nsACString & aFingerprint, 
-                                           PRUint32 *aOverrideBits,
-                                           PRBool *aIsTemporary,
-                                           PRBool *_found)
+                                           uint32_t *aOverrideBits,
+                                           bool *aIsTemporary,
+                                           bool *_found)
 {
   NS_ENSURE_ARG_POINTER(_found);
   NS_ENSURE_ARG_POINTER(aIsTemporary);
   NS_ENSURE_ARG_POINTER(aOverrideBits);
-  *_found = PR_FALSE;
+  *_found = false;
   *aOverrideBits = nsCertOverride::ob_None;
 
-  nsCAutoString hostPort;
+  nsAutoCString hostPort;
   GetHostWithPort(aHostName, aPort, hostPort);
   nsCertOverride settings;
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     nsCertOverrideEntry *entry = mSettingsTable.GetEntry(hostPort.get());
   
     if (entry) {
-      *_found = PR_TRUE;
+      *_found = true;
       settings = entry->mSettings; // copy
     }
   }
@@ -657,19 +605,19 @@ nsCertOverrideService::GetValidityOverride(const nsACString & aHostName, PRInt32
 }
 
 nsresult
-nsCertOverrideService::AddEntryToList(const nsACString &aHostName, PRInt32 aPort,
+nsCertOverrideService::AddEntryToList(const nsACString &aHostName, int32_t aPort,
                                       nsIX509Cert *aCert,
-                                      const PRBool aIsTemporary,
+                                      const bool aIsTemporary,
                                       const nsACString &fingerprintAlgOID, 
                                       const nsACString &fingerprint,
                                       nsCertOverride::OverrideBits ob,
                                       const nsACString &dbKey)
 {
-  nsCAutoString hostPort;
+  nsAutoCString hostPort;
   GetHostWithPort(aHostName, aPort, hostPort);
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     nsCertOverrideEntry *entry = mSettingsTable.PutEntry(hostPort.get());
 
     if (!entry) {
@@ -694,12 +642,17 @@ nsCertOverrideService::AddEntryToList(const nsACString &aHostName, PRInt32 aPort
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::ClearValidityOverride(const nsACString & aHostName, PRInt32 aPort)
+nsCertOverrideService::ClearValidityOverride(const nsACString & aHostName, int32_t aPort)
 {
-  nsCAutoString hostPort;
+  if (aPort == 0 &&
+      aHostName.EqualsLiteral("all:temporary-certificates")) {
+    RemoveAllTemporaryOverrides();
+    return NS_OK;
+  }
+  nsAutoCString hostPort;
   GetHostWithPort(aHostName, aPort, hostPort);
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     mSettingsTable.RemoveEntry(hostPort.get());
     Write();
   }
@@ -708,21 +661,21 @@ nsCertOverrideService::ClearValidityOverride(const nsACString & aHostName, PRInt
 }
 
 NS_IMETHODIMP
-nsCertOverrideService::GetAllOverrideHostsWithPorts(PRUint32 *aCount, 
-                                                        PRUnichar ***aHostsWithPortsArray)
+nsCertOverrideService::GetAllOverrideHostsWithPorts(uint32_t *aCount, 
+                                                        char16_t ***aHostsWithPortsArray)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-static PRBool
+static bool
 matchesDBKey(nsIX509Cert *cert, const char *match_dbkey)
 {
-  char *dbkey = NULL;
+  char *dbkey = nullptr;
   nsresult rv = cert->GetDbKey(&dbkey);
   if (NS_FAILED(rv) || !dbkey)
-    return PR_FALSE;
+    return false;
 
-  PRBool found_mismatch = PR_FALSE;
+  bool found_mismatch = false;
   const char *key1 = dbkey;
   const char *key2 = match_dbkey;
 
@@ -750,7 +703,7 @@ matchesDBKey(nsIX509Cert *cert, const char *match_dbkey)
     }
 
     if (c1 != c2) {
-      found_mismatch = PR_TRUE;
+      found_mismatch = true;
       break;
     }
 
@@ -765,15 +718,15 @@ matchesDBKey(nsIX509Cert *cert, const char *match_dbkey)
 struct nsCertAndBoolsAndInt
 {
   nsIX509Cert *cert;
-  PRBool aCheckTemporaries;
-  PRBool aCheckPermanents;
-  PRUint32 counter;
+  bool aCheckTemporaries;
+  bool aCheckPermanents;
+  uint32_t counter;
 
   SECOidTag mOidTagForStoringNewHashes;
   nsCString mDottedOidForStoringNewHashes;
 };
 
-PR_STATIC_CALLBACK(PLDHashOperator)
+static PLDHashOperator
 FindMatchingCertCallback(nsCertOverrideEntry *aEntry,
                          void *aArg)
 {
@@ -782,16 +735,16 @@ FindMatchingCertCallback(nsCertOverrideEntry *aEntry,
   if (cai && aEntry)
   {
     const nsCertOverride &settings = aEntry->mSettings;
-    PRBool still_ok = PR_TRUE;
+    bool still_ok = true;
 
     if ((settings.mIsTemporary && !cai->aCheckTemporaries)
         ||
         (!settings.mIsTemporary && !cai->aCheckPermanents)) {
-      still_ok = PR_FALSE;
+      still_ok = false;
     }
 
     if (still_ok && matchesDBKey(cai->cert, settings.mDBKey.get())) {
-      nsCAutoString cert_fingerprint;
+      nsAutoCString cert_fingerprint;
       nsresult rv;
       if (settings.mFingerprintAlgOID.Equals(cai->mDottedOidForStoringNewHashes)) {
         rv = GetCertFingerprintByOidTag(cai->cert,
@@ -813,9 +766,9 @@ FindMatchingCertCallback(nsCertOverrideEntry *aEntry,
 
 NS_IMETHODIMP
 nsCertOverrideService::IsCertUsedForOverrides(nsIX509Cert *aCert, 
-                                              PRBool aCheckTemporaries,
-                                              PRBool aCheckPermanents,
-                                              PRUint32 *_retval)
+                                              bool aCheckTemporaries,
+                                              bool aCheckPermanents,
+                                              uint32_t *_retval)
 {
   NS_ENSURE_ARG(aCert);
   NS_ENSURE_ARG(_retval);
@@ -829,7 +782,7 @@ nsCertOverrideService::IsCertUsedForOverrides(nsIX509Cert *aCert,
   cai.mDottedOidForStoringNewHashes = mDottedOidForStoringNewHashes;
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     mSettingsTable.EnumerateEntries(FindMatchingCertCallback, &cai);
   }
   *_retval = cai.counter;
@@ -846,7 +799,7 @@ struct nsCertAndPointerAndCallback
   nsCString mDottedOidForStoringNewHashes;
 };
 
-PR_STATIC_CALLBACK(PLDHashOperator)
+static PLDHashOperator
 EnumerateCertOverridesCallback(nsCertOverrideEntry *aEntry,
                                void *aArg)
 {
@@ -861,7 +814,7 @@ EnumerateCertOverridesCallback(nsCertOverrideEntry *aEntry,
     }
     else {
       if (matchesDBKey(capac->cert, settings.mDBKey.get())) {
-        nsCAutoString cert_fingerprint;
+        nsAutoCString cert_fingerprint;
         nsresult rv;
         if (settings.mFingerprintAlgOID.Equals(capac->mDottedOidForStoringNewHashes)) {
           rv = GetCertFingerprintByOidTag(capac->cert,
@@ -895,16 +848,16 @@ nsCertOverrideService::EnumerateCertOverrides(nsIX509Cert *aCert,
   capac.mDottedOidForStoringNewHashes = mDottedOidForStoringNewHashes;
 
   {
-    nsAutoMonitor lock(monitor);
+    ReentrantMonitorAutoEnter lock(monitor);
     mSettingsTable.EnumerateEntries(EnumerateCertOverridesCallback, &capac);
   }
   return NS_OK;
 }
 
 void
-nsCertOverrideService::GetHostWithPort(const nsACString & aHostName, PRInt32 aPort, nsACString& _retval)
+nsCertOverrideService::GetHostWithPort(const nsACString & aHostName, int32_t aPort, nsACString& _retval)
 {
-  nsCAutoString hostPort(aHostName);
+  nsAutoCString hostPort(aHostName);
   if (aPort == -1) {
     aPort = 443;
   }
@@ -914,3 +867,4 @@ nsCertOverrideService::GetHostWithPort(const nsACString & aHostName, PRInt32 aPo
   }
   _retval.Assign(hostPort);
 }
+
