@@ -35,15 +35,22 @@
 #include <string.h>
 
 #include "breakpad_googletest_includes.h"
+#include "common/module.h"
+#include "common/using_std_string.h"
 #include "processor/cfi_frame_info.h"
 #include "google_breakpad/processor/memory_region.h"
 
 using google_breakpad::CFIFrameInfo;
 using google_breakpad::CFIFrameInfoParseHandler;
 using google_breakpad::CFIRuleParser;
+using google_breakpad::FromUniqueString;
 using google_breakpad::MemoryRegion;
+using google_breakpad::Module;
 using google_breakpad::SimpleCFIWalker;
-using std::string;
+using google_breakpad::ToUniqueString;
+using google_breakpad::UniqueString;
+using google_breakpad::ustr__ZDcfa;
+using google_breakpad::ustr__ZDra;
 using testing::_;
 using testing::A;
 using testing::AtMost;
@@ -54,12 +61,12 @@ using testing::Test;
 
 class MockMemoryRegion: public MemoryRegion {
  public:
-  MOCK_CONST_METHOD0(GetBase, u_int64_t());
-  MOCK_CONST_METHOD0(GetSize, u_int32_t());
-  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(u_int64_t, u_int8_t *));
-  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(u_int64_t, u_int16_t *));
-  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(u_int64_t, u_int32_t *));
-  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(u_int64_t, u_int64_t *));
+  MOCK_CONST_METHOD0(GetBase, uint64_t());
+  MOCK_CONST_METHOD0(GetSize, uint32_t());
+  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(uint64_t, uint8_t *));
+  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(uint64_t, uint16_t *));
+  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(uint64_t, uint32_t *));
+  MOCK_CONST_METHOD2(GetMemoryAtAddress, bool(uint64_t, uint64_t *));
 };
 
 // Handy definitions for all tests.
@@ -69,15 +76,15 @@ struct CFIFixture {
   void ExpectNoMemoryReferences() {
     EXPECT_CALL(memory, GetBase()).Times(0);
     EXPECT_CALL(memory, GetSize()).Times(0);
-    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<u_int8_t *>())).Times(0);
-    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<u_int16_t *>())).Times(0);
-    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<u_int32_t *>())).Times(0);
-    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<u_int64_t *>())).Times(0);
+    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<uint8_t *>())).Times(0);
+    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<uint16_t *>())).Times(0);
+    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<uint32_t *>())).Times(0);
+    EXPECT_CALL(memory, GetMemoryAtAddress(_, A<uint64_t *>())).Times(0);
   }
 
   CFIFrameInfo cfi;
   MockMemoryRegion memory;
-  CFIFrameInfo::RegisterValueMap<u_int64_t> registers, caller_registers;
+  CFIFrameInfo::RegisterValueMap<uint64_t> registers, caller_registers;
 };
 
 class Simple: public CFIFixture, public Test { };
@@ -86,63 +93,80 @@ class Simple: public CFIFixture, public Test { };
 TEST_F(Simple, NoCFA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetRARule("0");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetRARule(Module::Expr("0"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
+  ASSERT_EQ(".ra: 0", cfi.Serialize());
 }
 
 // FindCallerRegs should fail if no .ra rule is provided.
 TEST_F(Simple, NoRA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("0");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("0"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
+  ASSERT_EQ(".cfa: 0", cfi.Serialize());
 }
 
 TEST_F(Simple, SetCFAAndRARule) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("330903416631436410");
-  cfi.SetRARule("5870666104170902211");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("330903416631436410"));
+  cfi.SetRARule(Module::Expr("5870666104170902211"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(2U, caller_registers.size());
-  ASSERT_EQ(330903416631436410ULL, caller_registers[".cfa"]);
-  ASSERT_EQ(5870666104170902211ULL, caller_registers[".ra"]);
+  ASSERT_EQ(330903416631436410ULL, caller_registers.get(ustr__ZDcfa()));
+  ASSERT_EQ(5870666104170902211ULL, caller_registers.get(ustr__ZDra()));
+
+  ASSERT_EQ(".cfa: 330903416631436410 .ra: 5870666104170902211",
+            cfi.Serialize());
 }
 
 TEST_F(Simple, SetManyRules) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("$temp1 68737028 = $temp2 61072337 = $temp1 $temp2 -");
-  cfi.SetRARule(".cfa 99804755 +");
-  cfi.SetRegisterRule("register1", ".cfa 54370437 *");
-  cfi.SetRegisterRule("vodkathumbscrewingly", "24076308 .cfa +");
-  cfi.SetRegisterRule("pubvexingfjordschmaltzy", ".cfa 29801007 -");
-  cfi.SetRegisterRule("uncopyrightables", "92642917 .cfa /");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("$temp1 68737028 = $temp2 61072337 = $temp1 $temp2 -"));
+  cfi.SetRARule(Module::Expr(".cfa 99804755 +"));
+
+  const UniqueString* reg1 = ToUniqueString("register1");
+  const UniqueString* reg2 = ToUniqueString("vodkathumbscrewingly");
+  const UniqueString* reg3 = ToUniqueString("pubvexingfjordschmaltzy");
+  const UniqueString* reg4 = ToUniqueString("uncopyrightables");
+
+  cfi.SetRegisterRule(reg1, Module::Expr(".cfa 54370437 *"));
+  cfi.SetRegisterRule(reg2, Module::Expr("24076308 .cfa +"));
+  cfi.SetRegisterRule(reg3, Module::Expr(".cfa 29801007 -"));
+  cfi.SetRegisterRule(reg4, Module::Expr("92642917 .cfa /"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(6U, caller_registers.size());
-  ASSERT_EQ(7664691U,           caller_registers[".cfa"]);
-  ASSERT_EQ(107469446U,         caller_registers[".ra"]);
-  ASSERT_EQ(416732599139967ULL, caller_registers["register1"]);
-  ASSERT_EQ(31740999U,          caller_registers["vodkathumbscrewingly"]);
-  ASSERT_EQ(-22136316ULL,       caller_registers["pubvexingfjordschmaltzy"]);
-  ASSERT_EQ(12U,                caller_registers["uncopyrightables"]);
+  ASSERT_EQ(7664691U,           caller_registers.get(ustr__ZDcfa()));
+  ASSERT_EQ(107469446U,         caller_registers.get(ustr__ZDra()));
+  ASSERT_EQ(416732599139967ULL, caller_registers.get(reg1));
+  ASSERT_EQ(31740999U,          caller_registers.get(reg2));
+  ASSERT_EQ(-22136316ULL,       caller_registers.get(reg3));
+  ASSERT_EQ(12U,                caller_registers.get(reg4));
+  ASSERT_EQ(".cfa: $temp1 68737028 = $temp2 61072337 = $temp1 $temp2 - "
+            ".ra: .cfa 99804755 + "
+            "pubvexingfjordschmaltzy: .cfa 29801007 - "
+            "register1: .cfa 54370437 * "
+            "uncopyrightables: 92642917 .cfa / "
+            "vodkathumbscrewingly: 24076308 .cfa +",
+            cfi.Serialize());
 }
 
 TEST_F(Simple, RulesOverride) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("330903416631436410");
-  cfi.SetRARule("5870666104170902211");
-  cfi.SetCFARule("2828089117179001");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("330903416631436410"));
+  cfi.SetRARule(Module::Expr("5870666104170902211"));
+  cfi.SetCFARule(Module::Expr("2828089117179001"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(2U, caller_registers.size());
-  ASSERT_EQ(2828089117179001ULL, caller_registers[".cfa"]);
-  ASSERT_EQ(5870666104170902211ULL, caller_registers[".ra"]);
+  ASSERT_EQ(2828089117179001ULL, caller_registers.get(ustr__ZDcfa()));
+  ASSERT_EQ(5870666104170902211ULL, caller_registers.get(ustr__ZDra()));
+  ASSERT_EQ(".cfa: 2828089117179001 .ra: 5870666104170902211",
+            cfi.Serialize());
 }
 
 class Scope: public CFIFixture, public Test { };
@@ -151,9 +175,9 @@ class Scope: public CFIFixture, public Test { };
 TEST_F(Scope, CFALacksCFA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule(".cfa");
-  cfi.SetRARule("0");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr(".cfa"));
+  cfi.SetRARule(Module::Expr("0"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
 }
 
@@ -161,9 +185,9 @@ TEST_F(Scope, CFALacksCFA) {
 TEST_F(Scope, CFALacksRA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule(".ra");
-  cfi.SetRARule("0");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr(".ra"));
+  cfi.SetRARule(Module::Expr("0"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
 }
 
@@ -172,36 +196,36 @@ TEST_F(Scope, CFALacksRA) {
 TEST_F(Scope, CFASeesCurrentRegs) {
   ExpectNoMemoryReferences();
 
-  registers[".baraminology"] = 0x06a7bc63e4f13893ULL;
-  registers[".ornithorhynchus"] = 0x5e0bf850bafce9d2ULL;
-  cfi.SetCFARule(".baraminology .ornithorhynchus +");
-  cfi.SetRARule("0");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  const UniqueString* reg1 = ToUniqueString(".baraminology");
+  const UniqueString* reg2 = ToUniqueString(".ornithorhynchus");
+  registers.set(reg1, 0x06a7bc63e4f13893ULL);
+  registers.set(reg2, 0x5e0bf850bafce9d2ULL);
+  cfi.SetCFARule(Module::Expr(".baraminology .ornithorhynchus +"));
+  cfi.SetRARule(Module::Expr("0"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(2U, caller_registers.size());
   ASSERT_EQ(0x06a7bc63e4f13893ULL + 0x5e0bf850bafce9d2ULL,
-            caller_registers[".cfa"]);
+            caller_registers.get(ustr__ZDcfa()));
 }
 
 // .cfa should be in scope in the return address expression.
 TEST_F(Scope, RASeesCFA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("48364076");
-  cfi.SetRARule(".cfa");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("48364076"));
+  cfi.SetRARule(Module::Expr(".cfa"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(2U, caller_registers.size());
-  ASSERT_EQ(48364076U, caller_registers[".ra"]);
+  ASSERT_EQ(48364076U, caller_registers.get(ustr__ZDra()));
 }
 
 // There should be no value for .ra in scope when evaluating the CFA rule.
 TEST_F(Scope, RALacksRA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("0");
-  cfi.SetRARule(".ra");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("0"));
+  cfi.SetRARule(Module::Expr(".ra"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
 }
 
@@ -210,36 +234,37 @@ TEST_F(Scope, RALacksRA) {
 TEST_F(Scope, RASeesCurrentRegs) {
   ExpectNoMemoryReferences();
 
-  registers["noachian"] = 0x54dc4a5d8e5eb503ULL;
-  cfi.SetCFARule("10359370");
-  cfi.SetRARule("noachian");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("10359370"));
+  const UniqueString* reg1 = ToUniqueString("noachian");
+  registers.set(reg1, 0x54dc4a5d8e5eb503ULL);
+  cfi.SetRARule(Module::Expr(reg1, 0, false));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(2U, caller_registers.size());
-  ASSERT_EQ(0x54dc4a5d8e5eb503ULL, caller_registers[".ra"]);
+  ASSERT_EQ(0x54dc4a5d8e5eb503ULL, caller_registers.get(ustr__ZDra()));
 }
 
 // .cfa should be in scope for register rules.
 TEST_F(Scope, RegistersSeeCFA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("6515179");
-  cfi.SetRARule(".cfa");
-  cfi.SetRegisterRule("rogerian", ".cfa");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("6515179"));
+  cfi.SetRARule(Module::Expr(".cfa"));
+  const UniqueString* reg1 = ToUniqueString("rogerian");
+  cfi.SetRegisterRule(reg1, Module::Expr(".cfa"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(3U, caller_registers.size());
-  ASSERT_EQ(6515179U, caller_registers["rogerian"]);
+  ASSERT_EQ(6515179U, caller_registers.get(reg1));
 }
 
 // The return address should not be in scope for register rules.
 TEST_F(Scope, RegsLackRA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("42740329");
-  cfi.SetRARule("27045204");
-  cfi.SetRegisterRule("$r1", ".ra");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("42740329"));
+  cfi.SetRARule(Module::Expr("27045204"));
+  const UniqueString* reg1 = ToUniqueString("$r1");
+  cfi.SetRegisterRule(reg1, Module::Expr(".ra"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
 }
 
@@ -247,31 +272,32 @@ TEST_F(Scope, RegsLackRA) {
 TEST_F(Scope, RegsSeeRegs) {
   ExpectNoMemoryReferences();
 
-  registers["$r1"] = 0x6ed3582c4bedb9adULL;
-  registers["$r2"] = 0xd27d9e742b8df6d0ULL;
-  cfi.SetCFARule("88239303");
-  cfi.SetRARule("30503835");
-  cfi.SetRegisterRule("$r1", "$r1 42175211 = $r2");
-  cfi.SetRegisterRule("$r2", "$r2 21357221 = $r1");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  const UniqueString* reg1 = ToUniqueString("$r1");
+  const UniqueString* reg2 = ToUniqueString("$r2");
+  registers.set(reg1, 0x6ed3582c4bedb9adULL);
+  registers.set(reg2, 0xd27d9e742b8df6d0ULL);
+  cfi.SetCFARule(Module::Expr("88239303"));
+  cfi.SetRARule(Module::Expr("30503835"));
+  cfi.SetRegisterRule(reg1, Module::Expr("$r1 42175211 = $r2"));
+  cfi.SetRegisterRule(reg2, Module::Expr("$r2 21357221 = $r1"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(4U, caller_registers.size());
-  ASSERT_EQ(0xd27d9e742b8df6d0ULL, caller_registers["$r1"]);
-  ASSERT_EQ(0x6ed3582c4bedb9adULL, caller_registers["$r2"]);
+  ASSERT_EQ(0xd27d9e742b8df6d0ULL, caller_registers.get(reg1));
+  ASSERT_EQ(0x6ed3582c4bedb9adULL, caller_registers.get(reg2));
 }
 
 // Each rule's temporaries are separate.
 TEST_F(Scope, SeparateTempsRA) {
   ExpectNoMemoryReferences();
 
-  cfi.SetCFARule("$temp1 76569129 = $temp1");
-  cfi.SetRARule("0");
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("$temp1 76569129 = $temp1"));
+  cfi.SetRARule(Module::Expr("0"));
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
 
-  cfi.SetCFARule("$temp1 76569129 = $temp1");
-  cfi.SetRARule("$temp1");
-  ASSERT_FALSE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  cfi.SetCFARule(Module::Expr("$temp1 76569129 = $temp1"));
+  cfi.SetRARule(Module::Expr("$temp1"));
+  ASSERT_FALSE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                              &caller_registers));
 }
 
@@ -279,7 +305,7 @@ class MockCFIRuleParserHandler: public CFIRuleParser::Handler {
  public:
   MOCK_METHOD1(CFARule, void(const string &));
   MOCK_METHOD1(RARule,  void(const string &));
-  MOCK_METHOD2(RegisterRule, void(const string &, const string &));
+  MOCK_METHOD2(RegisterRule, void(const UniqueString*, const string &));
 };
 
 // A fixture class for testing CFIRuleParser.
@@ -350,7 +376,7 @@ TEST_F(Parser, RA) {
 }
 
 TEST_F(Parser, Reg) {
-  EXPECT_CALL(mock_handler, RegisterRule("nemo", "mellifluous"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("nemo"), "mellifluous"))
       .WillOnce(Return());
   EXPECT_TRUE(parser.Parse("nemo: mellifluous"));
 }
@@ -358,18 +384,18 @@ TEST_F(Parser, Reg) {
 TEST_F(Parser, CFARARegs) {
   EXPECT_CALL(mock_handler, CFARule("cfa expression")).WillOnce(Return());
   EXPECT_CALL(mock_handler, RARule("ra expression")).WillOnce(Return());
-  EXPECT_CALL(mock_handler, RegisterRule("galba", "praetorian"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("galba"), "praetorian"))
       .WillOnce(Return());
-  EXPECT_CALL(mock_handler, RegisterRule("otho", "vitellius"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("otho"), "vitellius"))
       .WillOnce(Return());
   EXPECT_TRUE(parser.Parse(".cfa: cfa expression .ra: ra expression "
                     "galba: praetorian otho: vitellius"));
 }
 
 TEST_F(Parser, Whitespace) {
-  EXPECT_CALL(mock_handler, RegisterRule("r1", "r1 expression"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("r1"), "r1 expression"))
       .WillOnce(Return());
-  EXPECT_CALL(mock_handler, RegisterRule("r2", "r2 expression"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("r2"), "r2 expression"))
       .WillOnce(Return());
   EXPECT_TRUE(parser.Parse(" r1:\tr1\nexpression \tr2:\t\rr2\r\n "
                            "expression  \n"));
@@ -380,21 +406,21 @@ TEST_F(Parser, WhitespaceLoneColon) {
 }
 
 TEST_F(Parser, EmptyName) {
-  EXPECT_CALL(mock_handler, RegisterRule("reg", _))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("reg"), _))
       .Times(AtMost(1))
       .WillRepeatedly(Return());
   EXPECT_FALSE(parser.Parse("reg: expr1 : expr2"));
 }
 
 TEST_F(Parser, RuleLoneColon) {
-  EXPECT_CALL(mock_handler, RegisterRule("r1", "expr"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("r1"), "expr"))
       .Times(AtMost(1))
       .WillRepeatedly(Return());
   EXPECT_FALSE(parser.Parse(" r1:   expr   :"));
 }
 
 TEST_F(Parser, RegNoExprRule) {
-  EXPECT_CALL(mock_handler, RegisterRule("r1", "expr"))
+  EXPECT_CALL(mock_handler, RegisterRule(ToUniqueString("r1"), "expr"))
       .Times(AtMost(1))
       .WillRepeatedly(Return());
   EXPECT_FALSE(parser.Parse("r0: r1:   expr"));
@@ -411,34 +437,34 @@ class ParseHandler: public ParseHandlerFixture, public Test { };
 TEST_F(ParseHandler, CFARARule) {
   handler.CFARule("reg-for-cfa");
   handler.RARule("reg-for-ra");
-  registers["reg-for-cfa"] = 0x268a9a4a3821a797ULL;
-  registers["reg-for-ra"] = 0x6301b475b8b91c02ULL;
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  registers.set(ToUniqueString("reg-for-cfa"), 0x268a9a4a3821a797ULL);
+  registers.set(ToUniqueString("reg-for-ra"), 0x6301b475b8b91c02ULL);
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(0x268a9a4a3821a797ULL, caller_registers[".cfa"]);
-  ASSERT_EQ(0x6301b475b8b91c02ULL, caller_registers[".ra"]);
+  ASSERT_EQ(0x268a9a4a3821a797ULL, caller_registers.get(ustr__ZDcfa()));
+  ASSERT_EQ(0x6301b475b8b91c02ULL, caller_registers.get(ustr__ZDra()));
 }
 
 TEST_F(ParseHandler, RegisterRules) {
   handler.CFARule("reg-for-cfa");
   handler.RARule("reg-for-ra");
-  handler.RegisterRule("reg1", "reg-for-reg1");
-  handler.RegisterRule("reg2", "reg-for-reg2");
-  registers["reg-for-cfa"] = 0x268a9a4a3821a797ULL;
-  registers["reg-for-ra"] = 0x6301b475b8b91c02ULL;
-  registers["reg-for-reg1"] = 0x06cde8e2ff062481ULL;
-  registers["reg-for-reg2"] = 0xff0c4f76403173e2ULL;
-  ASSERT_TRUE(cfi.FindCallerRegs<u_int64_t>(registers, memory,
+  handler.RegisterRule(ToUniqueString("reg1"), "reg-for-reg1");
+  handler.RegisterRule(ToUniqueString("reg2"), "reg-for-reg2");
+  registers.set(ToUniqueString("reg-for-cfa"), 0x268a9a4a3821a797ULL);
+  registers.set(ToUniqueString("reg-for-ra"), 0x6301b475b8b91c02ULL);
+  registers.set(ToUniqueString("reg-for-reg1"), 0x06cde8e2ff062481ULL);
+  registers.set(ToUniqueString("reg-for-reg2"), 0xff0c4f76403173e2ULL);
+  ASSERT_TRUE(cfi.FindCallerRegs<uint64_t>(registers, memory,
                                             &caller_registers));
-  ASSERT_EQ(0x268a9a4a3821a797ULL, caller_registers[".cfa"]);
-  ASSERT_EQ(0x6301b475b8b91c02ULL, caller_registers[".ra"]);
-  ASSERT_EQ(0x06cde8e2ff062481ULL, caller_registers["reg1"]);
-  ASSERT_EQ(0xff0c4f76403173e2ULL, caller_registers["reg2"]);
+  ASSERT_EQ(0x268a9a4a3821a797ULL, caller_registers.get(ustr__ZDcfa()));
+  ASSERT_EQ(0x6301b475b8b91c02ULL, caller_registers.get(ustr__ZDra()));
+  ASSERT_EQ(0x06cde8e2ff062481ULL, caller_registers.get(ToUniqueString("reg1")));
+  ASSERT_EQ(0xff0c4f76403173e2ULL, caller_registers.get(ToUniqueString("reg2")));
 }
 
 struct SimpleCFIWalkerFixture {
   struct RawContext {
-    u_int64_t r0, r1, r2, r3, r4, sp, pc;
+    uint64_t r0, r1, r2, r3, r4, sp, pc;
   };
   enum Validity {
     R0_VALID = 0x01,
@@ -449,7 +475,7 @@ struct SimpleCFIWalkerFixture {
     SP_VALID = 0x20,
     PC_VALID = 0x40
   };
-  typedef SimpleCFIWalker<u_int64_t, RawContext> CFIWalker;
+  typedef SimpleCFIWalker<uint64_t, RawContext> CFIWalker;
 
   SimpleCFIWalkerFixture()
       : walker(register_map,
@@ -464,13 +490,13 @@ struct SimpleCFIWalkerFixture {
 
 SimpleCFIWalkerFixture::CFIWalker::RegisterSet
 SimpleCFIWalkerFixture::register_map[7] = {
-  { "r0", NULL,   true,  R0_VALID, &RawContext::r0 },
-  { "r1", NULL,   true,  R1_VALID, &RawContext::r1 },
-  { "r2", NULL,   false, R2_VALID, &RawContext::r2 },
-  { "r3", NULL,   false, R3_VALID, &RawContext::r3 },
-  { "r4", NULL,   true,  R4_VALID, &RawContext::r4 },
-  { "sp", ".cfa", true,  SP_VALID, &RawContext::sp },
-  { "pc", ".ra",  true,  PC_VALID, &RawContext::pc },
+  { ToUniqueString("r0"), NULL,   true,  R0_VALID, &RawContext::r0 },
+  { ToUniqueString("r1"), NULL,   true,  R1_VALID, &RawContext::r1 },
+  { ToUniqueString("r2"), NULL,   false, R2_VALID, &RawContext::r2 },
+  { ToUniqueString("r3"), NULL,   false, R3_VALID, &RawContext::r3 },
+  { ToUniqueString("r4"), NULL,   true,  R4_VALID, &RawContext::r4 },
+  { ToUniqueString("sp"), ustr__ZDcfa(), true,  SP_VALID, &RawContext::sp },
+  { ToUniqueString("pc"), ustr__ZDra(),  true,  PC_VALID, &RawContext::pc },
 };
 
 class SimpleWalker: public SimpleCFIWalkerFixture, public Test { };
@@ -490,23 +516,25 @@ TEST_F(SimpleWalker, Walk) {
   // r4 is not recoverable, even though it is a callee-saves register.
   //    Some earlier frame's unwinder must have failed to recover it.
 
-  u_int64_t stack_top = 0x83254944b20d5512ULL;
+  uint64_t stack_top = 0x83254944b20d5512ULL;
 
   // Saved r0.
   EXPECT_CALL(memory,
-              GetMemoryAtAddress(stack_top, A<u_int64_t *>()))
+              GetMemoryAtAddress(stack_top, A<uint64_t *>()))
       .WillRepeatedly(DoAll(SetArgumentPointee<1>(0xdc1975eba8602302ULL),
                             Return(true)));
   // Saved return address.
   EXPECT_CALL(memory,
-              GetMemoryAtAddress(stack_top + 16, A<u_int64_t *>()))
+              GetMemoryAtAddress(stack_top + 16, A<uint64_t *>()))
       .WillRepeatedly(DoAll(SetArgumentPointee<1>(0xba5ad6d9acce28deULL),
                             Return(true)));
 
-  call_frame_info.SetCFARule("sp 24 +");
-  call_frame_info.SetRARule(".cfa 8 - ^");
-  call_frame_info.SetRegisterRule("r0", ".cfa 24 - ^");
-  call_frame_info.SetRegisterRule("r1", "r2");
+  call_frame_info.SetCFARule(Module::Expr("sp 24 +"));
+  call_frame_info.SetRARule(Module::Expr(".cfa 8 - ^"));
+  call_frame_info.SetRegisterRule(ToUniqueString("r0"),
+                                  Module::Expr(".cfa 24 - ^"));
+  call_frame_info.SetRegisterRule(ToUniqueString("r1"),
+                                  Module::Expr("r2"));
 
   callee_context.r0 = 0x94e030ca79edd119ULL;
   callee_context.r1 = 0x937b4d7e95ce52d9ULL;
