@@ -13,6 +13,7 @@ const ADDON_DOWNGRADE                 = 8;
 
 // This verifies that bootstrappable add-ons can be used without restarts.
 Components.utils.import("resource://gre/modules/Services.jsm");
+Components.utils.import("resource://gre/modules/Promise.jsm");
 
 // Enable loading extensions from the user scopes
 Services.prefs.setIntPref("extensions.enabledScopes",
@@ -27,8 +28,12 @@ userExtDir.append("extensions2");
 userExtDir.append(gAppInfo.ID);
 registerDirectory("XREUSysExt", userExtDir.parent);
 
-do_load_httpd_js();
-var testserver;
+Components.utils.import("resource://testing-common/httpd.js");
+var testserver = new HttpServer();
+testserver.start(-1);
+gPort = testserver.identity.primaryPort;
+
+testserver.registerDirectory("/addons/", do_get_file("addons"));
 
 function resetPrefs() {
   Services.prefs.setIntPref("bootstraptest.active_version", -1);
@@ -39,6 +44,37 @@ function resetPrefs() {
   Services.prefs.setIntPref("bootstraptest.shutdown_reason", -1);
   Services.prefs.setIntPref("bootstraptest.install_reason", -1);
   Services.prefs.setIntPref("bootstraptest.uninstall_reason", -1);
+  Services.prefs.setIntPref("bootstraptest.startup_oldversion", -1);
+  Services.prefs.setIntPref("bootstraptest.shutdown_newversion", -1);
+  Services.prefs.setIntPref("bootstraptest.install_oldversion", -1);
+  Services.prefs.setIntPref("bootstraptest.uninstall_newversion", -1);
+}
+
+function waitForPref(aPref, aCallback) {
+  function prefChanged() {
+    Services.prefs.removeObserver(aPref, prefChanged);
+    // Always let whoever set the preference keep running
+    do_execute_soon(aCallback);
+  }
+  Services.prefs.addObserver(aPref, prefChanged, false);
+}
+
+function promisePref(aPref) {
+  let deferred = Promise.defer();
+
+  waitForPref(aPref, deferred.resolve.bind(deferred));
+
+  return deferred.promise;
+}
+
+function promiseInstall(aFiles) {
+  let deferred = Promise.defer();
+
+  installAllFiles(aFiles, function() {
+    deferred.resolve();
+  });
+
+  return deferred.promise;
 }
 
 function getActiveVersion() {
@@ -73,59 +109,67 @@ function getUninstallReason() {
   return Services.prefs.getIntPref("bootstraptest.uninstall_reason");
 }
 
-function manuallyInstall(aXPIFile, aInstallLocation, aID) {
-  if (TEST_UNPACKED) {
-    let dir = aInstallLocation.clone();
-    dir.append(aID);
-    dir.create(AM_Ci.nsIFile.DIRECTORY_TYPE, 0755);
-    let zip = AM_Cc["@mozilla.org/libjar/zip-reader;1"].
-              createInstance(AM_Ci.nsIZipReader);
-    zip.open(aXPIFile);
-    let entries = zip.findEntries(null);
-    while (entries.hasMore()) {
-      let entry = entries.getNext();
-      let target = dir.clone();
-      entry.split("/").forEach(function(aPart) {
-        target.append(aPart);
-      });
-      zip.extract(entry, target);
+function getStartupOldVersion() {
+  return Services.prefs.getIntPref("bootstraptest.startup_oldversion");
+}
+
+function getShutdownNewVersion() {
+  return Services.prefs.getIntPref("bootstraptest.shutdown_newversion");
+}
+
+function getInstallOldVersion() {
+  return Services.prefs.getIntPref("bootstraptest.install_oldversion");
+}
+
+function getUninstallNewVersion() {
+  return Services.prefs.getIntPref("bootstraptest.uninstall_newversion");
+}
+
+function do_check_bootstrappedPref(aCallback) {
+  let data = "{}";
+  try {
+    // This is ok to fail, as the pref won't exist on a fresh profile.
+    data = Services.prefs.getCharPref("extensions.bootstrappedAddons");
+  } catch (e) {}
+  data = JSON.parse(data);
+
+  AddonManager.getAddonsByTypes(["extension"], function(aAddons) {
+    for (let addon of aAddons) {
+      if (!addon.id.endsWith("@tests.mozilla.org"))
+        continue;
+      if (!addon.isActive)
+        continue;
+      if (addon.operationsRequiringRestart != AddonManager.OP_NEEDS_RESTART_NONE)
+        continue;
+
+      do_check_true(addon.id in data);
+      let addonData = data[addon.id];
+      delete data[addon.id];
+
+      do_check_eq(addonData.version, addon.version);
+      do_check_eq(addonData.type, addon.type);
+      let file = addon.getResourceURI().QueryInterface(Components.interfaces.nsIFileURL).file;
+      do_check_eq(addonData.descriptor, file.persistentDescriptor);
     }
-    zip.close();
+    do_check_eq(Object.keys(data).length, 0);
 
-    return dir;
-  }
-  else {
-    let target = aInstallLocation.clone();
-    target.append(aID + ".xpi");
-    aXPIFile.copyTo(target.parent, target.leafName);
-    return target;
-  }
+    aCallback();
+  });
 }
 
-function manuallyUninstall(aInstallLocation, aID) {
-  let file = getFileForAddon(aInstallLocation, aID);
-
-  // In reality because the app is restarted a flush isn't necessary for XPIs
-  // removed outside the app, but for testing we must flush manually.
-  if (file.isFile())
-    Services.obs.notifyObservers(file, "flush-cache-entry", null);
-
-  file.remove(true);
-}
 
 function run_test() {
   do_test_pending();
 
   resetPrefs();
 
-  // Create and configure the HTTP server.
-  testserver = new nsHttpServer();
-  testserver.registerDirectory("/addons/", do_get_file("addons"));
-  testserver.start(4444);
-
   startupManager();
 
-  run_test_1();
+  do_check_false(gExtensionsJSON.exists());
+
+  do_check_false(gExtensionsINI.exists());
+
+  do_check_bootstrappedPref(run_test_1);
 }
 
 // Tests that installing doesn't require a restart
@@ -142,6 +186,7 @@ function run_test_1() {
     do_check_eq(install.version, "1.0");
     do_check_eq(install.name, "Test Bootstrap 1");
     do_check_eq(install.state, AddonManager.STATE_DOWNLOADED);
+    do_check_neq(install.addon.syncGUID, null);
     do_check_true(install.addon.hasResource("install.rdf"));
     do_check_true(install.addon.hasResource("bootstrap.js"));
     do_check_false(install.addon.hasResource("foo.bar"));
@@ -150,6 +195,13 @@ function run_test_1() {
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
     let addon = install.addon;
+
+    waitForPref("bootstraptest.startup_reason", function() {
+      do_check_bootstrappedPref(function() {
+        check_test_1(addon.syncGUID);
+      });
+    });
+
     prepare_test({
       "bootstrap1@tests.mozilla.org": [
         ["onInstalling", false],
@@ -160,13 +212,17 @@ function run_test_1() {
       "onInstallEnded",
     ], function() {
       do_check_true(addon.hasResource("install.rdf"));
-      check_test_1();
+
+      // startup should not have been called yet.
+      do_check_eq(getActiveVersion(), -1);
     });
     install.install();
   });
 }
 
-function check_test_1() {
+function check_test_1(installSyncGUID) {
+  do_check_false(gExtensionsINI.exists());
+
   AddonManager.getAllInstalls(function(installs) {
     // There should be no active installs now since the install completed and
     // doesn't require a restart.
@@ -175,12 +231,15 @@ function check_test_1() {
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
       do_check_neq(b1, null);
       do_check_eq(b1.version, "1.0");
+      do_check_neq(b1.syncGUID, null);
+      do_check_eq(b1.syncGUID, installSyncGUID);
       do_check_false(b1.appDisabled);
       do_check_false(b1.userDisabled);
       do_check_true(b1.isActive);
       do_check_eq(getInstalledVersion(), 1);
       do_check_eq(getActiveVersion(), 1);
       do_check_eq(getStartupReason(), ADDON_INSTALL);
+      do_check_eq(getStartupOldVersion(), 0);
       do_check_true(b1.hasResource("install.rdf"));
       do_check_true(b1.hasResource("bootstrap.js"));
       do_check_false(b1.hasResource("foo.bar"));
@@ -192,7 +251,7 @@ function check_test_1() {
       AddonManager.getAddonsWithOperationsByTypes(null, function(list) {
         do_check_eq(list.length, 0);
 
-        run_test_2();
+        do_execute_soon(run_test_2);
       });
     });
   });
@@ -221,6 +280,7 @@ function run_test_2() {
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 0);
     do_check_eq(getShutdownReason(), ADDON_DISABLE);
+    do_check_eq(getShutdownNewVersion(), 0);
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(newb1) {
@@ -230,7 +290,7 @@ function run_test_2() {
       do_check_true(newb1.userDisabled);
       do_check_false(newb1.isActive);
 
-      run_test_3();
+      do_check_bootstrappedPref(run_test_3);
     });
   });
 }
@@ -241,12 +301,16 @@ function run_test_3() {
   do_check_eq(getInstalledVersion(), 1);
   do_check_eq(getActiveVersion(), 0);
   do_check_eq(getShutdownReason(), ADDON_DISABLE);
+  do_check_eq(getShutdownNewVersion(), 0);
   startupManager(false);
   do_check_eq(getInstalledVersion(), 1);
   do_check_eq(getActiveVersion(), 0);
   do_check_eq(getShutdownReason(), ADDON_DISABLE);
+  do_check_eq(getShutdownNewVersion(), 0);
   do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
+  do_check_false(gExtensionsINI.exists());
+
   AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
     do_check_neq(b1, null);
     do_check_eq(b1.version, "1.0");
@@ -254,7 +318,7 @@ function run_test_3() {
     do_check_true(b1.userDisabled);
     do_check_false(b1.isActive);
 
-    run_test_4();
+    do_check_bootstrappedPref(run_test_4);
   });
 }
 
@@ -281,6 +345,7 @@ function run_test_4() {
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
     do_check_eq(getStartupReason(), ADDON_ENABLE);
+    do_check_eq(getStartupOldVersion(), 0);
     do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(newb1) {
@@ -290,7 +355,7 @@ function run_test_4() {
       do_check_false(newb1.userDisabled);
       do_check_true(newb1.isActive);
 
-      run_test_5();
+      do_check_bootstrappedPref(run_test_5);
     });
   });
 }
@@ -298,14 +363,19 @@ function run_test_4() {
 // Tests that a restart shuts down and restarts the add-on
 function run_test_5() {
   shutdownManager();
+  // By the time we've shut down, the database must have been written
+  do_check_true(gExtensionsJSON.exists());
+
   do_check_eq(getInstalledVersion(), 1);
   do_check_eq(getActiveVersion(), 0);
   do_check_eq(getShutdownReason(), APP_SHUTDOWN);
+  do_check_eq(getShutdownNewVersion(), 0);
   do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
   startupManager(false);
   do_check_eq(getInstalledVersion(), 1);
   do_check_eq(getActiveVersion(), 1);
   do_check_eq(getStartupReason(), APP_STARTUP);
+  do_check_eq(getStartupOldVersion(), 0);
   do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
   AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
@@ -316,7 +386,7 @@ function run_test_5() {
     do_check_true(b1.isActive);
     do_check_false(isExtensionInAddonsList(profileDir, b1.id));
 
-    run_test_6();
+    do_check_bootstrappedPref(run_test_6);
   });
 }
 
@@ -335,6 +405,7 @@ function run_test_6() {
     do_check_eq(install.name, "Test Bootstrap 1");
     do_check_eq(install.state, AddonManager.STATE_DOWNLOADED);
 
+    waitForPref("bootstraptest.startup_reason", check_test_6);
     prepare_test({
       "bootstrap1@tests.mozilla.org": [
         ["onInstalling", false],
@@ -343,7 +414,8 @@ function run_test_6() {
     }, [
       "onInstallStarted",
       "onInstallEnded",
-    ], check_test_6);
+    ], function() {
+    });
     install.install();
   });
 }
@@ -358,11 +430,15 @@ function check_test_6() {
     do_check_eq(getInstalledVersion(), 2);
     do_check_eq(getActiveVersion(), 2);
     do_check_eq(getStartupReason(), ADDON_UPGRADE);
+    do_check_eq(getInstallOldVersion(), 1);
+    do_check_eq(getStartupOldVersion(), 1);
     do_check_eq(getShutdownReason(), ADDON_UPGRADE);
+    do_check_eq(getShutdownNewVersion(), 2);
+    do_check_eq(getUninstallNewVersion(), 2);
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
     do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "2.0");
 
-    run_test_7();
+    do_check_bootstrappedPref(run_test_7);
   });
 }
 
@@ -380,7 +456,7 @@ function run_test_7() {
                 AddonManager.OP_NEEDS_RESTART_UNINSTALL, 0);
     b1.uninstall();
 
-    check_test_7();
+    do_check_bootstrappedPref(check_test_7);
   });
 }
 
@@ -389,9 +465,10 @@ function check_test_7() {
   do_check_eq(getInstalledVersion(), 0);
   do_check_eq(getActiveVersion(), 0);
   do_check_eq(getShutdownReason(), ADDON_UNINSTALL);
+  do_check_eq(getShutdownNewVersion(), 0);
   do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "2.0");
 
-  AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+  AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
     do_check_eq(b1, null);
 
     restartManager();
@@ -399,9 +476,9 @@ function check_test_7() {
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(newb1) {
       do_check_eq(newb1, null);
 
-      run_test_8();
+      do_check_bootstrappedPref(run_test_8);
     });
-  });
+  }));
 }
 
 // Test that a bootstrapped extension dropped into the profile loads properly
@@ -422,10 +499,11 @@ function run_test_8() {
     do_check_true(b1.isActive);
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
-    do_check_eq(getStartupReason(), APP_STARTUP);
+    do_check_eq(getStartupReason(), ADDON_INSTALL);
+    do_check_eq(getStartupOldVersion(), 0);
     do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
-    run_test_9();
+    do_check_bootstrappedPref(run_test_9);
   });
 }
 
@@ -441,13 +519,14 @@ function run_test_9() {
     do_check_eq(b1, null);
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
-    run_test_10();
+    do_check_bootstrappedPref(run_test_10);
   });
 }
 
 
 // Tests that installing a downgrade sends the right reason
 function run_test_10() {
+  resetPrefs();
   prepare_test({ }, [
     "onNewInstall"
   ]);
@@ -465,6 +544,7 @@ function run_test_10() {
     do_check_false(install.addon.hasResource("foo.bar"));
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "2.0");
 
+    waitForPref("bootstraptest.startup_reason", check_test_10_pt1);
     prepare_test({
       "bootstrap1@tests.mozilla.org": [
         ["onInstalling", false],
@@ -473,7 +553,9 @@ function run_test_10() {
     }, [
       "onInstallStarted",
       "onInstallEnded",
-    ], check_test_10_pt1);
+    ], function() {
+      do_print("Waiting for startup of bootstrap1_2");
+    });
     install.install();
   });
 }
@@ -488,6 +570,7 @@ function check_test_10_pt1() {
     do_check_eq(getInstalledVersion(), 2);
     do_check_eq(getActiveVersion(), 2);
     do_check_eq(getStartupReason(), ADDON_INSTALL);
+    do_check_eq(getStartupOldVersion(), 0);
     do_check_true(b1.hasResource("install.rdf"));
     do_check_true(b1.hasResource("bootstrap.js"));
     do_check_false(b1.hasResource("foo.bar"));
@@ -506,6 +589,7 @@ function check_test_10_pt1() {
       do_check_eq(install.name, "Test Bootstrap 1");
       do_check_eq(install.state, AddonManager.STATE_DOWNLOADED);
 
+      waitForPref("bootstraptest.startup_reason", check_test_10_pt2);
       prepare_test({
         "bootstrap1@tests.mozilla.org": [
           ["onInstalling", false],
@@ -514,7 +598,7 @@ function check_test_10_pt1() {
       }, [
         "onInstallStarted",
         "onInstallEnded",
-      ], check_test_10_pt2);
+      ], function() { });
       install.install();
     });
   });
@@ -530,11 +614,15 @@ function check_test_10_pt2() {
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
     do_check_eq(getStartupReason(), ADDON_DOWNGRADE);
+    do_check_eq(getInstallOldVersion(), 2);
+    do_check_eq(getStartupOldVersion(), 2);
     do_check_eq(getShutdownReason(), ADDON_DOWNGRADE);
+    do_check_eq(getShutdownNewVersion(), 1);
+    do_check_eq(getUninstallNewVersion(), 1);
     do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "2.0");
 
-    run_test_11();
+    do_check_bootstrappedPref(run_test_11);
   });
 }
 
@@ -555,6 +643,7 @@ function run_test_11() {
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 0);
     do_check_eq(getShutdownReason(), ADDON_DISABLE);
+    do_check_eq(getShutdownNewVersion(), 0);
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
     b1.uninstall();
@@ -569,7 +658,7 @@ function check_test_11() {
   do_check_eq(getActiveVersion(), 0);
   do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
-  run_test_12();
+  do_check_bootstrappedPref(run_test_12);
 }
 
 // Tests that bootstrapped extensions are correctly loaded even if the app is
@@ -590,16 +679,20 @@ function run_test_12() {
     do_check_true(b1.isActive);
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
-    do_check_eq(getStartupReason(), APP_STARTUP);
+    do_check_eq(getStartupReason(), ADDON_INSTALL);
+    do_check_eq(getStartupOldVersion(), 0);
     do_check_in_crash_annotation("bootstrap1@tests.mozilla.org", "1.0");
 
     b1.uninstall();
-    restartManager();
-
-    run_test_13();
+    do_execute_soon(test_12_restart);
   });
 }
 
+function test_12_restart() {
+  restartManager();
+  do_check_bootstrappedPref(run_test_13);
+}
+
 
 // Tests that installing a bootstrapped extension with an invalid application
 // entry doesn't call it's startup method
@@ -626,7 +719,7 @@ function run_test_13() {
     }, [
       "onInstallStarted",
       "onInstallEnded",
-    ], check_test_13);
+    ], callback_soon(check_test_13));
     install.install();
   });
 }
@@ -647,23 +740,27 @@ function check_test_13() {
       do_check_eq(getActiveVersion(), 0);     // Should not have called startup though
       do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "3.0");
 
-      restartManager();
+      do_execute_soon(test_13_restart);
+    });
+  });
+}
 
-      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
-        do_check_neq(b1, null);
-        do_check_eq(b1.version, "3.0");
-        do_check_true(b1.appDisabled);
-        do_check_false(b1.userDisabled);
-        do_check_false(b1.isActive);
-        do_check_eq(getInstalledVersion(), 3);  // We call install even for disabled add-ons
-        do_check_eq(getActiveVersion(), 0);     // Should not have called startup though
-        do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "3.0");
+function test_13_restart() {
+  restartManager();
 
-        b1.uninstall();
-        restartManager();
+  AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+    do_check_neq(b1, null);
+    do_check_eq(b1.version, "3.0");
+    do_check_true(b1.appDisabled);
+    do_check_false(b1.userDisabled);
+    do_check_false(b1.isActive);
+    do_check_eq(getInstalledVersion(), 3);  // We call install even for disabled add-ons
+    do_check_eq(getActiveVersion(), 0);     // Should not have called startup though
+    do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "3.0");
 
-        run_test_14();
-      });
+    do_check_bootstrappedPref(function() {
+      b1.uninstall();
+      do_execute_soon(run_test_14);
     });
   });
 }
@@ -671,6 +768,8 @@ function check_test_13() {
 // Tests that a bootstrapped extension with an invalid target application entry
 // does not get loaded when detected during startup
 function run_test_14() {
+  restartManager();
+
   shutdownManager();
 
   manuallyInstall(do_get_addon("test_bootstrap1_3"), profileDir,
@@ -688,16 +787,19 @@ function run_test_14() {
     do_check_eq(getActiveVersion(), 0);      // Should not have called startup though
     do_check_not_in_crash_annotation("bootstrap1@tests.mozilla.org", "3.0");
 
-    b1.uninstall();
+    do_check_bootstrappedPref(function() {
+      b1.uninstall();
 
-    run_test_15();
+      run_test_15();
+    });
   });
 }
 
 // Tests that upgrading a disabled bootstrapped extension still calls uninstall
 // and install but doesn't startup the new version
 function run_test_15() {
-  installAllFiles([do_get_addon("test_bootstrap1_1")], function() {
+  resetPrefs();
+  waitForPref("bootstraptest.startup_reason", function test_15_after_startup() {
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
       do_check_neq(b1, null);
       do_check_eq(b1.version, "1.0");
@@ -730,11 +832,12 @@ function run_test_15() {
         }, [
           "onInstallStarted",
           "onInstallEnded",
-        ], check_test_15);
+        ], callback_soon(check_test_15));
         install.install();
       });
     });
   });
+  installAllFiles([do_get_addon("test_bootstrap1_1")], function test_15_addon_installed() { });
 }
 
 function check_test_15() {
@@ -747,28 +850,31 @@ function check_test_15() {
     do_check_eq(getInstalledVersion(), 2);
     do_check_eq(getActiveVersion(), 0);
 
-    restartManager();
+    do_check_bootstrappedPref(function() {
+      restartManager();
 
-    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
-      do_check_neq(b1, null);
-      do_check_eq(b1.version, "2.0");
-      do_check_false(b1.appDisabled);
-      do_check_true(b1.userDisabled);
-      do_check_false(b1.isActive);
-      do_check_eq(getInstalledVersion(), 2);
-      do_check_eq(getActiveVersion(), 0);
+      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+        do_check_neq(b1, null);
+        do_check_eq(b1.version, "2.0");
+        do_check_false(b1.appDisabled);
+        do_check_true(b1.userDisabled);
+        do_check_false(b1.isActive);
+        do_check_eq(getInstalledVersion(), 2);
+        do_check_eq(getActiveVersion(), 0);
 
-      b1.uninstall();
+        b1.uninstall();
 
-      run_test_16();
+        run_test_16();
+      });
     });
   });
 }
 
 // Tests that bootstrapped extensions don't get loaded when in safe mode
 function run_test_16() {
-  installAllFiles([do_get_addon("test_bootstrap1_1")], function() {
-    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+  resetPrefs();
+  waitForPref("bootstraptest.startup_reason", function test_16_after_startup() {
+    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
       // Should have installed and started
       do_check_eq(getInstalledVersion(), 1);
       do_check_eq(getActiveVersion(), 1);
@@ -786,7 +892,7 @@ function run_test_16() {
       gAppInfo.inSafeMode = true;
       startupManager(false);
 
-      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
         // Should still be stopped
         do_check_eq(getInstalledVersion(), 1);
         do_check_eq(getActiveVersion(), 0);
@@ -806,11 +912,12 @@ function run_test_16() {
         AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
           b1.uninstall();
 
-          run_test_17();
+          do_execute_soon(run_test_17);
         });
-      });
-    });
+      }));
+    }));
   });
+  installAllFiles([do_get_addon("test_bootstrap1_1")], function() { });
 }
 
 // Check that a bootstrapped extension in a non-profile location is loaded
@@ -831,7 +938,7 @@ function run_test_17() {
     do_check_eq(b1.version, "1.0");
     do_check_true(b1.isActive);
 
-    run_test_18();
+    do_check_bootstrappedPref(run_test_18);
   });
 }
 
@@ -839,7 +946,7 @@ function run_test_17() {
 // the existing one
 function run_test_18() {
   resetPrefs();
-  installAllFiles([do_get_addon("test_bootstrap1_2")], function() {
+  waitForPref("bootstraptest.startup_reason", function test_16_after_startup() {
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
       // Should have installed and started
       do_check_eq(getInstalledVersion(), 2);
@@ -853,9 +960,15 @@ function run_test_18() {
       do_check_eq(getInstallReason(), ADDON_UPGRADE);
       do_check_eq(getStartupReason(), ADDON_UPGRADE);
 
-      run_test_19();
+      do_check_eq(getShutdownNewVersion(), 2);
+      do_check_eq(getUninstallNewVersion(), 2);
+      do_check_eq(getInstallOldVersion(), 1);
+      do_check_eq(getStartupOldVersion(), 1);
+
+      do_check_bootstrappedPref(run_test_19);
     });
   });
+  installAllFiles([do_get_addon("test_bootstrap1_2")], function() { });
 }
 
 // Check that uninstalling the profile version reveals the non-profile one
@@ -891,7 +1004,12 @@ function check_test_19() {
     do_check_eq(getInstallReason(), ADDON_INSTALL);
     do_check_eq(getStartupReason(), ADDON_INSTALL);
 
-    run_test_20();
+    do_check_eq(getShutdownNewVersion(), 0);
+    do_check_eq(getUninstallNewVersion(), 0);
+    do_check_eq(getInstallOldVersion(), 0);
+    do_check_eq(getStartupOldVersion(), 0);
+
+    do_check_bootstrappedPref(run_test_20);
   });
 }
 
@@ -919,7 +1037,12 @@ function run_test_20() {
     do_check_eq(getInstallReason(), ADDON_UPGRADE);
     do_check_eq(getStartupReason(), APP_STARTUP);
 
-    run_test_21();
+    do_check_eq(getShutdownNewVersion(), 0);
+    do_check_eq(getUninstallNewVersion(), 2);
+    do_check_eq(getInstallOldVersion(), 1);
+    do_check_eq(getStartupOldVersion(), 0);
+
+    do_execute_soon(run_test_21);
   });
 }
 
@@ -941,21 +1064,26 @@ function run_test_21() {
     do_check_true(b1.isActive);
 
     do_check_eq(getShutdownReason(), APP_SHUTDOWN);
+    do_check_eq(getShutdownNewVersion(), 0);
 
     // This won't be set as the bootstrap script was gone so we couldn't
     // uninstall it properly
     do_check_eq(getUninstallReason(), -1);
+    do_check_eq(getUninstallNewVersion(), -1);
 
     // TODO this reason should probably be ADDON_DOWNGRADE (bug 607818)
     do_check_eq(getInstallReason(), ADDON_INSTALL);
+    do_check_eq(getInstallOldVersion(), 0);
 
     do_check_eq(getStartupReason(), APP_STARTUP);
+    do_check_eq(getStartupOldVersion(), 0);
 
-    manuallyUninstall(userExtDir, "bootstrap1@tests.mozilla.org");
+    do_check_bootstrappedPref(function() {
+      manuallyUninstall(userExtDir, "bootstrap1@tests.mozilla.org");
 
-    restartManager();
-
-    run_test_22();
+      restartManager();
+      run_test_22();
+    });
   });
 }
 
@@ -971,7 +1099,7 @@ function run_test_22() {
 
   startupManager();
 
-  AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+  AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
     // Should have installed and started
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
@@ -997,19 +1125,25 @@ function run_test_22() {
       do_check_true(b1.isActive);
 
       do_check_eq(getShutdownReason(), APP_SHUTDOWN);
+      do_check_eq(getShutdownNewVersion(), 0);
 
       // This won't be set as the bootstrap script was gone so we couldn't
       // uninstall it properly
       do_check_eq(getUninstallReason(), -1);
+      do_check_eq(getUninstallNewVersion(), -1);
 
       do_check_eq(getInstallReason(), ADDON_UPGRADE);
+      do_check_eq(getInstallOldVersion(), 1);
       do_check_eq(getStartupReason(), APP_STARTUP);
+      do_check_eq(getStartupOldVersion(), 0);
 
-      b1.uninstall();
+      do_check_bootstrappedPref(function() {
+        b1.uninstall();
 
-      run_test_23();
+        run_test_23();
+      });
     });
-  });
+  }));
 }
 
 
@@ -1019,7 +1153,7 @@ function run_test_23() {
     "onNewInstall"
   ]);
 
-  let url = "http://localhost:4444/addons/test_bootstrap1_1.xpi";
+  let url = "http://localhost:" + gPort + "/addons/test_bootstrap1_1.xpi";
   AddonManager.getInstallForURL(url, function(install) {
     ensure_test_completed();
 
@@ -1051,7 +1185,7 @@ function run_test_23() {
         "onInstallEnded",
       ], function() {
         do_check_true(addon.hasResource("install.rdf"));
-        check_test_23();
+        do_check_bootstrappedPref(check_test_23);
       });
     });
     install.install();
@@ -1065,6 +1199,7 @@ function check_test_23() {
     do_check_eq(installs.length, 0);
 
     AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+     do_execute_soon(function test_23_after_startup() {
       do_check_neq(b1, null);
       do_check_eq(b1.version, "1.0");
       do_check_false(b1.appDisabled);
@@ -1073,6 +1208,7 @@ function check_test_23() {
       do_check_eq(getInstalledVersion(), 1);
       do_check_eq(getActiveVersion(), 1);
       do_check_eq(getStartupReason(), ADDON_INSTALL);
+      do_check_eq(getStartupOldVersion(), 0);
       do_check_true(b1.hasResource("install.rdf"));
       do_check_true(b1.hasResource("bootstrap.js"));
       do_check_false(b1.hasResource("foo.bar"));
@@ -1081,15 +1217,18 @@ function check_test_23() {
       let dir = do_get_addon_root_uri(profileDir, "bootstrap1@tests.mozilla.org");
       do_check_eq(b1.getResourceURI("bootstrap.js").spec, dir + "bootstrap.js");
 
-      AddonManager.getAddonsWithOperationsByTypes(null, function(list) {
+      AddonManager.getAddonsWithOperationsByTypes(null, callback_soon(function(list) {
         do_check_eq(list.length, 0);
 
         restartManager();
-        b1.uninstall();
-        restartManager();
+        AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
+          b1.uninstall();
+          restartManager();
 
-        testserver.stop(run_test_24);
-      });
+          testserver.stop(run_test_24);
+        }));
+      }));
+     });
     });
   });
 }
@@ -1097,8 +1236,12 @@ function check_test_23() {
 // Tests that we recover from a broken preference
 function run_test_24() {
   resetPrefs();
-  installAllFiles([do_get_addon("test_bootstrap1_1"), do_get_addon("test_bootstrap2_1")],
-                  function() {
+  do_print("starting 24");
+
+  Promise.all([promisePref("bootstraptest2.active_version"),
+              promiseInstall([do_get_addon("test_bootstrap1_1"), do_get_addon("test_bootstrap2_1")])])
+         .then(function test_24_pref() {
+    do_print("test 24 got prefs");
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getActiveVersion(), 1);
     do_check_eq(getInstalledVersion2(), 1);
@@ -1139,37 +1282,42 @@ function run_test_24() {
 // Tests that updating from a bootstrappable add-on to a normal add-on calls
 // the uninstall method
 function run_test_25() {
-  installAllFiles([do_get_addon("test_bootstrap1_1")], function() {
-    do_check_eq(getInstalledVersion(), 1);
-    do_check_eq(getActiveVersion(), 1);
-
-    installAllFiles([do_get_addon("test_bootstrap1_4")], function() {
-      // Needs a restart to complete this so the old version stays running
+  waitForPref("bootstraptest.startup_reason", function test_25_after_pref() {
+      do_print("test 25 pref change detected");
       do_check_eq(getInstalledVersion(), 1);
       do_check_eq(getActiveVersion(), 1);
 
-      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
-        do_check_neq(b1, null);
-        do_check_eq(b1.version, "1.0");
-        do_check_true(b1.isActive);
-        do_check_true(hasFlag(b1.pendingOperations, AddonManager.PENDING_UPGRADE));
+      installAllFiles([do_get_addon("test_bootstrap1_4")], function() {
+        // Needs a restart to complete this so the old version stays running
+        do_check_eq(getInstalledVersion(), 1);
+        do_check_eq(getActiveVersion(), 1);
 
-        restartManager();
-
-        do_check_eq(getInstalledVersion(), 0);
-        do_check_eq(getUninstallReason(), ADDON_UPGRADE);
-        do_check_eq(getActiveVersion(), 0);
-
-        AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+        AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
           do_check_neq(b1, null);
-          do_check_eq(b1.version, "4.0");
+          do_check_eq(b1.version, "1.0");
           do_check_true(b1.isActive);
-          do_check_eq(b1.pendingOperations, AddonManager.PENDING_NONE);
+          do_check_true(hasFlag(b1.pendingOperations, AddonManager.PENDING_UPGRADE));
 
-          run_test_26();
-        });
+          restartManager();
+
+          do_check_eq(getInstalledVersion(), 0);
+          do_check_eq(getUninstallReason(), ADDON_UPGRADE);
+          do_check_eq(getUninstallNewVersion(), 4);
+          do_check_eq(getActiveVersion(), 0);
+
+          AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+            do_check_neq(b1, null);
+            do_check_eq(b1.version, "4.0");
+            do_check_true(b1.isActive);
+            do_check_eq(b1.pendingOperations, AddonManager.PENDING_NONE);
+
+            do_check_bootstrappedPref(run_test_26);
+          });
+        }));
       });
-    });
+  });
+  installAllFiles([do_get_addon("test_bootstrap1_1")], function test_25_installed() {
+    do_print("test 25 install done");
   });
 }
 
@@ -1181,7 +1329,7 @@ function run_test_26() {
     do_check_eq(getInstalledVersion(), 0);
     do_check_eq(getActiveVersion(), 0);
 
-    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
       do_check_neq(b1, null);
       do_check_eq(b1.version, "4.0");
       do_check_true(b1.isActive);
@@ -1191,6 +1339,7 @@ function run_test_26() {
 
       do_check_eq(getInstalledVersion(), 1);
       do_check_eq(getInstallReason(), ADDON_DOWNGRADE);
+      do_check_eq(getInstallOldVersion(), 4);
       do_check_eq(getActiveVersion(), 1);
 
       AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
@@ -1199,9 +1348,9 @@ function run_test_26() {
         do_check_true(b1.isActive);
         do_check_eq(b1.pendingOperations, AddonManager.PENDING_NONE);
 
-        run_test_27();
+        do_check_bootstrappedPref(run_test_27);
       });
-    });
+    }));
   });
 }
 
@@ -1221,9 +1370,10 @@ function run_test_27() {
       // Updating disabled things happens immediately
       do_check_eq(getInstalledVersion(), 0);
       do_check_eq(getUninstallReason(), ADDON_UPGRADE);
+      do_check_eq(getUninstallNewVersion(), 4);
       do_check_eq(getActiveVersion(), 0);
 
-      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+      AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
         do_check_neq(b1, null);
         do_check_eq(b1.version, "4.0");
         do_check_false(b1.isActive);
@@ -1240,26 +1390,29 @@ function run_test_27() {
           do_check_false(b1.isActive);
           do_check_eq(b1.pendingOperations, AddonManager.PENDING_NONE);
 
-          run_test_28();
+          do_check_bootstrappedPref(run_test_28);
         });
-      });
+      }));
     });
   });
 }
 
 // Tests that updating from a normal add-on to a bootstrappable add-on when
-// disabled calls the install method
+// disabled calls the install method but not the startup method
 function run_test_28() {
   installAllFiles([do_get_addon("test_bootstrap1_1")], function() {
+   do_execute_soon(function bootstrap_disabled_downgrade_check() {
     // Doesn't need a restart to complete this
     do_check_eq(getInstalledVersion(), 1);
     do_check_eq(getInstallReason(), ADDON_DOWNGRADE);
+    do_check_eq(getInstallOldVersion(), 4);
     do_check_eq(getActiveVersion(), 0);
 
-    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
+    AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", callback_soon(function(b1) {
       do_check_neq(b1, null);
       do_check_eq(b1.version, "1.0");
       do_check_false(b1.isActive);
+      do_check_true(b1.userDisabled);
       do_check_eq(b1.pendingOperations, AddonManager.PENDING_NONE);
 
       restartManager();
@@ -1269,6 +1422,7 @@ function run_test_28() {
 
       AddonManager.getAddonByID("bootstrap1@tests.mozilla.org", function(b1) {
         do_check_neq(b1, null);
+        do_check_true(b1.userDisabled);
         b1.userDisabled = false;
         do_check_eq(b1.version, "1.0");
         do_check_true(b1.isActive);
@@ -1276,8 +1430,9 @@ function run_test_28() {
         do_check_eq(getInstalledVersion(), 1);
         do_check_eq(getActiveVersion(), 1);
 
-        do_test_finished();
+        do_check_bootstrappedPref(do_test_finished);
       });
-    });
+    }));
+   });
   });
 }
