@@ -1,43 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is spacetrace.h/spacetrace.c code, released
- * Nov 6, 2001.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Garrett Arch Blythe, 31-October-2001
- *   Suresh Duddi <dp@netscape.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifndef spacetrace_h__
 #define spacetrace_h__
@@ -52,6 +17,7 @@
 /*
 ** Required includes.
 */
+#include <stdint.h>
 #include "nspr.h"
 #include "prlock.h"
 #include "prrwlock.h"
@@ -179,15 +145,15 @@
 */
 #define ST_TIMEVAL_RESOLUTION 1000
 #define ST_TIMEVAL_FORMAT "%.3f"
-#define ST_TIMEVAL_PRINTABLE(timeval) ((PRFloat64)(timeval) / (PRFloat64)ST_TIMEVAL_RESOLUTION)
-#define ST_TIMEVAL_PRINTABLE64(timeval) ((PRFloat64)((PRInt64)(timeval)) / (PRFloat64)ST_TIMEVAL_RESOLUTION)
-#define ST_TIMEVAL_MAX ((PRUint32)-1 - ((PRUint32)-1 % ST_TIMEVAL_RESOLUTION))
+#define ST_TIMEVAL_PRINTABLE(timeval) ((double)(timeval) / (double)ST_TIMEVAL_RESOLUTION)
+#define ST_TIMEVAL_PRINTABLE64(timeval) ((double)((int64_t)(timeval)) / (double)ST_TIMEVAL_RESOLUTION)
+#define ST_TIMEVAL_MAX ((uint32_t)-1 - ((uint32_t)-1 % ST_TIMEVAL_RESOLUTION))
 
 #define ST_MICROVAL_RESOLUTION 1000000
 #define ST_MICROVAL_FORMAT "%.6f"
-#define ST_MICROVAL_PRINTABLE(timeval) ((PRFloat64)(timeval) / (PRFloat64)ST_MICROVAL_RESOLUTION)
-#define ST_MICROVAL_PRINTABLE64(timeval) ((PRFloat64)((PRInt64)(timeval)) / (PRFloat64)ST_MICROVAL_RESOLUTION)
-#define ST_MICROVAL_MAX ((PRUint32)-1 - ((PRUint32)-1 % ST_MICROVAL_RESOLUTION))
+#define ST_MICROVAL_PRINTABLE(timeval) ((double)(timeval) / (double)ST_MICROVAL_RESOLUTION)
+#define ST_MICROVAL_PRINTABLE64(timeval) ((double)((int64_t)(timeval)) / (double)ST_MICROVAL_RESOLUTION)
+#define ST_MICROVAL_MAX ((uint32_t)-1 - ((uint32_t)-1 % ST_MICROVAL_RESOLUTION))
 
 /*
 ** Forward Declaration
@@ -214,21 +180,21 @@ typedef struct __struct_STAllocEvent
         **  relation to other allocation events.  This is a time stamp
         **  of sorts.
         */
-        PRUint32 mTimeval;
+        uint32_t mTimeval;
         
         /*
         ** Every event has a heap ID (pointer).
         ** In the event of a realloc, this is the new heap ID.
         ** In the event of a free, this is the previous heap ID value.
         */
-        PRUint32 mHeapID;
+        uint32_t mHeapID;
         
         /*
         ** Every event, along with the heap ID, tells of the size.
         ** In the event of a realloc, this is the new size.
         ** In th event of a free, this is the previous size.
         */
-        PRUint32 mHeapSize;
+        uint32_t mHeapSize;
 
         /*
         ** Every event has a callsite/stack backtrace.
@@ -252,19 +218,19 @@ typedef struct __struct_STAllocation
         /*
         ** The array of events.
         */
-        PRUint32 mEventCount;
+        uint32_t mEventCount;
         STAllocEvent* mEvents;
         
         /*
         ** The lifetime/lifespan of the allocation.
         */
-        PRUint32 mMinTimeval;
-        PRUint32 mMaxTimeval;
+        uint32_t mMinTimeval;
+        uint32_t mMaxTimeval;
 
         /*
         ** Index of this allocation in the global run.
         */
-        PRUint32 mRunIndex;
+        uint32_t mRunIndex;
 
         /*
         ** The runtime cost of heap events in this allocation.
@@ -272,7 +238,7 @@ typedef struct __struct_STAllocation
         **  spent in heap code (time of malloc, free, et al.).
         ** We do not track individual event cost in order to save space.
         */
-        PRUint32 mHeapRuntimeCost;
+        uint32_t mHeapRuntimeCost;
 } STAllocation;
 
 /*
@@ -286,40 +252,40 @@ typedef struct __struct_STCallsiteStats
         ** Sum timeval of the allocations.
         ** Callsite runs total all allocations below the callsite.
         */
-        PRUint64 mTimeval64;
+        uint64_t mTimeval64;
 
         /*
         ** Sum weight of the allocations.
         ** Callsite runs total all allocations below the callsite.
         */
-        PRUint64 mWeight64;
+        uint64_t mWeight64;
 
         /*
         ** Sum size of the allocations.
         ** Callsite runs total all allocations below the callsite.
         */
-        PRUint32 mSize;
+        uint32_t mSize;
 
         /*
         ** A stamp, indicated the relevance of the run.
         ** If the stamp does not match the origin value, the
         **  data contained here-in is considered invalid.
         */
-        PRUint32 mStamp;
+        uint32_t mStamp;
 
         /*
         ** A sum total of allocations (note, not sizes)  below the callsite.
         ** This is NOT the same as STRun::mAllocationCount which
         **  tracks the STRun::mAllocations array size.
         */
-        PRUint32 mCompositeCount;
+        uint32_t mCompositeCount;
 
         /*
         ** A sum total runtime cost of heap operations below the calliste.
         ** The cost is defined as the number of time units recorded as being
         **  spent in heap code (time of malloc, free, et al.).
         */
-        PRUint32 mHeapRuntimeCost;
+        uint32_t mHeapRuntimeCost;
 } STCallsiteStats;
 
 /*
@@ -337,7 +303,7 @@ typedef struct __struct_STRun
         /*
         ** The array of allocations.
         */
-        PRUint32 mAllocationCount;
+        uint32_t mAllocationCount;
         STAllocation** mAllocations;
 
         /*
@@ -379,7 +345,7 @@ struct __struct_STCategoryNode
         ** NULL if leaf node.
         */
         STCategoryNode** children;
-        PRUint32 nchildren;
+        uint32_t nchildren;
 
         /*
         ** The Run(s). Valid for both leaf and parent nodes.
@@ -398,8 +364,8 @@ struct __struct_STCategoryRule
         ** A callsite needs to pass substring match for all the strings.
         */
         char* pats[ST_MAX_PATTERNS_PER_RULE];
-        PRUint32 patlen[ST_MAX_PATTERNS_PER_RULE];
-        PRUint32 npats;
+        uint32_t patlen[ST_MAX_PATTERNS_PER_RULE];
+        uint32_t npats;
 
         /*
         ** Category name that this rule belongs to
@@ -458,9 +424,9 @@ STOptionGenre;
 #define ST_CMD_OPTION_BOOL(option_name, option_genre, option_help) PRBool m##option_name;
 #define ST_CMD_OPTION_STRING(option_name, option_genre, default_value, option_help) char m##option_name[ST_OPTION_STRING_MAX];
 #define ST_CMD_OPTION_STRING_ARRAY(option_name, option_genre, array_size, option_help) char m##option_name[array_size][ST_OPTION_STRING_MAX];
-#define ST_CMD_OPTION_STRING_PTR_ARRAY(option_name, option_genre, option_help) const char** m##option_name; PRUint32 m##option_name##Count;
-#define ST_CMD_OPTION_UINT32(option_name, option_genre, default_value, multiplier, option_help) PRUint32 m##option_name;
-#define ST_CMD_OPTION_UINT64(option_name, option_genre, default_value, multiplier, option_help) PRUint64 m##option_name##64;
+#define ST_CMD_OPTION_STRING_PTR_ARRAY(option_name, option_genre, option_help) const char** m##option_name; uint32_t m##option_name##Count;
+#define ST_CMD_OPTION_UINT32(option_name, option_genre, default_value, multiplier, option_help) uint32_t m##option_name;
+#define ST_CMD_OPTION_UINT64(option_name, option_genre, default_value, multiplier, option_help) uint64_t m##option_name##64;
 
 typedef struct __struct_STOptions
 {
@@ -508,7 +474,7 @@ typedef struct __struct_STContext
 */
 {
     PRRWLock* mRWLock;
-    PRUint32 mIndex;
+    uint32_t mIndex;
     STRun* mSortedRun;
 #if ST_WANT_GRAPHS
     PRLock* mImageLock;
@@ -516,10 +482,10 @@ typedef struct __struct_STContext
     PRBool mTimevalCached;
     PRBool mLifespanCached;
     PRBool mWeightCached;
-    PRUint32 mFootprintYData[STGD_SPACE_X];
-    PRUint32 mTimevalYData[STGD_SPACE_X];
-    PRUint32 mLifespanYData[STGD_SPACE_X];
-    PRUint64 mWeightYData64[STGD_SPACE_X];
+    uint32_t mFootprintYData[STGD_SPACE_X];
+    uint32_t mTimevalYData[STGD_SPACE_X];
+    uint32_t mLifespanYData[STGD_SPACE_X];
+    uint64_t mWeightYData64[STGD_SPACE_X];
 #endif
 }
 STContext;
@@ -550,7 +516,7 @@ typedef struct __struct_STContextCacheItem
 {
     STOptions mOptions;
     STContext mContext;
-    PRInt32 mReferenceCount;
+    int32_t mReferenceCount;
     PRIntervalTime mLastAccessed;
     PRBool mInUse;
 }
@@ -575,7 +541,7 @@ typedef struct __struct_STContextCache
     PRLock* mLock;
     PRCondVar* mCacheMiss;
     STContextCacheItem* mItems;
-    PRUint32 mItemCount;
+    uint32_t mItemCount;
 }
 STContextCache;
 
@@ -643,15 +609,15 @@ typedef struct __struct_STGlobals
         /*
         ** Various counters for different types of events.
         */
-        PRUint32 mMallocCount;
-        PRUint32 mCallocCount;
-        PRUint32 mReallocCount;
-        PRUint32 mFreeCount;
+        uint32_t mMallocCount;
+        uint32_t mCallocCount;
+        uint32_t mReallocCount;
+        uint32_t mFreeCount;
 
         /*
         ** Total events, operation counter.
         */
-        PRUint32 mOperationCount;
+        uint32_t mOperationCount;
 
         /*
         ** The "run" of the input.
@@ -663,26 +629,26 @@ typedef struct __struct_STGlobals
         ** So that we can determine the overall timeval of the run.
         ** NOTE:  These are NOT the options to control the data set.
         */
-        PRUint32 mMinTimeval;
-        PRUint32 mMaxTimeval;
+        uint32_t mMinTimeval;
+        uint32_t mMaxTimeval;
 
         /*
         ** Calculates peak allocation overall for all allocations.
         */
-        PRUint32 mPeakMemoryUsed;
-        PRUint32 mMemoryUsed;
+        uint32_t mPeakMemoryUsed;
+        uint32_t mMemoryUsed;
 
         /*
         ** A list of rules for categorization read in from the mCategoryFile
         */
        STCategoryRule** mCategoryRules;
-       PRUint32 mNRules;
+       uint32_t mNRules;
 
        /*
        ** CategoryName to Node mapping table
        */
        STCategoryMapEntry** mCategoryMap;
-       PRUint32 mNCategoryMap;
+       uint32_t mNCategoryMap;
 
        /*
        ** Categorized allocations. For now we support only one tree.
@@ -701,7 +667,7 @@ typedef struct __struct_STGlobals
 /*
 ** Function prototypes
 */
-extern STRun* createRun(STContext* inContext, PRUint32 aStamp);
+extern STRun* createRun(STContext* inContext, uint32_t aStamp);
 extern void freeRun(STRun* aRun);
 extern int initCategories(STGlobals* g);
 extern int categorizeRun(STOptions* inOptions, STContext* inContext, const STRun* aRun, STGlobals* g);
@@ -718,7 +684,7 @@ extern void htmlAnchor(STRequest* inRequest,
                        const char* aTarget,
                        const char* aClass,
                        STOptions* inOptions);
-extern char *FormatNumber(PRInt32 num);
+extern char *FormatNumber(int32_t num);
 
 /*
 ** shared globals
