@@ -1,3 +1,7 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 import os
 import shutil
 import sha
@@ -16,94 +20,91 @@ class PatchInfo:
     """ Represents the meta-data associated with a patch
         work_dir = working dir where files are stored for this patch
         archive_files = list of files to include in this patch
-        manifestv1 = set of manifest version 1 patch instructions
         manifestv2 = set of manifest version 2 patch instructions
+        manifestv3 = set of manifest version 3 patch instructions
         file_exclusion_list = 
         files to exclude from this patch. names without slashes will be
-                              excluded anywhere in the directory hiearchy.   names with slashes
-                              will only be excluded at that exact path
+        excluded anywhere in the directory hiearchy.   names with slashes
+        will only be excluded at that exact path
         """
     def __init__(self, work_dir, file_exclusion_list, path_exclusion_list):
         self.work_dir=work_dir
         self.archive_files=[]
-        self.manifestv1=[]
         self.manifestv2=[]
+        self.manifestv3=[]
         self.file_exclusion_list=file_exclusion_list
         self.path_exclusion_list=path_exclusion_list
-        
+
     def append_add_instruction(self, filename):
-        """ Appends an add instruction for this patch.   
-            if the filename starts with extensions/ adds an add-if instruction
-            to test the existence of the subdirectory.  This was ported from
-            mozilla/tools/update-packaging/common.sh/make_add_instruction
+        """ Appends an add instruction for this patch.
+            if filename starts with distribution/extensions/.*/ this will add an
+            add-if instruction that will add the file if the parent directory
+            of the file exists.  This was ported from
+            mozilla/tools/update-packaging/common.sh's make_add_instruction.
         """
-        if filename.startswith("extensions/"):
+        m = re.match("((?:|.*/)distribution/extensions/.*)/", filename)
+        if m:
             # Directory immediately following extensions is used for the test
-            testdir = "extensions/"+filename.split("/")[1]
-            self.manifestv1.append('add-if "'+testdir+'" "'+filename+'"')
-            self.manifestv2.append('add-if "'+testdir+'" "'+filename+'"')
-        elif filename.startswith("Contents/MacOS/extensions/"):
-            testdir = "Contents/MacOS/extensions/"+filename.split("/")[3]
-            self.manifestv1.append('add-if "'+testdir+'" "'+filename+'"')
+            testdir = m.group(1)
+            print '     add-if "'+testdir+'" "'+filename+'"'
             self.manifestv2.append('add-if "'+testdir+'" "'+filename+'"')
+            self.manifestv3.append('add-if "'+testdir+'" "'+filename+'"')
         else:
-            self.manifestv1.append('add "'+filename+'"')
+            print '        add "'+filename+'"'
             self.manifestv2.append('add "'+filename+'"')
-           
+            self.manifestv3.append('add "'+filename+'"')
+
+    def append_add_if_not_instruction(self, filename):
+        """ Appends an add-if-not instruction to the version 3 manifest for this patch.
+            This was ported from mozilla/tools/update-packaging/common.sh's
+            make_add_if_not_instruction.
+        """
+        print ' add-if-not "'+filename+'" "'+filename+'"'
+        self.manifestv3.append('add-if-not "'+filename+'" "'+filename+'"')
+
     def append_patch_instruction(self, filename, patchname):
-        """ Appends an patch instruction for this patch.   
-            
+        """ Appends a patch instruction for this patch.
+
             filename = file to patch
             patchname = patchfile to apply to file
-            
-            if the filename starts with extensions/ adds a patch-if instruction
-            to test the existence of the subdirectory.  
-            if the filename starts with searchplugins/ add a add-if instruction for the filename
-            This was ported from
-            mozilla/tools/update-packaging/common.sh/make_patch_instruction
+
+            if filename starts with distribution/extensions/.*/ this will add a
+            patch-if instruction that will patch the file if the parent
+            directory of the file exists. This was ported from
+            mozilla/tools/update-packaging/common.sh's make_patch_instruction.
         """
-        if filename.startswith("extensions/"):
-            testdir = "extensions/"+filename.split("/")[1]
-            self.manifestv1.append('patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"')
+        m = re.match("((?:|.*/)distribution/extensions/.*)/", filename)
+        if m:
+            testdir = m.group(1)
+            print '   patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"'
             self.manifestv2.append('patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"')
-        elif filename.startswith("Contents/MacOS/extensions/"):
-            testdir = "Contents/MacOS/extensions/"+filename.split("/")[3]
-            self.manifestv1.append('patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"')
-            self.manifestv2.append('patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"')
-        elif (filename.startswith("searchplugins/") or
-             filename.startswith("Contents/MacOS/searchplugins/")):
-            self.manifestv1.append('patch-if "'+filename+'" "'+patchname+'" "'+filename+'"')
-            self.manifestv2.append('patch-if "'+filename+'" "'+patchname+'" "'+filename+'"')
+            self.manifestv3.append('patch-if "'+testdir+'" "'+patchname+'" "'+filename+'"')
         else:
-            self.manifestv1.append('patch "'+patchname+'" "'+filename+'"')
+            print '      patch "'+patchname+'" "'+filename+'"'
             self.manifestv2.append('patch "'+patchname+'" "'+filename+'"')
-                
+            self.manifestv3.append('patch "'+patchname+'" "'+filename+'"')
+
     def append_remove_instruction(self, filename):
-        """ Appends an remove instruction for this patch.   
+        """ Appends an remove instruction for this patch.
             This was ported from
             mozilla/tools/update-packaging/common.sh/make_remove_instruction
         """
         if filename.endswith("/"):
+            print '      rmdir "'+filename+'"'
             self.manifestv2.append('rmdir "'+filename+'"')
+            self.manifestv3.append('rmdir "'+filename+'"')
         elif filename.endswith("/*"):
             filename = filename[:-1]
+            print '    rmrfdir "'+filename+'"'
             self.manifestv2.append('rmrfdir "'+filename+'"')
+            self.manifestv3.append('rmrfdir "'+filename+'"')
         else:
-            self.manifestv1.append('remove "'+filename+'"')
+            print '     remove "'+filename+'"'
             self.manifestv2.append('remove "'+filename+'"')
+            self.manifestv3.append('remove "'+filename+'"')
 
     def create_manifest_files(self):
-        """ Createst the v1 manifest file in the root of the work_dir """
-        manifest_file_path = os.path.join(self.work_dir,"update.manifest")
-        manifest_file = open(manifest_file_path, "wb")
-        manifest_file.writelines(string.join(self.manifestv1, '\n'))
-        manifest_file.writelines("\n")
-        manifest_file.close()    
-
-        bzip_file(manifest_file_path)
-        self.archive_files.append('"update.manifest"')
-
-        """ Createst the v2 manifest file in the root of the work_dir """
+        """ Create the v2 manifest file in the root of the work_dir """
         manifest_file_path = os.path.join(self.work_dir,"updatev2.manifest")
         manifest_file = open(manifest_file_path, "wb")
         manifest_file.writelines("type \"partial\"\n")
@@ -114,6 +115,17 @@ class PatchInfo:
         bzip_file(manifest_file_path)
         self.archive_files.append('"updatev2.manifest"')
 
+        """ Create the v3 manifest file in the root of the work_dir """
+        manifest_file_path = os.path.join(self.work_dir,"updatev3.manifest")
+        manifest_file = open(manifest_file_path, "wb")
+        manifest_file.writelines("type \"partial\"\n")
+        manifest_file.writelines(string.join(self.manifestv3, '\n'))
+        manifest_file.writelines("\n")
+        manifest_file.close()
+
+        bzip_file(manifest_file_path)
+        self.archive_files.append('"updatev3.manifest"')
+
     def build_marfile_entry_hash(self, root_path):
         """ Iterates through the root_path, creating a MarFileEntry for each file
             and directory in that path.  Excludes any filenames in the file_exclusion_list
@@ -143,9 +155,9 @@ class PatchInfo:
 
         return mar_entry_hash, filename_set, dirname_set
  
-       
+
 class MarFileEntry:
-    """Represents a file inside a Mozilla Archive Format (MAR)
+    """Represents a file inside a GNU Archive Format (MAR)
         abs_path = abspath to the the file
         name =  relative path within the mar.  e.g.
           foo.mar/dir/bar.txt extracted into /tmp/foo:
@@ -162,7 +174,7 @@ class MarFileEntry:
     def __str__(self):
         return 'Name: %s FullPath: %s' %(self.name,self.abs_path)
 
-    def calc_file_sha_digest(self, filename):        
+    def calc_file_sha_digest(self, filename):
         """ Returns sha digest of given filename"""
         file_content = open(filename, 'r').read()
         return sha.new(file_content).digest()
@@ -192,7 +204,7 @@ def bzip_file(filename):
         assumes the path is absolute"""
     exec_shell_cmd('bzip2 -z9 "' + filename+'"')
     os.rename(filename+".bz2",filename)
-    
+
 def bunzip_file(filename):
     """ Bzip's the file in palce.   The original file is replaced with a bunzip'd version of itself.
         doesn't matter if the filename ends in .bz2 or not"""
@@ -209,7 +221,7 @@ def extract_mar(filename, work_dir):
     saved_path = os.getcwd()
     try:
         os.chdir(work_dir)
-        exec_shell_cmd("mar -x "+filename)    
+        exec_shell_cmd("mar -x "+filename)
     finally:
         os.chdir(saved_path)
 
@@ -217,8 +229,7 @@ def create_partial_patch_for_file(from_marfile_entry, to_marfile_entry, shas, pa
     """ Creates the partial patch file and manifest entry for the pair of files passed in
     """
     if not (from_marfile_entry.sha(),to_marfile_entry.sha()) in shas:
-        print "diffing: " + from_marfile_entry.name
-        
+        print 'diffing "'+from_marfile_entry.name+'\"'
         #bunzip to/from
         bunzip_file(from_marfile_entry.abs_path)
         bunzip_file(to_marfile_entry.abs_path)
@@ -235,48 +246,52 @@ def create_partial_patch_for_file(from_marfile_entry, to_marfile_entry, shas, pa
         bzip_file(patch_file_abs_path)
 
         # Create bzip's full file
-        full_file_abs_path =  os.path.join(patch_info.work_dir, to_marfile_entry.name)   
+        full_file_abs_path =  os.path.join(patch_info.work_dir, to_marfile_entry.name)
         shutil.copy2(to_marfile_entry.abs_path, full_file_abs_path)
         bzip_file(full_file_abs_path)
-  
-        ## TOODO NEED TO ADD HANDLING FOR FORCED UPDATES
+
         if os.path.getsize(patch_file_abs_path) < os.path.getsize(full_file_abs_path):
             # Patch is smaller than file.  Remove the file and add patch to manifest
             os.remove(full_file_abs_path)
             file_in_manifest_name = from_marfile_entry.name+".patch"
             file_in_manifest_abspath = patch_file_abs_path
             patch_info.append_patch_instruction(to_marfile_entry.name, file_in_manifest_name)
-        else:            
+        else:
             # File is smaller than patch.  Remove the patch and add file to manifest
             os.remove(patch_file_abs_path)
             file_in_manifest_name = from_marfile_entry.name
             file_in_manifest_abspath = full_file_abs_path
             patch_info.append_add_instruction(file_in_manifest_name)
-            
+
         shas[from_marfile_entry.sha(),to_marfile_entry.sha()] = (file_in_manifest_name,file_in_manifest_abspath)
-        patch_info.archive_files.append('"'+file_in_manifest_name+'"')        
+        patch_info.archive_files.append('"'+file_in_manifest_name+'"')
     else:
         filename, src_file_abs_path = shas[from_marfile_entry.sha(),to_marfile_entry.sha()]
-        # We've already calculated the patch for this pair of files.   
+        # We've already calculated the patch for this pair of files.
         if (filename.endswith(".patch")):
-            print "skipping diff: " + from_marfile_entry.name
+            # print "skipping diff: "+from_marfile_entry.name
             # Patch was smaller than file - add patch instruction to manifest
             file_in_manifest_name = to_marfile_entry.name+'.patch';
             patch_info.append_patch_instruction(to_marfile_entry.name, file_in_manifest_name)
         else:
             # File was smaller than file - add file to manifest
             file_in_manifest_name = to_marfile_entry.name
-            patch_info.append_add_instruction(file_in_manifest_name)                
+            patch_info.append_add_instruction(file_in_manifest_name)
         # Copy the pre-calculated file into our new patch work aread
         copy_file(src_file_abs_path, os.path.join(patch_info.work_dir, file_in_manifest_name))
         patch_info.archive_files.append('"'+file_in_manifest_name+'"')
  
-def create_add_patch_for_file(to_marfile_entry, patch_info):           
+def create_add_patch_for_file(to_marfile_entry, patch_info):
     """  Copy the file to the working dir, add the add instruction, and add it to the list of archive files """
-    print "Adding New File " + to_marfile_entry.name    
     copy_file(to_marfile_entry.abs_path, os.path.join(patch_info.work_dir, to_marfile_entry.name))
     patch_info.append_add_instruction(to_marfile_entry.name)
-    patch_info.archive_files.append('"'+to_marfile_entry.name+'"')    
+    patch_info.archive_files.append('"'+to_marfile_entry.name+'"')
+
+def create_add_if_not_patch_for_file(to_marfile_entry, patch_info):
+    """  Copy the file to the working dir, add the add-if-not instruction, and add it to the list of archive files """
+    copy_file(to_marfile_entry.abs_path, os.path.join(patch_info.work_dir, to_marfile_entry.name))
+    patch_info.append_add_if_not_instruction(to_marfile_entry.name)
+    patch_info.archive_files.append('"'+to_marfile_entry.name+'"')
 
 def process_explicit_remove_files(dir_path, patch_info): 
     """ Looks for a 'removed-files' file in the dir_path.  If the removed-files does not exist
@@ -313,15 +328,15 @@ def process_explicit_remove_files(dir_path, patch_info):
                 line = line.replace("\\", "/")
                 patch_info.append_remove_instruction(line)
 
-def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch_info, forced_updates):
-    """ Builds a partial patch by comparing the files in from_dir_path to thoes of to_dir_path"""
+def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch_info, forced_updates, add_if_not_list):
+    """ Builds a partial patch by comparing the files in from_dir_path to those of to_dir_path"""
     # Cannocolize the paths for safey
     from_dir_path = os.path.abspath(from_dir_path)
     to_dir_path = os.path.abspath(to_dir_path)
     # Create a hashtable of the from  and to directories
     from_dir_hash,from_file_set,from_dir_set = patch_info.build_marfile_entry_hash(from_dir_path)
     to_dir_hash,to_file_set,to_dir_set = patch_info.build_marfile_entry_hash(to_dir_path)
-    # Require that the precomplete file is included in the to complete update
+    # Require that the precomplete file is included in the complete update
     if "precomplete" not in to_file_set:
         raise Exception, "missing precomplete file in: "+to_dir_path
     # Create a list of the forced updates 
@@ -334,8 +349,11 @@ def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch
     for filename in patch_filenames:
         from_marfile_entry = from_dir_hash[filename]
         to_marfile_entry = to_dir_hash[filename]
-        if filename in forced_list:
-            print "Forcing "+ filename
+        if os.path.basename(filename) in add_if_not_list:
+            # This filename is in the add if not list, explicitly add-if-not
+            create_add_if_not_patch_for_file(to_dir_hash[filename], patch_info)
+        elif filename in forced_list:
+            print 'Forcing "'+filename+'"'
             # This filename is in the forced list, explicitly add
             create_add_patch_for_file(to_dir_hash[filename], patch_info)
         else: 
@@ -347,7 +365,10 @@ def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch
     add_filenames = list(to_file_set - from_file_set)
     add_filenames.sort(reverse=True)
     for filename in add_filenames:
-        create_add_patch_for_file(to_dir_hash[filename], patch_info)
+        if os.path.basename(filename) in add_if_not_list:
+            create_add_if_not_patch_for_file(to_dir_hash[filename], patch_info)
+        else:
+            create_add_patch_for_file(to_dir_hash[filename], patch_info)
 
     # files in from_dir not in to_dir need to be removed
     remove_filenames = list(from_file_set - to_file_set)
@@ -356,7 +377,7 @@ def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch
         patch_info.append_remove_instruction(from_dir_hash[filename].name)
 
     process_explicit_remove_files(to_dir_path, patch_info)
-    
+
     # directories in from_dir not in to_dir need to be removed
     remove_dirnames = list(from_dir_set - to_dir_set)
     remove_dirnames.sort(reverse=True)
@@ -365,9 +386,9 @@ def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch
 
     # Construct the Manifest files
     patch_info.create_manifest_files()
-    
+
     # And construct the mar
-    mar_cmd = 'mar -C '+patch_info.work_dir+' -c output.mar '+string.join(patch_info.archive_files, ' ')    
+    mar_cmd = 'mar -C '+patch_info.work_dir+' -c output.mar '+string.join(patch_info.archive_files, ' ')
     exec_shell_cmd(mar_cmd)
 
     # Copy mar to final destination
@@ -376,7 +397,7 @@ def create_partial_patch(from_dir_path, to_dir_path, patch_filename, shas, patch
         os.makedirs(patch_file_dir)
     shutil.copy2(os.path.join(patch_info.work_dir,"output.mar"), patch_filename)
     return patch_filename
-    
+
 def usage():
     print "-h for help"
     print "-f for patchlist_file"
@@ -401,8 +422,8 @@ def get_buildid(work_dir, platform):
 
 def decode_filename(filepath):
     """ Breaks filename/dir structure into component parts based on regex
-        for example: firefox-3.0b3pre.en-US.linux-i686.complete.mar
-        Or linux-i686/en-US/firefox-3.0b3.complete.mar
+        for example: icecat-3.0b3pre.en-US.linux-i686.complete.mar
+        Or linux-i686/en-US/icecat-3.0b3.complete.mar
         Returns dict with keys product, version, locale, platform, type
     """
     try:
@@ -449,7 +470,7 @@ def create_partial_patches(patches):
             from_buildid = get_buildid(work_dir_from, from_decoded['platform'])
             from_shasum = sha.sha(open(from_filename).read()).hexdigest()
             from_size = str(os.path.getsize(to_filename))
-            
+
             # Extract to mar into to dir
             work_dir_to =  os.path.join(work_dir,"to")
             os.mkdir(work_dir_to)
@@ -461,7 +482,7 @@ def create_partial_patches(patches):
 
             mar_extract_time = time.time()
 
-            partial_filename = create_partial_patch(work_dir_from, work_dir_to, patch_filename, shas, PatchInfo(work_dir, ['channel-prefs.js','update.manifest','updatev2.manifest','removed-files'],['/readme.txt']),forced_updates)
+            partial_filename = create_partial_patch(work_dir_from, work_dir_to, patch_filename, shas, PatchInfo(work_dir, ['update.manifest','updatev2.manifest','updatev3.manifest','removed-files'],['/readme.txt']),forced_updates,['channel-prefs.js','update-settings.ini'])
             partial_buildid = to_buildid
             partial_shasum = sha.sha(open(partial_filename).read()).hexdigest()
             partial_size = str(os.path.getsize(partial_filename))
@@ -489,26 +510,26 @@ def create_partial_patches(patches):
     finally:
         # If we fail or get a ctrl-c during run be sure to clean up temp dir
         if (work_dir_root and os.path.exists(work_dir_root)):
-            shutil.rmtree(work_dir_root)        
+            shutil.rmtree(work_dir_root)
 
-def main(argv):                          
+def main(argv):
     patchlist_file = None
-    try:                                
+    try:
          opts, args = getopt.getopt(argv, "hf:", ["help", "patchlist_file="])
-         for opt, arg in opts:                
-            if opt in ("-h", "--help"):      
-                usage()                     
-                sys.exit()       
+         for opt, arg in opts:
+            if opt in ("-h", "--help"):
+                usage()
+                sys.exit()
             elif opt in ("-f", "--patchlist_file"):
-                patchlist_file = arg               
-    except getopt.GetoptError:          
-          usage()                         
-          sys.exit(2)                     
-    
+                patchlist_file = arg
+    except getopt.GetoptError:
+          usage()
+          sys.exit(2)
+
     if not patchlist_file:
         usage()
         sys.exit(2)
-        
+
     patches = []
     f = open(patchlist_file, 'r')
     for line in f.readlines():
