@@ -1,52 +1,20 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Application Update.
- * This file is based on code originally located at
- * mozilla/toolkit/mozapps/update/updater/updater.cpp
- *
- * The Initial Developer of the Original Code is
- * Benjamin Smedberg <benjamin@smedbergs.us>
- *
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Darin Fisher <darin@meer.net>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsCRTGlue.h"
 #include "nsXPCOM.h"
 #include "nsDebug.h"
+#include "prtime.h"
+
+#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdarg.h>
 
 #ifdef XP_WIN
 #include <io.h>
+#include <windows.h>
 #endif
 
 #ifdef ANDROID
@@ -73,13 +41,13 @@ char*
 NS_strtok(const char *delims, char **str)
 {
   if (!*str)
-    return NULL;
+    return nullptr;
 
   char *ret = (char*) NS_strspnp(delims, *str);
 
   if (!*ret) {
     *str = ret;
-    return NULL;
+    return nullptr;
   }
 
   char *i = ret;
@@ -94,14 +62,14 @@ NS_strtok(const char *delims, char **str)
     ++i;
   } while (*i);
 
-  *str = NULL;
+  *str = nullptr;
   return ret;
 }
 
-PRUint32
-NS_strlen(const PRUnichar *aString)
+uint32_t
+NS_strlen(const char16_t *aString)
 {
-  const PRUnichar *end;
+  const char16_t *end;
 
   for (end = aString; *end; ++end) {
     // empty loop
@@ -111,7 +79,7 @@ NS_strlen(const PRUnichar *aString)
 }
 
 int
-NS_strcmp(const PRUnichar *a, const PRUnichar *b)
+NS_strcmp(const char16_t *a, const char16_t *b)
 {
   while (*b) {
     int r = *a - *b;
@@ -125,19 +93,19 @@ NS_strcmp(const PRUnichar *a, const PRUnichar *b)
   return *a != '\0';
 }
 
-PRUnichar*
-NS_strdup(const PRUnichar *aString)
+char16_t*
+NS_strdup(const char16_t *aString)
 {
-  PRUint32 len = NS_strlen(aString);
+  uint32_t len = NS_strlen(aString);
   return NS_strndup(aString, len);
 }
 
-PRUnichar*
-NS_strndup(const PRUnichar *aString, PRUint32 aLen)
+char16_t*
+NS_strndup(const char16_t *aString, uint32_t aLen)
 {
-  PRUnichar *newBuf = (PRUnichar*) NS_Alloc((aLen + 1) * sizeof(PRUnichar));
+  char16_t *newBuf = (char16_t*) NS_Alloc((aLen + 1) * sizeof(char16_t));
   if (newBuf) {
-    memcpy(newBuf, aString, aLen * sizeof(PRUnichar));
+    memcpy(newBuf, aString, aLen * sizeof(char16_t));
     newBuf[aLen] = '\0';
   }
   return newBuf;
@@ -146,7 +114,7 @@ NS_strndup(const PRUnichar *aString, PRUint32 aLen)
 char*
 NS_strdup(const char *aString)
 {
-  PRUint32 len = strlen(aString);
+  uint32_t len = strlen(aString);
   char *str = (char*) NS_Alloc(len + 1);
   if (str) {
     memcpy(str, aString, len);
@@ -205,59 +173,59 @@ const unsigned char nsLowerUpperUtils::kLower2Upper[256] = {
   240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255
 };
 
-PRBool NS_IsUpper(char aChar)
+bool NS_IsUpper(char aChar)
 {
   return aChar != (char)nsLowerUpperUtils::kUpper2Lower[(unsigned char)aChar];
 }
 
-PRBool NS_IsLower(char aChar)
+bool NS_IsLower(char aChar)
 {
   return aChar != (char)nsLowerUpperUtils::kLower2Upper[(unsigned char)aChar];
 }
 
-PRBool NS_IsAscii(PRUnichar aChar)
+bool NS_IsAscii(char16_t aChar)
 {
   return (0x0080 > aChar);
 }
 
-PRBool NS_IsAscii(const PRUnichar *aString)
+bool NS_IsAscii(const char16_t *aString)
 {
   while(*aString) {
     if( 0x0080 <= *aString)
-      return PR_FALSE;
+      return false;
     aString++;
   }
-  return PR_TRUE;
+  return true;
 }
 
-PRBool NS_IsAscii(const char *aString)
+bool NS_IsAscii(const char *aString)
 {
   while(*aString) {
     if( 0x80 & *aString)
-      return PR_FALSE;
+      return false;
     aString++;
   }
-  return PR_TRUE;
+  return true;
 }
 
-PRBool NS_IsAscii(const char* aString, PRUint32 aLength)
+bool NS_IsAscii(const char* aString, uint32_t aLength)
 {
   const char* end = aString + aLength;
   while (aString < end) {
     if (0x80 & *aString)
-      return PR_FALSE;
+      return false;
     ++aString;
   }
-  return PR_TRUE;
+  return true;
 }
 
-PRBool NS_IsAsciiAlpha(PRUnichar aChar)
+bool NS_IsAsciiAlpha(char16_t aChar)
 {
   return ((aChar >= 'A') && (aChar <= 'Z')) ||
          ((aChar >= 'a') && (aChar <= 'z'));
 }
 
-PRBool NS_IsAsciiWhitespace(PRUnichar aChar)
+bool NS_IsAsciiWhitespace(char16_t aChar)
 {
   return aChar == ' ' ||
          aChar == '\r' ||
@@ -265,42 +233,102 @@ PRBool NS_IsAsciiWhitespace(PRUnichar aChar)
          aChar == '\t';
 }
 
-PRBool NS_IsAsciiDigit(PRUnichar aChar)
+bool NS_IsAsciiDigit(char16_t aChar)
 {
   return aChar >= '0' && aChar <= '9';
 }
 
-#if defined(XP_WIN)
-void
-printf_stderr(const char *fmt, ...)
+
+#ifndef XPCOM_GLUE_AVOID_NSPR
+#define TABLE_SIZE 36
+static const char table[] = {
+  'a','b','c','d','e','f','g','h','i','j',
+  'k','l','m','n','o','p','q','r','s','t',
+  'u','v','w','x','y','z','0','1','2','3',
+  '4','5','6','7','8','9'
+};
+
+void NS_MakeRandomString(char *aBuf, int32_t aBufLen)
 {
+  // turn PR_Now() into milliseconds since epoch
+  // and salt rand with that.
+  static unsigned int seed = 0;
+  if (seed == 0) {
+    double fpTime = double(PR_Now());
+    seed = (unsigned int)(fpTime * 1e-6 + 0.5); // use 1e-6, granularity of PR_Now() on the mac is seconds
+    srand(seed);
+  }
+
+  int32_t i;
+  for (i=0;i<aBufLen;i++) {
+    *aBuf++ = table[rand()%TABLE_SIZE];
+  }
+  *aBuf = 0;
+}
+
+#endif
+#if defined(XP_WIN)
+
+#define va_copy(dest, src) (dest = src)
+
+void
+vprintf_stderr(const char *fmt, va_list args)
+{
+  if (IsDebuggerPresent()) {
+    char buf[2048];
+    va_list argsCpy;
+    va_copy(argsCpy, args);
+    vsnprintf(buf, sizeof(buf), fmt, argsCpy);
+    buf[sizeof(buf) - 1] = '\0';
+    va_end(argsCpy);
+    OutputDebugStringA(buf);
+  }
+
   FILE *fp = _fdopen(_dup(2), "a");
   if (!fp)
       return;
 
-  va_list args;
-  va_start(args, fmt);
   vfprintf(fp, fmt, args);
-  va_end(args);
 
   fclose(fp);
 }
+
+#undef va_copy
+
 #elif defined(ANDROID)
 void
-printf_stderr(const char *fmt, ...)
+vprintf_stderr(const char *fmt, va_list args)
 {
-  va_list args;
-  va_start(args, fmt);
   __android_log_vprint(ANDROID_LOG_INFO, "Gecko", fmt, args);
-  va_end(args);
 }
 #else
+void
+vprintf_stderr(const char *fmt, va_list args)
+{
+  vfprintf(stderr, fmt, args);
+}
+#endif
+
 void
 printf_stderr(const char *fmt, ...)
 {
   va_list args;
   va_start(args, fmt);
-  vfprintf(stderr, fmt, args);
+  vprintf_stderr(fmt, args);
   va_end(args);
 }
-#endif
+
+void
+fprintf_stderr(FILE* aFile, const char *fmt, ...)
+{
+  va_list args;
+  va_start(args, fmt);
+  if (aFile == stderr) {
+    vprintf_stderr(fmt, args);
+  } else {
+    vfprintf(aFile, fmt, args);
+  }
+  va_end(args);
+}
+
+
