@@ -1,48 +1,12 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Doug Turner <dougt@netscape.com>
- *   IBM Corp.
- *   Fredrik Holmqvist <thesuckiestemail@yahoo.se>
- *   Jungshik Shin <jshin@i18nl10n.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "SpecialSystemDirectory.h"
 #include "nsString.h"
 #include "nsDependentString.h"
+#include "nsAutoPtr.h"
 
 #if defined(XP_WIN)
 
@@ -52,19 +16,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <direct.h>
+#include <shlobj.h>
+#include <knownfolders.h>
+#include <guiddef.h>
+#include "mozilla/WindowsVersion.h"
 
-#elif defined(XP_OS2)
-
-#define MAX_PATH _MAX_PATH
-#define INCL_WINWORKPLACE
-#define INCL_DOSMISC
-#define INCL_DOSMODULEMGR
-#define INCL_DOSPROCESS
-#define INCL_WINSHELLDATA
-#include <os2.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include "prenv.h"
+using mozilla::IsWin7OrLater;
 
 #elif defined(XP_UNIX)
 
@@ -100,65 +57,50 @@ typedef HRESULT (WINAPI* nsGetKnownFolderPath)(GUID& rfid,
                                                HANDLE hToken,
                                                PWSTR *ppszPath);
 
-static nsGetKnownFolderPath gGetKnownFolderPath = NULL;
-
-static HINSTANCE gShell32DLLInst = NULL;
+static nsGetKnownFolderPath gGetKnownFolderPath = nullptr;
 #endif
 
-NS_COM void StartupSpecialSystemDirectory()
+void StartupSpecialSystemDirectory()
 {
 #if defined (XP_WIN)
     // SHGetKnownFolderPath is only available on Windows Vista
     // so that we need to use GetProcAddress to get the pointer.
-    gShell32DLLInst = LoadLibraryW(L"shell32.dll");
-    if(gShell32DLLInst)
+    HMODULE hShell32DLLInst = GetModuleHandleW(L"shell32.dll");
+    if(hShell32DLLInst)
     {
         gGetKnownFolderPath = (nsGetKnownFolderPath)
-            GetProcAddress(gShell32DLLInst, "SHGetKnownFolderPath");
-    }
-#endif
-}
-
-NS_COM void ShutdownSpecialSystemDirectory()
-{
-#if defined (XP_WIN)
-    if (gShell32DLLInst)
-    {
-        FreeLibrary(gShell32DLLInst);
-        gShell32DLLInst = NULL;
-        gGetKnownFolderPath = NULL;
+            GetProcAddress(hShell32DLLInst, "SHGetKnownFolderPath");
     }
 #endif
 }
 
 #if defined (XP_WIN)
 
-static nsresult GetKnownFolder(GUID* guid, nsILocalFile** aFile)
+static nsresult GetKnownFolder(GUID* guid, nsIFile** aFile)
 {
     if (!guid || !gGetKnownFolderPath)
         return NS_ERROR_FAILURE;
 
-    PWSTR path = NULL;
-    gGetKnownFolderPath(*guid, 0, NULL, &path);
+    PWSTR path = nullptr;
+    gGetKnownFolderPath(*guid, 0, nullptr, &path);
 
     if (!path)
         return NS_ERROR_FAILURE;
 
     nsresult rv = NS_NewLocalFile(nsDependentString(path),
-                                  PR_TRUE,
+                                  true,
                                   aFile);
 
     CoTaskMemFree(path);
     return rv;
 }
 
-//----------------------------------------------------------------------------------------
-static nsresult GetWindowsFolder(int folder, nsILocalFile** aFile)
-//----------------------------------------------------------------------------------------
+static nsresult
+GetWindowsFolder(int folder, nsIFile** aFile)
 {
     WCHAR path_orig[MAX_PATH + 3];
     WCHAR *path = path_orig+1;
-    HRESULT result = SHGetSpecialFolderPathW(NULL, path, folder, true);
+    HRESULT result = SHGetSpecialFolderPathW(nullptr, path, folder, true);
 
     if (!SUCCEEDED(result))
         return NS_ERROR_FAILURE;
@@ -171,7 +113,63 @@ static nsresult GetWindowsFolder(int folder, nsILocalFile** aFile)
         path[++len] = L'\0';
     }
 
-    return NS_NewLocalFile(nsDependentString(path, len), PR_TRUE, aFile);
+    return NS_NewLocalFile(nsDependentString(path, len), true, aFile);
+}
+
+__inline HRESULT
+SHLoadLibraryFromKnownFolder(REFKNOWNFOLDERID aFolderId, DWORD aMode,
+                             REFIID riid, void **ppv)
+{
+    *ppv = nullptr;
+    IShellLibrary *plib;
+    HRESULT hr = CoCreateInstance(CLSID_ShellLibrary, nullptr,
+                                  CLSCTX_INPROC_SERVER,
+                                  IID_PPV_ARGS(&plib));
+    if (SUCCEEDED(hr)) {
+        hr = plib->LoadLibraryFromKnownFolder(aFolderId, aMode);
+        if (SUCCEEDED(hr)) {
+            hr = plib->QueryInterface(riid, ppv);
+        }
+        plib->Release();
+    }
+    return hr;
+}
+
+/*
+ * Check to see if we're on Win7 and up, and if so, returns the default
+ * save-to location for the Windows Library passed in through aFolderId.
+ * Otherwise falls back on pre-win7 GetWindowsFolder.
+ */
+static nsresult
+GetLibrarySaveToPath(int aFallbackFolderId, REFKNOWNFOLDERID aFolderId,
+                     nsIFile** aFile)
+{
+    // Skip off checking for library support if the os is Vista or lower.
+    if (!IsWin7OrLater())
+      return GetWindowsFolder(aFallbackFolderId, aFile);
+
+    nsRefPtr<IShellLibrary> shellLib;
+    nsRefPtr<IShellItem> savePath;
+    HRESULT hr =
+        SHLoadLibraryFromKnownFolder(aFolderId, STGM_READ,
+                                     IID_IShellLibrary, getter_AddRefs(shellLib));
+
+    if (shellLib &&
+        SUCCEEDED(shellLib->GetDefaultSaveFolder(DSFT_DETECT, IID_IShellItem,
+                                                 getter_AddRefs(savePath)))) {
+        wchar_t* str = nullptr;
+        if (SUCCEEDED(savePath->GetDisplayName(SIGDN_FILESYSPATH, &str))) {
+            nsAutoString path;
+            path.Assign(str);
+            path.AppendLiteral("\\");
+            nsresult rv =
+                NS_NewLocalFile(path, false, aFile);
+            CoTaskMemFree(str);
+            return rv;
+        }
+    }
+
+    return GetWindowsFolder(aFallbackFolderId, aFile);
 }
 
 /**
@@ -179,7 +177,7 @@ static nsresult GetWindowsFolder(int folder, nsILocalFile** aFile)
  * querying the registry when the call to SHGetSpecialFolderPathW is unable to
  * provide these paths (Bug 513958).
  */
-static nsresult GetRegWindowsAppDataFolder(PRBool aLocal, nsILocalFile** aFile)
+static nsresult GetRegWindowsAppDataFolder(bool aLocal, nsIFile** aFile)
 {
     HKEY key;
     NS_NAMED_LITERAL_STRING(keyName,
@@ -191,8 +189,8 @@ static nsresult GetRegWindowsAppDataFolder(PRBool aLocal, nsILocalFile** aFile)
 
     WCHAR path[MAX_PATH + 2];
     DWORD type, size;
-    res = RegQueryValueExW(key, (aLocal ? L"Local AppData" : L"AppData"), NULL,
-                           &type, (LPBYTE)&path, &size);
+    res = RegQueryValueExW(key, (aLocal ? L"Local AppData" : L"AppData"),
+                           nullptr, &type, (LPBYTE)&path, &size);
     ::RegCloseKey(key);
     // The call to RegQueryValueExW must succeed, the type must be REG_SZ, the
     // buffer size must not equal 0, and the buffer size be a multiple of 2.
@@ -207,25 +205,25 @@ static nsresult GetRegWindowsAppDataFolder(PRBool aLocal, nsILocalFile** aFile)
         path[++len] = L'\0';
     }
 
-    return NS_NewLocalFile(nsDependentString(path, len), PR_TRUE, aFile);
+    return NS_NewLocalFile(nsDependentString(path, len), true, aFile);
 }
 
 #endif // XP_WIN
 
 #if defined(XP_UNIX)
 static nsresult
-GetUnixHomeDir(nsILocalFile** aFile)
+GetUnixHomeDir(nsIFile** aFile)
 {
 #ifdef VMS
     char *pHome;
     pHome = getenv("HOME");
     if (*pHome == '/') {
         return NS_NewNativeLocalFile(nsDependentCString(pHome),
-                                     PR_TRUE,
+                                     true,
                                      aFile);
     } else {
         return NS_NewNativeLocalFile(nsDependentCString(decc$translate_vms(pHome)),
-                                     PR_TRUE,
+                                     true,
                                      aFile);
     }
 #elif defined(ANDROID)
@@ -233,7 +231,7 @@ GetUnixHomeDir(nsILocalFile** aFile)
     return NS_ERROR_FAILURE;
 #else
     return NS_NewNativeLocalFile(nsDependentCString(PR_GetEnv("HOME")),
-                                 PR_TRUE, aFile);
+                                 true, aFile);
 #endif
 }
 
@@ -276,14 +274,14 @@ xdg_user_dir_lookup (const char *type)
 
   home_dir = getenv ("HOME");
 
-  if (home_dir == NULL)
+  if (home_dir == nullptr)
     goto error;
 
   config_home = getenv ("XDG_CONFIG_HOME");
-  if (config_home == NULL || config_home[0] == 0)
+  if (config_home == nullptr || config_home[0] == 0)
     {
       config_file = (char*) malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
-      if (config_file == NULL)
+      if (config_file == nullptr)
         goto error;
 
       strcpy (config_file, home_dir);
@@ -292,7 +290,7 @@ xdg_user_dir_lookup (const char *type)
   else
     {
       config_file = (char*) malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
-      if (config_file == NULL)
+      if (config_file == nullptr)
         goto error;
 
       strcpy (config_file, config_home);
@@ -301,10 +299,10 @@ xdg_user_dir_lookup (const char *type)
 
   file = fopen (config_file, "r");
   free (config_file);
-  if (file == NULL)
+  if (file == nullptr)
     goto error;
 
-  user_dir = NULL;
+  user_dir = nullptr;
   while (fgets (buffer, sizeof (buffer), file))
     {
       /* Remove newline at end */
@@ -352,7 +350,7 @@ xdg_user_dir_lookup (const char *type)
       if (relative)
 	{
 	  user_dir = (char*) malloc (strlen (home_dir) + 1 + strlen (p) + 1);
-          if (user_dir == NULL)
+          if (user_dir == nullptr)
             goto error2;
 
 	  strcpy (user_dir, home_dir);
@@ -361,7 +359,7 @@ xdg_user_dir_lookup (const char *type)
       else
 	{
 	  user_dir = (char*) malloc (strlen (p) + 1);
-          if (user_dir == NULL)
+          if (user_dir == nullptr)
             goto error2;
 
 	  *user_dir = 0;
@@ -383,7 +381,7 @@ error2:
     return user_dir;
 
  error:
-  return NULL;
+  return nullptr;
 }
 
 static const char xdg_user_dirs[] =
@@ -396,7 +394,7 @@ static const char xdg_user_dirs[] =
     "TEMPLATES\0"
     "VIDEOS";
 
-static const PRUint8 xdg_user_dir_offsets[] = {
+static const uint8_t xdg_user_dir_offsets[] = {
     0,
     8,
     18,
@@ -409,16 +407,16 @@ static const PRUint8 xdg_user_dir_offsets[] = {
 
 static nsresult
 GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
-                        nsILocalFile** aFile)
+                        nsIFile** aFile)
 {
     char *dir = xdg_user_dir_lookup
                     (xdg_user_dirs + xdg_user_dir_offsets[aSystemDirectory -
                                                          Unix_XDG_Desktop]);
 
     nsresult rv;
-    nsCOMPtr<nsILocalFile> file;
+    nsCOMPtr<nsIFile> file;
     if (dir) {
-        rv = NS_NewNativeLocalFile(nsDependentCString(dir), PR_TRUE,
+        rv = NS_NewNativeLocalFile(nsDependentCString(dir), true,
                                    getter_AddRefs(file));
         free(dir);
     } else if (Unix_XDG_Desktop == aSystemDirectory) {
@@ -430,22 +428,6 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
 
         rv = file->AppendNative(NS_LITERAL_CSTRING("Desktop"));
     }
-#if defined(MOZ_PLATFORM_MAEMO)
-    // "MYDOCSDIR" is exported to point to "/home/user/MyDocs" in maemo.
-    else if (Unix_XDG_Documents == aSystemDirectory) {
-
-        char *myDocs = PR_GetEnv("MYDOCSDIR");
-        if (!myDocs || !*myDocs)
-            return NS_ERROR_FAILURE;
-
-        rv = NS_NewNativeLocalFile(nsDependentCString(myDocs), PR_TRUE,
-                                   getter_AddRefs(file));
-        if (NS_FAILED(rv))
-            return rv;
-
-        rv = file->AppendNative(NS_LITERAL_CSTRING(".documents"));
-    }
-#endif
     else {
       // no fallback for the other XDG dirs
       rv = NS_ERROR_FAILURE;
@@ -454,7 +436,7 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
     if (NS_FAILED(rv))
         return rv;
 
-    PRBool exists;
+    bool exists;
     rv = file->Exists(&exists);
     if (NS_FAILED(rv))
         return rv;
@@ -464,7 +446,7 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
             return rv;
     }
 
-    *aFile = nsnull;
+    *aFile = nullptr;
     file.swap(*aFile);
 
     return NS_OK;
@@ -473,7 +455,7 @@ GetUnixXDGUserDirectory(SystemDirectories aSystemDirectory,
 
 nsresult
 GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
-                          nsILocalFile** aFile)
+                          nsIFile** aFile)
 {
 #if defined(XP_WIN)
     WCHAR path[MAX_PATH];
@@ -488,11 +470,8 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             if (!_wgetcwd(path, MAX_PATH))
                 return NS_ERROR_FAILURE;
             return NS_NewLocalFile(nsDependentString(path),
-                                   PR_TRUE,
+                                   true,
                                    aFile);
-#elif defined(XP_OS2)
-            if (DosQueryPathInfo( ".", FIL_QUERYFULLNAME, path, MAXPATHLEN))
-                return NS_ERROR_FAILURE;
 #else
             if(!getcwd(path, MAXPATHLEN))
                 return NS_ERROR_FAILURE;
@@ -500,38 +479,26 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
 
 #if !defined(XP_WIN)
             return NS_NewNativeLocalFile(nsDependentCString(path),
-                                         PR_TRUE,
+                                         true,
                                          aFile);
 #endif
 
         case OS_DriveDirectory:
 #if defined (XP_WIN)
         {
-            PRInt32 len = ::GetWindowsDirectoryW(path, MAX_PATH);
+            int32_t len = ::GetWindowsDirectoryW(path, MAX_PATH);
             if (len == 0)
                 break;
-            if (path[1] == PRUnichar(':') && path[2] == PRUnichar('\\'))
+            if (path[1] == char16_t(':') && path[2] == char16_t('\\'))
                 path[3] = 0;
 
             return NS_NewLocalFile(nsDependentString(path),
-                                   PR_TRUE,
+                                   true,
                                    aFile);
         }
-#elif defined(XP_OS2)
-        {
-            ULONG ulBootDrive = 0;
-            char  buffer[] = " :\\OS2\\";
-            DosQuerySysInfo( QSV_BOOT_DRIVE, QSV_BOOT_DRIVE,
-                             &ulBootDrive, sizeof ulBootDrive);
-            buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
-
-            return NS_NewNativeLocalFile(nsDependentCString(buffer),
-                                         PR_TRUE,
-                                         aFile);
-        }
 #else
         return NS_NewNativeLocalFile(nsDependentCString("/"),
-                                     PR_TRUE,
+                                     true,
                                      aFile);
 
 #endif
@@ -543,34 +510,17 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             if (len == 0)
                 break;
             return NS_NewLocalFile(nsDependentString(path, len),
-                                   PR_TRUE,
+                                   true,
                                    aFile);
         }
-#elif defined(XP_OS2)
-        {
-            char *tPath = PR_GetEnv("TMP");
-            if (!tPath || !*tPath) {
-                tPath = PR_GetEnv("TEMP");
-                if (!tPath || !*tPath) {
-                    // if an OS/2 system has neither TMP nor TEMP defined
-                    // then it is severely broken, so this will never happen.
-                    return NS_ERROR_UNEXPECTED;
-                }
-            }
-            nsCString tString = nsDependentCString(tPath);
-            if (tString.Find("/", PR_FALSE, 0, -1)) {
-                tString.ReplaceChar('/','\\');
-            }
-            return NS_NewNativeLocalFile(tString, PR_TRUE, aFile);
-        }
-#elif defined(XP_MACOSX)
+#elif defined(MOZ_WIDGET_COCOA)
         {
             return GetOSXFolderType(kUserDomain, kTemporaryFolderType, aFile);
         }
 
 #elif defined(XP_UNIX)
         {
-            static const char *tPath = nsnull;
+            static const char *tPath = nullptr;
             if (!tPath) {
                 tPath = PR_GetEnv("TMPDIR");
                 if (!tPath || !*tPath) {
@@ -584,7 +534,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 }
             }
             return NS_NewNativeLocalFile(nsDependentCString(tPath),
-                                         PR_TRUE,
+                                         true,
                                          aFile);
         }
 #else
@@ -593,7 +543,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
 #if defined (XP_WIN)
         case Win_SystemDirectory:
         {
-            PRInt32 len = ::GetSystemDirectoryW(path, MAX_PATH);
+            int32_t len = ::GetSystemDirectoryW(path, MAX_PATH);
 
             // Need enough space to add the trailing backslash
             if (!len || len > MAX_PATH - 2)
@@ -602,13 +552,13 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             path[++len] = L'\0';
 
             return NS_NewLocalFile(nsDependentString(path, len),
-                                   PR_TRUE,
+                                   true,
                                    aFile);
         }
 
         case Win_WindowsDirectory:
         {
-            PRInt32 len = ::GetWindowsDirectoryW(path, MAX_PATH);
+            int32_t len = ::GetWindowsDirectoryW(path, MAX_PATH);
 
             // Need enough space to add the trailing backslash
             if (!len || len > MAX_PATH - 2)
@@ -618,7 +568,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             path[++len] = L'\0';
 
             return NS_NewLocalFile(nsDependentString(path, len),
-                                   PR_TRUE,
+                                   true,
                                    aFile);
         }
 
@@ -633,7 +583,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             if (NS_SUCCEEDED(rv))
                 return rv;
 
-            PRInt32 len;
+            int32_t len;
             if ((len = ::GetEnvironmentVariableW(L"HOME", path, MAX_PATH)) > 0)
             {
                 // Need enough space to add the trailing backslash
@@ -644,7 +594,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 path[++len] = L'\0';
 
                 rv = NS_NewLocalFile(nsDependentString(path, len),
-                                     PR_TRUE,
+                                     true,
                                      aFile);
                 if (NS_SUCCEEDED(rv))
                     return rv;
@@ -668,7 +618,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
                 path[++len] = L'\0';
 
                 return NS_NewLocalFile(nsDependentString(path, len),
-                                       PR_TRUE,
+                                       true,
                                        aFile);
             }
         }
@@ -687,7 +637,7 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             GUID folderid_downloads = {0x374de290, 0x123f, 0x4565, {0x91, 0x64,
                                        0x39, 0xc4, 0x92, 0x5e, 0x46, 0x7b}};
             nsresult rv = GetKnownFolder(&folderid_downloads, aFile);
-            // On WinXP and 2k, there is no downloads folder, default
+            // On WinXP, there is no downloads folder, default
             // to 'Desktop'.
             if(NS_ERROR_FAILURE == rv)
             {
@@ -772,6 +722,10 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
         {
             return GetWindowsFolder(CSIDL_COMMON_DESKTOPDIRECTORY, aFile);
         }
+        case Win_Common_AppData:
+        {
+            return GetWindowsFolder(CSIDL_COMMON_APPDATA, aFile);
+        }
         case Win_Printhood:
         {
             return GetWindowsFolder(CSIDL_PRINTHOOD, aFile);
@@ -784,26 +738,50 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
         {
             nsresult rv = GetWindowsFolder(CSIDL_APPDATA, aFile);
             if (NS_FAILED(rv))
-                rv = GetRegWindowsAppDataFolder(PR_FALSE, aFile);
+                rv = GetRegWindowsAppDataFolder(false, aFile);
             return rv;
         }
         case Win_LocalAppdata:
         {
             nsresult rv = GetWindowsFolder(CSIDL_LOCAL_APPDATA, aFile);
             if (NS_FAILED(rv))
-                rv = GetRegWindowsAppDataFolder(PR_TRUE, aFile);
+                rv = GetRegWindowsAppDataFolder(true, aFile);
             return rv;
         }
+        case Win_Documents:
+        {
+            return GetLibrarySaveToPath(CSIDL_MYDOCUMENTS,
+                                        FOLDERID_DocumentsLibrary,
+                                        aFile);
+        }
+        case Win_Pictures:
+        {
+            return GetLibrarySaveToPath(CSIDL_MYPICTURES,
+                                        FOLDERID_PicturesLibrary,
+                                        aFile);
+        }
+        case Win_Music:
+        {
+            return GetLibrarySaveToPath(CSIDL_MYMUSIC,
+                                        FOLDERID_MusicLibrary,
+                                        aFile);
+        }
+        case Win_Videos:
+        {
+            return GetLibrarySaveToPath(CSIDL_MYVIDEO,
+                                        FOLDERID_VideosLibrary,
+                                        aFile);
+        }
 #endif  // XP_WIN
 
 #if defined(XP_UNIX)
         case Unix_LocalDirectory:
             return NS_NewNativeLocalFile(nsDependentCString("/usr/local/netscape/"),
-                                         PR_TRUE,
+                                         true,
                                          aFile);
         case Unix_LibDirectory:
             return NS_NewNativeLocalFile(nsDependentCString("/usr/local/lib/netscape/"),
-                                         PR_TRUE,
+                                         true,
                                          aFile);
 
         case Unix_HomeDirectory:
@@ -820,86 +798,15 @@ GetSpecialSystemDirectory(SystemDirectories aSystemSystemDirectory,
             return GetUnixXDGUserDirectory(aSystemSystemDirectory, aFile);
 #endif
 
-#ifdef XP_OS2
-        case OS2_SystemDirectory:
-        {
-            ULONG ulBootDrive = 0;
-            char  buffer[] = " :\\OS2\\System\\";
-            DosQuerySysInfo( QSV_BOOT_DRIVE, QSV_BOOT_DRIVE,
-                             &ulBootDrive, sizeof ulBootDrive);
-            buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
-
-            return NS_NewNativeLocalFile(nsDependentCString(buffer),
-                                         PR_TRUE,
-                                         aFile);
-        }
-
-     case OS2_OS2Directory:
-        {
-            ULONG ulBootDrive = 0;
-            char  buffer[] = " :\\OS2\\";
-            DosQuerySysInfo( QSV_BOOT_DRIVE, QSV_BOOT_DRIVE,
-                             &ulBootDrive, sizeof ulBootDrive);
-            buffer[0] = 'A' - 1 + ulBootDrive; // duh, 1-based index...
-
-            return NS_NewNativeLocalFile(nsDependentCString(buffer),
-                                         PR_TRUE,
-                                         aFile);
-        }
-
-     case OS2_HomeDirectory:
-        {
-            nsresult rv;
-            char *tPath = PR_GetEnv("MOZILLA_HOME");
-            char buffer[CCHMAXPATH];
-            /* If MOZILLA_HOME is not set, use GetCurrentProcessDirectory */
-            /* To ensure we get a long filename system */
-            if (!tPath || !*tPath) {
-                PPIB ppib;
-                PTIB ptib;
-                DosGetInfoBlocks( &ptib, &ppib);
-                DosQueryModuleName( ppib->pib_hmte, CCHMAXPATH, buffer);
-                *strrchr( buffer, '\\') = '\0'; // XXX DBCS misery
-                tPath = buffer;
-            }
-            rv = NS_NewNativeLocalFile(nsDependentCString(tPath),
-                                       PR_TRUE,
-                                       aFile);
-
-            PrfWriteProfileString(HINI_USERPROFILE, "Mozilla", "Home", tPath);
-            return rv;
-        }
-
-        case OS2_DesktopDirectory:
-        {
-            char szPath[CCHMAXPATH + 1];
-            BOOL fSuccess;
-            fSuccess = WinQueryActiveDesktopPathname (szPath, sizeof(szPath));
-            if (!fSuccess) {
-                // this could happen if we are running without the WPS, return
-                // the Home directory instead
-                return GetSpecialSystemDirectory(OS2_HomeDirectory, aFile);
-            }
-            int len = strlen (szPath);
-            if (len > CCHMAXPATH -1)
-                break;
-            szPath[len] = '\\';
-            szPath[len + 1] = '\0';
-
-            return NS_NewNativeLocalFile(nsDependentCString(szPath),
-                                         PR_TRUE,
-                                         aFile);
-        }
-#endif
         default:
             break;
     }
     return NS_ERROR_NOT_AVAILABLE;
 }
 
-#if defined (XP_MACOSX)
+#if defined (MOZ_WIDGET_COCOA)
 nsresult
-GetOSXFolderType(short aDomain, OSType aFolderType, nsILocalFile **localFile)
+GetOSXFolderType(short aDomain, OSType aFolderType, nsIFile **localFile)
 {
     OSErr err;
     FSRef fsRef;
@@ -908,7 +815,7 @@ GetOSXFolderType(short aDomain, OSType aFolderType, nsILocalFile **localFile)
     err = ::FSFindFolder(aDomain, aFolderType, kCreateFolder, &fsRef);
     if (err == noErr)
     {
-        NS_NewLocalFile(EmptyString(), PR_TRUE, localFile);
+        NS_NewLocalFile(EmptyString(), true, localFile);
         nsCOMPtr<nsILocalFileMac> localMacFile(do_QueryInterface(*localFile));
         if (localMacFile)
             rv = localMacFile->InitWithFSRef(&fsRef);
