@@ -10,9 +10,8 @@ function checkOS(os) {
 }
 
 const isWin = checkOS("Win");
-const isOS2 = checkOS("OS2");
 const isMac = checkOS("Mac");
-const isUnix = !(isWin || isOS2 || isMac);
+const isUnix = !(isWin || isMac);
 
 var hiddenUnixFile;
 function createUNIXHiddenFile() {
@@ -29,7 +28,7 @@ function createUNIXHiddenFile() {
 
 function run_test() {
   // Skip this test on Windows
-  if (isWin || isOS2)
+  if (isWin)
     return;
 
   do_check_true(createUNIXHiddenFile());
